(define (load-file emacs text file)
   (emacs-message emacs "loading " file "...")
   (text-delete text "1.1" 'end)
   (let ((port (open-input-file file)))
      (if (not (input-port? port))
	  (emacs-message emacs "Can't open file for input " file ".")
	  (begin
	     (emacs-modeline-file-set! emacs file)
	     (emacs-modeline-mode-set! emacs (suffix file))
	     (focus text)
	     (let loop ((line (read-line port)))
		(if (not (eof-object? line))
		    (begin
		       (text-insert text line 'end)
		       (text-insert text #"\n" 'end) 
		       (loop (read-line port)))
		    (text-see-pos *text* 'end)))))))

(define request-file
   (let ((default ""))
      (lambda () (emacs-read-string
		  *emacs*
		  "Find file"
		  (lambda (file)
		     (set! default file)
		     (load-file *emacs* *text* file))
		  default))))

(define (end-of-buffer)
   (text-see-pos *text* 'end))

(define (beginning-of-buffer)
   (text-see-pos *text* "1.1"))

(define (request-eval)
   (emacs-read-string
    *emacs*
    "Eval"
    (lambda (str)
       (let* ((p (open-input-string str))
	      (o (read p)))
	  (close-input-port p)
	  (let ((value (eval o)))
	     (emacs-message *emacs* value)
	     (focus *text*))))))

(define *emacs*
   (instantiate::emacs
      (background "oldlace")
      (toolbar (list 5
		     (list "emacs/close.xpm"
			   "Close profile frame"
			   (lambda () (biglook-exit 0)))
		     5
		     (list "emacs/open.xpm"
			   "Open profile"
			   request-file)
		     -5
		     (list "emacs/help.xpm"
			   "Help"
			   (lambda () (print "help")))))
      (menu `(#f
	      ("File"
	       ("Open"  ,request-file "C-x C-f")
	       ("")
	       ("Exit BEmacs" ,(lambda () (biglook-exit 0)) "C-x C-c"))
	      -5
	      ("Help"
	       ("About BEmacs" ,(lambda () (print "So what..."))))))
      (width 582)
      (height 400)
      (modeline-file "*scratch*")
      (modeline-mode "Fundamental")))

(define *text*
   (instantiate::text
      (parent *emacs*)
      (border-width 0)
      (highlight-thickness 0)
      (relief "flat")
      (background (emacs-background *emacs*))))

;; we remove the toplevel window
(withdraw (top-level-root))
;; we pack the text inside the emacs top-level
(pack *text* :expand #t :fill 'both)
;; a welcome message
(emacs-message *emacs* "Welcome BEmacs...")
;; key bindings
(emacs-set-key! *emacs* "<Control-g>"
		(lambda (e) (emacs-message e "Quit")))
(emacs-set-key! *emacs* "<Control-x><Control-c>"
		(lambda (e) (biglook-exit 0)))
(emacs-set-key! *emacs* "<Control-x><Control-f>" (lambda (e) (request-file)))
(emacs-set-key! *emacs* "<Escape>:" (lambda (e) (request-eval)))

(define-macro (P x) ; A macro for printing traces (for this demo)
  `(lambda () (display ,x) (newline)))


