;; compute a pseudo-random speed
(define get-random-speed
   (let ((l (list 14 45 95 2 23 54 39 84 11 88 78 45 56 23 69 83)))
      (set-cdr! (last-pair l) l)
      (lambda ()
	 (set! l (cdr l))
	 (car l))))

;; the runner images
(define *runner-images*
   (vector (instantiate::image (file "runner1.xpm"))
	   (instantiate::image (file "runner2.xpm"))
	   (instantiate::image (file "runner3.xpm"))
	   (instantiate::image (file "runner4.xpm"))
	   (instantiate::image (file "runner5.xpm"))
	   (instantiate::image (file "runner6.xpm"))))

;; get the next image
(define (get-image-list)
   (let ((l (vector->list *runner-images*)))
      (set-cdr! (last-pair l) l)
      l))

;; the running play ground
(define *ground*
   (instantiate::canvas
      (height 30)
      (width  150)
      (background "white")))

(define *pace*
   (instantiate::scale
      (text        "Pace")
      (font        '(Helvetica -10 bold))
      (value       25)
      (from        0)
      (to          50)
      (orientation 'horizontal)))

;; A single line for a nicer ground
(instantiate::line
   (parent *ground*)
   (fill "red")
   (init-coords `(10 20 ,(- (canvas-width *ground*) 10) 20)))

;; Button for starting new runners
(define *start*
   (instantiate::button
      (text    "Start new runner")
      (font    '(Helvetica -10 bold))
      (command (let ((num 1))
		  (lambda ()
		     (let ((new (instantiate::runner
				   (init-coords '(10 15))
				   (parent *ground*))))
			(print ">>> runner " (runner-number new) " started")
			((make-runner-run new))))))))

;; Create a new runner
(define make-runner-run
   (lambda (runner)
      (define (run)
	 (with-access::runner runner (coords image images speed number)
	    (match-case coords
	       ((?x ?y)
		(if (< x (- (canvas-width *ground*) 13))
		    (begin
		       (set! coords (list (+ 1 x) y))
		       (set! image (car images))
		       (set! images (cdr images))
		       (after (+ speed (scale-value *pace*))
			      (lambda ()
				 (run))))
		    (begin
		       (print "<<< runner " number " arrived")
		       (destroy-canvas-item! runner))))
	       (else
		(error "run" "runner lost" coords)))))
      run))

;; We start the all things
(pack *ground*)
(pack *pace* *start* :expand #t :fill 'x)

	  
