/*---------------------------------------------------------------------*/
/*   This program is free software; you can redistribute it            */
/*   and/or modify it under the terms of the GNU General Public        */
/*   License as published by the Free Software Foundation; either      */
/*   version 2 of the License, or (at your option) any later version.  */
/*                                                                     */
/*   This program is distributed in the hope that it will be useful,   */
/*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
/*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
/*   GNU General Public License for more details.                      */
/*                                                                     */
/*   You should have received a copy of the GNU General Public         */
/*   License along with this program; if not, write to the Free        */
/*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
/*   MA 02111-1307, USA.                                               */
/*---------------------------------------------------------------------*/
/*=====================================================================*/
/*    serrano/prgm/project/bglk/runtime/Clib/tclglue.c                 */
/*    -------------------------------------------------------------    */
/*    Author      :  Manuel Serrano                                    */
/*    Creation    :  Fri Oct  9 10:41:37 1998                          */
/*    Last change :  Fri May 14 16:00:33 1999 (serrano)                */
/*    -------------------------------------------------------------    */
/*    The connection to Tcl                                            */
/*=====================================================================*/
#include <bigloo.h>
#include <tclInt.h>
#include <tclglue.h>
#include <pwd.h>
#include <sys/types.h>

#undef assert
#undef NDEBUG
#include <assert.h>

/*---------------------------------------------------------------------*/
/*    char *                                                           */
/*    SCM_string_undotify ...                                          */
/*---------------------------------------------------------------------*/
char *
SCM_string_undotify( char *string ) {
   int  len  = strlen( string );
   int  last = len - 1;
   char prev = ' ';
   int  i = 0;
   
   while( i < len ) {
      char cur;

      cur = string[ i ];

      if( cur == '.' ) {
	 if( i == last ) {
	    if( prev == ' ' )
	       string[ i ] = '!';
	 } else {
	    char next = string[ i + 1 ];
	    
	    if( ( next == ' ' ) && (prev == ' ') ) 
	       string[ i ] = '!';
	    
	    i+=2;
	    prev = next;
	 }
      } else {
	 prev = cur;
	 i++;
      }
   }

   return string;
}
      
   
/*---------------------------------------------------------------------*/
/*    static obj_t                                                     */
/*    SCM_last_Tk_result ...                                           */
/*    -------------------------------------------------------------    */
/*    Last result of Tcl_GlobalEval (as a obj_t object rather than     */
/*    a string)                                                        */
/*---------------------------------------------------------------------*/
static obj_t SCM_last_Tk_result;

/*---------------------------------------------------------------------*/
/*    obj_t                                                            */
/*    SCM_last_Tk_as_scheme ...                                        */
/*---------------------------------------------------------------------*/
obj_t
SCM_last_Tk_as_scheme( void ) {
  return SCM_last_Tk_result;
}

/*---------------------------------------------------------------------*/
/*    char *                                                           */
/*    SCM_stringify ...                                                */
/*---------------------------------------------------------------------*/
char *
SCM_stringify( char *s, int free_original )
{
   char *res, *d;
  
   if( s == NULL ) s = "";
   res = d = (char *)GC_MALLOC( 2 * strlen(s) + 3 ); /* worst overestimation */
  
   for( *d++ = '"'; *s; s++, d++ ) {
      if( *s == '"' || *s == '\\' ) *d++ = '\\';
      *d = *s;
   }
   *d++ = '"';
   *d   = '\0';
   
   return res;
}

/*---------------------------------------------------------------------*/
/*    Tcl_Interp *                                                     */
/*    Tcl_CreateInterp ...                                             */
/*---------------------------------------------------------------------*/
Tcl_Interp *Tcl_CreateInterp()
{
   Interp *iPtr = (Interp *)GC_MALLOC(sizeof(Interp));
   int i;

   iPtr->result		  = iPtr->resultSpace;
   iPtr->freeProc	  = 0;
   iPtr->errorLine	  = 0;
   iPtr->resultSpace[0]   = 0;

   for( i = 0; i < NUM_REGEXPS; i++ ) {
      iPtr->patterns[i]   = NULL;
      iPtr->patLengths[i] = -1;
      iPtr->regexps[i]    = NULL;
   }
  
   iPtr->appendResult	  = NULL;
   iPtr->appendAvl	  = 0;
   iPtr->appendUsed	  = 0;

   /* strcpy(iPtr->pdFormat, "%g"); No more needed for 8.0 */

   iPtr->assocData 	  = (Tcl_HashTable *) NULL;

   /* See Tcl_CreateCommand for this table utility  */
   iPtr->globalNsPtr      = (Namespace*) GC_MALLOC( sizeof( Namespace ) );
   Tcl_InitHashTable( &iPtr->globalNsPtr->cmdTable, TCL_STRING_KEYS );

   /* Protect the Tcl_Obj result */
   Tcl_ResetObjResult( iPtr );

   return (Tcl_Interp *) iPtr;
}
   
/*---------------------------------------------------------------------*/
/*    int                                                              */
/*    SCM_internal_Tcl_DeleteCommand ...                               */
/*---------------------------------------------------------------------*/
int
SCM_internal_Tcl_DeleteCommand( Tcl_Interp *interp,char *cmdName ) {
   struct Tk_command *W;
   Interp *iPtr = (Interp *) interp;
   Tcl_HashEntry *hPtr;
  
   /* the internal DeleteCommand do all the job (except setting cmdName to
    * UNBOUND, because this implies to use intern which can allocate a cell)
    * Consequently, GC calls this function, whereas Tk call the true 
    * DeleteCommand
    */
   hPtr = Tcl_FindHashEntry( &iPtr->globalNsPtr->cmdTable, cmdName );
   if( hPtr == NULL ) return -1;

   W = SCM_TO_TK_COMMAND( (obj_t)Tcl_GetHashValue(hPtr) );
   if( W->deleted ) return -1;

   /* Execute the delete handler */
   if( W->delproc != NULL ) (*W->delproc)( W->deldata );

   /* Note: W will be freed by the GC */
   Tcl_DeleteHashEntry( hPtr );

   /* Remember that the command has been deleted */
   W->deleted = 1;

   return 0;
}

/*---------------------------------------------------------------------*/
/*    int                                                              */
/*    Tcl_DeleteCommand ...                                            */
/*---------------------------------------------------------------------*/
int
Tcl_DeleteCommand( Tcl_Interp *interp, char *cmdName ) {
   int result;
   
   if( result = SCM_internal_Tcl_DeleteCommand( interp, cmdName ) )
      return result;
  
   return 0;
}

/*
 *----------------------------------------------------------------------
 *
 * Tcl_HideCommand --
 *
 *	Makes a command hidden so that it cannot be invoked from within
 *	an interpreter, only from within an ancestor.
 *
 *----------------------------------------------------------------------
 */

int
Tcl_HideCommand(interp, cmdName, hiddenCmdName)
    Tcl_Interp *interp;		/* Interpreter in which to hide command. */
    char *cmdName;		/* Name of hidden command. */
    char *hiddenCmdName;	/* Name of to-be-hidden command. */
{
  /* Easy job */
  return TCL_OK;
}

/*
 *----------------------------------------------------------------------
 *
 * Tcl_InterpDeleted --
 *
 *	Returns nonzero if the interpreter has been deleted with a call
 *	to Tcl_DeleteInterp.
 *
 * Results:
 *	Nonzero if the interpreter is deleted, zero otherwise.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */
int Tcl_InterpDeleted(Tcl_Interp *interp)
{
    return (((Interp *) interp)->flags & DELETED) ? 1 : 0;
}

/*
 *----------------------------------------------------------------------
 *
 * Tcl_DeleteInterp --
 *
 *	Delete an interpreter and free up all of the resources associated
 *	with it.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	The interpreter is destroyed.  The caller should never again
 *	use the interp token.
 *
 *----------------------------------------------------------------------
 */
void Tcl_DeleteInterp(Tcl_Interp *interp)
{
  Interp *iPtr = (Interp *) interp;
  Tcl_HashEntry *hPtr;
  Tcl_HashSearch search;
  struct Tk_command *W;

  if (!iPtr || (iPtr->flags & DELETED)) return;

  /* Mark the interpreter as deleted. No further evals will be allowed. */
  iPtr->flags |= DELETED;
  
  /* Delete result space */
  if (iPtr->appendResult != NULL) {
    GC_free(iPtr->appendResult);
  }
  
  /* delete hash table of Tk commands (see Tcl_CreateCommand) */
  for (hPtr = Tcl_FirstHashEntry(&iPtr->globalNsPtr->cmdTable, &search);
       hPtr != NULL; 
       hPtr = Tcl_NextHashEntry(&search)) {
    W = SCM_TO_TK_COMMAND( (obj_t) Tcl_GetHashValue(hPtr) );
    Tcl_DeleteCommand(interp, W->Id);
  }
  Tcl_DeleteHashTable(&iPtr->globalNsPtr->cmdTable);

  GC_free((char *) iPtr);
}

/* 
 * Tcl_DeleteCommandFromToken --
 *
 *	Removes the given command from the given interpreter. This procedure
 *	resembles Tcl_DeleteCommand, but takes a Tcl_Command token instead
 *	of a command name for efficiency.
 */

int
Tcl_DeleteCommandFromToken(interp, cmd)
    Tcl_Interp *interp;		/* Token for command interpreter returned by
				 * a previous call to Tcl_CreateInterp. */
    Tcl_Command cmd;            /* Token for command to delete. */
{
  /* In SCM this function is less efficient than the Tcl one since it 
   * searches the name of the given command and call Tcl_DeleteCommand
   * However, the time penalty should be lower than in Tcl here.
   */
  return Tcl_DeleteCommand(interp, Tcl_GetCommandName(interp, cmd));
}


/*
 *----------------------------------------------------------------------
 *
 * Tcl_CreateCommand --
 *
 *	Define a new command in a command table.
 *
 * Results:
 *	The return value is a token for the command, which can
 *	be used in future calls to Tcl_NameOfCommand.
 *
 * Side effects:
 *	If a command named cmdName already exists for interp, it is
 *	deleted.
 *----------------------------------------------------------------------
 */
Tcl_Command
Tcl_CreateCommand( Tcl_Interp *interp,
                   /* Token for command interpreter (returned
		    * by a previous call to Tcl_CreateInterp). */
		   char *cmdName,
                   /* Name of command. */
		   Tcl_CmdProc *proc,
                   /* Command procedure to associate with cmdName. */
		   ClientData clientData,
                   /* Arbitrary one-word value to pass to proc. */
		   Tcl_CmdDeleteProc *deleteProc )
		   /* If not NULL, gives a procedure to call when
		    * this command is deleted. */
{
  obj_t z;
  struct Tk_command *W;
  Interp *iPtr = (Interp *) interp;
  Tcl_HashEntry *hPtr;
  int new;

  /* 
   * There are two things to do here.  
   * 	- We must create a new variable and associate the newly
   * 	  created Tk object to it. This is the Scheme part.
   *    - We must  store the cmd in a hashtable contained in the interp data
   *      structure. This is necessary to delete all the command associated
   *      to the interpreter. This is important at least for the "send"
   *      command which un-register the interpreter from the X server.
   * 
   */
 
  /* Initialize the Tk_command structure */
  z = make_va_procedure( tcl_call_command,
			 -1,
 			 sizeof( struct Tk_command ) + strlen( cmdName ) );
  W = SCM_TO_TK_COMMAND( z );
  W->ptr        = clientData;
  W->marker     = z;
  W->fct 	= proc;
  W->delproc    = deleteProc;
  W->deldata	= clientData;
  W->deleted	= 0;
  W->objproc	= 0;
  W->object     = BUNSPEC;
  strcpy(W->Id, cmdName);

  /* Register the command in the Tcl command hash table */
  hPtr = Tcl_CreateHashEntry(&iPtr->globalNsPtr->cmdTable, cmdName, &new);
  if (!new) {
    /* Command already exists: delete the old one */
    Tcl_DeleteCommand(interp, cmdName); /* not efficient but safer */
  }
/*   printf( "Je plug: %8x [%s] W:%8x\n", z, cmdName, W );             */
/*   dprint( z );                                                      */
  Tcl_SetHashValue(hPtr, z);

  return (Tcl_Command) W;
}

/*
 *----------------------------------------------------------------------
 *
 * Tcl_SetAssocData --
 *
 *	Creates a named association between user-specified data, a delete
 *	function and this interpreter. If the association already exists
 *	the data is overwritten with the new data. The delete function will
 *	be invoked when the interpreter is deleted.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	Sets the associated data, creates the association if needed.
 *
 *----------------------------------------------------------------------
 */
void
Tcl_SetAssocData(interp, name, proc, clientData)
    Tcl_Interp *interp;		/* Interpreter to associate with. */
    char *name;			/* Name for association. */
    Tcl_InterpDeleteProc *proc;	/* Proc to call when interpreter is
                                 * about to be deleted. */
    ClientData clientData;	/* One-word value to pass to proc. */
{
    Interp *iPtr = (Interp *) interp;
    AssocData *dPtr;
    Tcl_HashEntry *hPtr;
    int new;

    if (iPtr->assocData == (Tcl_HashTable *) NULL) {
	if (iPtr->flags & DELETED) {
	    /*
	     * Don't create new entries after interpreter deletion
	     * has started;  it isn't even safe to muck with the
	     * interpreter right now
	     */

	    return;
	}
        iPtr->assocData = (Tcl_HashTable *) GC_MALLOC(sizeof(Tcl_HashTable));
        Tcl_InitHashTable(iPtr->assocData, TCL_STRING_KEYS);
    }
    hPtr = Tcl_CreateHashEntry(iPtr->assocData, name, &new);
    if (new == 0) {
        dPtr = (AssocData *) Tcl_GetHashValue(hPtr);
    } else {
        dPtr = (AssocData *) GC_MALLOC(sizeof(AssocData));
    }
    dPtr->proc = proc;
    dPtr->clientData = clientData;

    Tcl_SetHashValue(hPtr, dPtr);
}

/*
 *----------------------------------------------------------------------
 *
 * Tcl_DeleteAssocData --
 *
 *	Deletes a named association of user-specified data with
 *	the specified interpreter.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	Deletes the association.
 *
 *----------------------------------------------------------------------
 */
void
Tcl_DeleteAssocData(interp, name)
    Tcl_Interp *interp;			/* Interpreter to associate with. */
    char *name;				/* Name of association. */
{
    Interp *iPtr = (Interp *) interp;
    AssocData *dPtr;
    Tcl_HashEntry *hPtr;

    if (iPtr->assocData == (Tcl_HashTable *) NULL) {
        return;
    }
    hPtr = Tcl_FindHashEntry(iPtr->assocData, name);
    if (hPtr == (Tcl_HashEntry *) NULL) {
        return;
    }
    dPtr = (AssocData *) Tcl_GetHashValue(hPtr);
    GC_free((char *) dPtr);
    Tcl_DeleteHashEntry(hPtr);
}

/*
 *----------------------------------------------------------------------
 *
 * Tcl_GetAssocData --
 *
 *	Returns the client data associated with this name in the
 *	specified interpreter.
 *
 * Results:
 *	The client data in the AssocData record denoted by the named
 *	association, or NULL.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */
ClientData
Tcl_GetAssocData(interp, name, procPtr)
    Tcl_Interp *interp;			/* Interpreter associated with. */
    char *name;				/* Name of association. */
    Tcl_InterpDeleteProc **procPtr;	/* Pointer to place to store address
					 * of current deletion callback. */
{
    Interp *iPtr = (Interp *) interp;
    AssocData *dPtr;
    Tcl_HashEntry *hPtr;

    if (iPtr->assocData == (Tcl_HashTable *) NULL) {
        return (ClientData) NULL;
    }
    hPtr = Tcl_FindHashEntry(iPtr->assocData, name);
    if (hPtr == (Tcl_HashEntry *) NULL) {
        return (ClientData) NULL;
    }
    dPtr = (AssocData *) Tcl_GetHashValue(hPtr);
    if (procPtr != (Tcl_InterpDeleteProc **) NULL) {
        *procPtr = dPtr->proc;
    }
    return dPtr->clientData;
}

/*
 *----------------------------------------------------------------------
 *
 * Tcl_Backslash --
 *
 *	Figure out how to handle a backslash sequence.
 *
 * Results:
 *	The return value is the character that should be substituted
 *	in place of the backslash sequence that starts at src.  If
 *	readPtr isn't NULL then it is filled in with a count of the
 *	number of characters in the backslash sequence.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */
char
Tcl_Backslash(src, readPtr)
    CONST char *src;		/* Points to the backslash character of
				 * a backslash sequence. */
    int *readPtr;		/* Fill in with number of characters read
				 * from src, unless NULL. */
{
    CONST char *p = src+1;
    char result;
    int count;

    count = 2;

    switch (*p) {
	case 'a':
	    result = 0x7;	/* Don't say '\a' here, since some compilers */
	    break;		/* don't support it. */
	case 'b':
	    result = '\b';
	    break;
	case 'f':
	    result = '\f';
	    break;
	case 'n':
	    result = '\n';
	    break;
	case 'r':
	    result = '\r';
	    break;
	case 't':
	    result = '\t';
	    break;
	case 'v':
	    result = '\v';
	    break;
	case 'x':
	    if (isxdigit(UCHAR(p[1]))) {
		char *end;

		result = strtoul(p+1, &end, 16);
		count = end - src;
	    } else {
		count = 2;
		result = 'x';
	    }
	    break;
	case '\n':
	    do {
		p++;
	    } while (isspace(UCHAR(*p)));
	    result = ' ';
	    count = p - src;
	    break;
	case 0:
	    result = '\\';
	    count = 1;
	    break;
	default:
	    if (isdigit(UCHAR(*p))) {
		result = *p - '0';
		p++;
		if (!isdigit(UCHAR(*p))) {
		    break;
		}
		count = 3;
		result = (result << 3) + (*p - '0');
		p++;
		if (!isdigit(UCHAR(*p))) {
		    break;
		}
		count = 4;
		result = (result << 3) + (*p - '0');
		break;
	    }
	    result = *p;
	    count = 2;
	    break;
    }

    if (readPtr != NULL) {
	*readPtr = count;
    }
    return result;
}

/*
 *----------------------------------------------------------------------
 *
 * Tcl_AllowExceptions --
 *
 *	Sets a flag in an interpreter so that exceptions can occur
 *	in the next call to Tcl_Eval without them being turned into
 *	errors.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	The TCL_ALLOW_EXCEPTIONS flag gets set in the interpreter's
 *	evalFlags structure.  See the reference documentation for
 *	more details.
 *
 *----------------------------------------------------------------------
 */
void
Tcl_AllowExceptions(interp)
    Tcl_Interp *interp;		/* Interpreter in which to set flag. */
{
    Interp *iPtr = (Interp *) interp;

    iPtr->evalFlags |= TCL_ALLOW_EXCEPTIONS;
}

/******************************************************************************
 *
 * Tcl channels simulation.
 *
 *	Current version is very minimal. It should probably be extended to be 
 *	more Tcl compatible, since Tcl model is very neat.
 *
 ******************************************************************************/

Tcl_Channel Tcl_OpenFileChannel(Tcl_Interp *interp, char *fileName, 
				char *modeString, int permissions)
{
  FILE *f = fopen(fileName, modeString);
  
  if (f == NULL) {
    if (interp != (Tcl_Interp *) NULL) {
      Tcl_AppendResult(interp, "couldn't open \"", fileName, "\": ",
		       Tcl_PosixError(interp), (char *) NULL);
    }
  }
  return (Tcl_Channel) f;
}

int Tcl_Close(Tcl_Interp *interp, Tcl_Channel chan)
{
  return fclose((FILE *) chan);
}

int Tcl_Read(Tcl_Channel chan, char *bufPtr, int toRead)
{
  clearerr((FILE*) chan);
  return fread(bufPtr, 1, (size_t) toRead, (FILE*) chan);
}

int Tcl_Write(Tcl_Channel chan, char *s, int slen)
{
  int len = (slen < 0) ? strlen(s) : slen;
  
  return fwrite(s, 1, (size_t) len, (FILE*) chan);
}


int Tcl_Seek(Tcl_Channel chan, int offset, int mode)
{
  int res;

  if ((res=fseek((FILE*) chan, (long) offset, mode)) != -1)
    return (int) ftell((FILE*) chan);

  return res;
}

int Tcl_Flush(Tcl_Channel chan)
{
  return fflush((FILE *) chan);
}

Tcl_Channel Tcl_GetStdChannel(int type) /*  TCL_STDIN, TCL_STDOUT, TCL_STDERR. */
{
  switch (type) {
    case TCL_STDIN:  return (Tcl_Channel) SCM_stdin;
    case TCL_STDOUT: return (Tcl_Channel) SCM_stdout;
    case TCL_STDERR: return (Tcl_Channel) SCM_stderr;
  }
  return NULL;
}

int Tcl_Eof(Tcl_Channel chan)		/* Does this channel have EOF? */
{
  return feof((FILE *) chan);
}

int Tcl_SetChannelOption(Tcl_Interp *interp, Tcl_Channel chan, char *optionName, 
			 char *newValue)
{
  /* Do nothing */
  return TCL_OK;
}

void SCM_stringify_result(Tcl_Interp *interp, char *value)
{
  /* Transform Tcl result in "result" with " and \ escaped */
  Tcl_SetResult(interp,  SCM_stringify(value, 0), TCL_VOLATILE);
}


/*---------------------------------------------------------------------*/
/*    void                                                             */
/*    Tcl_ResetObjResult ...                                           */
/*---------------------------------------------------------------------*/
void Tcl_ResetObjResult( Interp *interp)
{
  interp->objResultPtr = (Tcl_Obj *)MAKE_CELL( BUNSPEC );
}

/*
 *----------------------------------------------------------------------
 *
 * Tcl_CreateObjCommand --
 *
 *	Define a new object-based command in a command table.
 *
 *----------------------------------------------------------------------
 */

Tcl_Command
Tcl_CreateObjCommand(interp, cmdName, proc, clientData, deleteProc)
    Tcl_Interp *interp;		/* Token for command interpreter (returned
				 * by previous call to Tcl_CreateInterp). */
    char *cmdName;		/* Name of command. If it contains namespace
				 * qualifiers, the new command is put in the
				 * specified namespace; otherwise it is put
				 * in the global namespace. */
    Tcl_ObjCmdProc *proc;	/* Object-based procedure to associate with
				 * name. */
    ClientData clientData;	/* Arbitrary value to pass to object
    				 * procedure. */
    Tcl_CmdDeleteProc *deleteProc;
				/* If not NULL, gives a procedure to call
				 * when this command is deleted. */
{
  struct Tk_command *res;

  res = (struct Tk_command *) Tcl_CreateCommand(interp, 
						cmdName, 
						(Tcl_CmdProc *) proc, 
						clientData, 
						deleteProc);
  res->objproc = 1;
  return (Tcl_Command) res;
}

/*****************************************************************************/
/* Lists */


Tcl_Obj *
Tcl_NewListObj(int objc, Tcl_Obj *CONST objv[] )
{
   obj_t res;
   int i;
  
   res = BNIL;
   if (objc > 0) {
      for (i = 0;  i < objc;  i++) 
	 res = MAKE_PAIR((obj_t) objv[i], res);
      res = reverse__39___r4_pairs_and_lists_6_3(res);
   }
  
   return (Tcl_Obj *)MAKE_CELL(res);
}

int
Tcl_ListObjLength(interp, listPtr, intPtr)
     Tcl_Interp *interp;	/* Used to report errors if not NULL. */
     register Tcl_Obj *listPtr;	/* List object whose #elements to return. */
     register int *intPtr;	/* The resulting int is stored here. */
{
   obj_t data = CELL_REF((obj_t) listPtr);
   assert( CELLP( listPtr ) );
  
   if (!data) 
      CELL_SET((obj_t) listPtr , BNIL);
/*    else                                                             */
/*       if (!PAIRP(data) && !NULLP(data))                             */
/* 	 CELL_SET(((obj_t) listPtr), MAKE_PAIR(data,BNIL));            */
  
/*    *intPtr = list_length(data);                                     */
   *intPtr = -1;
   return TCL_OK;
}

int
Tcl_ListObjAppendElement(interp, listPtr, objPtr)
    Tcl_Interp *interp;		/* Used to report errors if not NULL. */
    Tcl_Obj *listPtr;		/* List object to append objPtr to. */
    Tcl_Obj *objPtr;		/* Object to append to listPtr's list. */
{
  obj_t data;
  obj_t list = (obj_t) listPtr;
  obj_t obj  = (obj_t) objPtr;


  data = CELL_REF(list);
  assert( CELLP( list ) );

  if (!data || NULLP(data) || (data == BUNSPEC))
    data = MAKE_PAIR(obj, BNIL);
  else
    if (PAIRP(data))
      data = append_2_18___r4_pairs_and_lists_6_3(data, MAKE_PAIR(obj, BNIL));
    else
    {
      obj_t aux = MAKE_PAIR(obj, BNIL);
      data = MAKE_PAIR(data, aux);
    }

  CELL_SET( list, data );

  return TCL_OK;
}

Tcl_ListObjReplace(interp, listPtr, first, count, objc, objv)
    Tcl_Interp *interp;		/* Used for error reporting if not NULL. */
    Tcl_Obj *listPtr;		/* List object whose elements to replace. */
    int first;			/* Index of first element to replace. */
    int count;			/* Number of elements to replace. */
    int objc;			/* Number of objects to insert. */
    Tcl_Obj *CONST objv[];	/* An array of objc pointers to Tcl objects
				 * to insert. */
{
  obj_t data = CELL_REF((obj_t) listPtr);
  int len;
  Tcl_Obj *objs;

  assert( CELLP(listPtr) );
  
  if (!data) 
    data = BNIL;
  else
    if (!PAIRP(data) && !NULLP(data))
      data = MAKE_PAIR(data, BNIL);

  len  = list_length(data);
  objs = Tcl_NewListObj(objc, objv);

  if (first < 0) first = 0;
  if (first >= len) {
     assert( CELLP((obj_t)objs) );
    data = append_2_18___r4_pairs_and_lists_6_3(data, CELL_REF((obj_t) objs));
    /* Tcl_DecrRefCount(objs); */
  }		       
  else {
    panic("Problem in Tcl_ListObjReplace");
  }
  return 0; /* never reached */
}

int
Tcl_ListObjIndex(interp, listPtr, index, objPtrPtr)
    Tcl_Interp *interp;		/* Used to report errors if not NULL. */
    register Tcl_Obj *listPtr;	/* List object to index into. */
    register int index;		/* Index of element to return. */
    Tcl_Obj **objPtrPtr;	/* The resulting Tcl_Obj* is stored here. */
{
  obj_t data = CELL_REF((obj_t) listPtr);
  int len;

  assert( CELLP((obj_t)listPtr ) );
	  
  if (!data) 
    data = BNIL;
  else
    if (!PAIRP(data) && !NULLP(data))
      data = MAKE_PAIR(data, BNIL);
  
  len = list_length(data);
  if ((index < 0) || (index >= len)) {
    /* FIXME: Est ce intelligent de retourner NULL? pourquoi pas NIL?  */
    *objPtrPtr = NULL;
  } else {
    *objPtrPtr = (Tcl_Obj *)list_ref_194___r4_pairs_and_lists_6_3(data, index);
  } 
  return TCL_OK;
}

/* This function uses a malloced array for returning the Tcl_Obj pointers
 * This is not fully compatible with Tcl, but This function is used only
 * for parsing the new font name scheme.
 * Should be rewritten if used elsewhere (in particular with long lists)
 */
int
Tcl_ListObjGetElements(interp, listPtr, objcPtr, objvPtr)
    Tcl_Interp *interp;		/* Used to report errors if not NULL. */
    register Tcl_Obj *listPtr;	/* List object for which an element array
				 * is to be returned. */
    int *objcPtr;		/* Where to store the count of objects
				 * referenced by objv. */
    Tcl_Obj ***objvPtr;		/* Where to store the pointer to an array
				 * of pointers to the list's objects. */
{
  static Tcl_Obj *obj[100];
  int i, len;
  obj_t data = CELL_REF((obj_t) listPtr);

  assert( CELLP((obj_t) listPtr) );

  if (!data) 
    data = BNIL;
  else 
    if (STRINGP(data))
      /* We have a string and we want to look at it as a list */
      data = SCM_tk_string_to_scm_list( BSTRING_TO_STRING(data) );
    else
      if (!PAIRP(data) && !NULLP(data))
	data = MAKE_PAIR(data, BNIL);
  
  len = list_length(data);
  if (len > 100)
    panic("Too much data in Tcl_ListObjGetElements\n");

  for (i = 0; i < len; i++, data=CDR(data))
    obj[i] = (Tcl_Obj *)MAKE_CELL(CAR(data));

  *objcPtr = len;
  *objvPtr = obj;

  return TCL_OK;
}

/*****************************************************************************/
/* Strings */

void Tcl_InitStringRep(Tcl_Obj *objPtr, char *bytePtr, int len)
{
  CELL_SET((obj_t) objPtr, string_to_bstring_len(bytePtr, len));
}

Tcl_Obj *
Tcl_NewStringObj(bytes, length)
    register char *bytes;	/* Points to the first of the length bytes
				 * used to initialize the new object. */
    register int length;	/* The number of bytes to copy from "bytes"
				 * when initializing the new object. If 
				 * negative, use bytes up to the first
				 * NULL byte. */
{
  if (length < 0) length = strlen(bytes);
  return (Tcl_Obj *)MAKE_CELL(string_to_bstring_len(bytes, length));
}

void
Tcl_AppendToObj(objPtr, bytes, length)
    register Tcl_Obj *objPtr;	/* Points to the object to append to. */
    char *bytes;		/* Points to the bytes to append to the
				 * object. */
    register int length;	/* The number of bytes to append from
				 * "bytes". If < 0, then append all bytes
				 * up to NULL byte. */
{
  obj_t str = CELL_REF((obj_t) objPtr);
  char *s;
  int len;

  assert( CELLP( (obj_t)objPtr ) );
  
  if (!str || NULLP(str))
    str = c_constant_string_to_string("");
  else
    if (!STRINGP(str))
      C_FAILURE("internal error (Tcl_AppendToObj only accepts strings).",
		"It was",
		str);

  if (length < 0) length = strlen(bytes);
  len = STRING_LENGTH(str) + length;	
  s = (char *)alloca((size_t) len + 1);		/* +1 for the null  */

  sprintf(s, "%s%s", BSTRING_TO_STRING(str), bytes);
  CELL_SET((obj_t) objPtr, string_to_bstring_len(s, len));
}

char *
Tcl_GetStringFromObj(objPtr, lengthPtr)
    register Tcl_Obj *objPtr;	/* Object whose string rep byte pointer
				 * should be returned. */
    register int *lengthPtr;	/* If non-NULL, the location where the
				 * string rep's byte array length should be
				 * stored. If NULL, no length is stored. */
{
   if( !objPtr )
      return "";
   else
   {
      obj_t s = CELL_REF((obj_t) objPtr);

      assert( CELLP((obj_t)objPtr ) );
      if (!s)
	 s = c_constant_string_to_string("");
      else
	 if (!STRINGP(s)) {
	    char *string;
      
	    string = SCM_scm_to_tk_string(s);
	    s = string_to_bstring(string);
	 }

      if (lengthPtr) *lengthPtr = STRING_LENGTH(s);
      return BSTRING_TO_STRING(s);
   }
}

void
Tcl_SetStringObj(objPtr, bytes, length)
    register Tcl_Obj *objPtr;	/* Object whose internal rep to init. */
    char *bytes;		/* Points to the first of the length bytes
				 * used to initialize the object. */
    register int length;	/* The number of bytes to copy from "bytes"
				 * when initializing the object. If 
				 * negative, use bytes up to the first
				 * NULL byte.*/
{
  if (length < 0) length = strlen(bytes);
   CELL_SET((obj_t) objPtr, string_to_bstring_len(bytes, length));
  /* old string will be automatically garbaged */
}

void
Tcl_AppendStringsToObj TCL_VARARGS_DEF(Tcl_Obj *,arg1)
{
    va_list argList;
    register Tcl_Obj *objPtr;
    int newLength, oldLength;
    register char *string, *dst;
    obj_t data;
    obj_t res;
    
    objPtr = (Tcl_Obj *) TCL_VARARGS_START(Tcl_Obj *,arg1,argList);

    data = CELL_REF((obj_t) objPtr);

    assert(CELLP((obj_t)objPtr));

    if (!data) 
      CELL_SET(((obj_t) objPtr) ,data = c_constant_string_to_string(""));
    else
      if (!STRINGP(data)) {
	obj_t newdata;
	char *aux;

	aux = SCM_scm_to_tk_string(data);
	newdata = string_to_bstring( aux );
	CELL_SET(((obj_t) objPtr) ,data = newdata);
      }

    /*
     * Figure out how much space is needed for all the strings, and
     * expand the string representation if it isn't big enough. If no
     * bytes would be appended, just return.
     */

    newLength = oldLength = STRING_LENGTH(data);
    while (1) {
	string = va_arg(argList, char *);
	if (string == NULL) {
	    break;
	}
	newLength += strlen(string);
    }
    if (newLength == oldLength) {
	return;
    }

    res = make_string((size_t)newLength+1, 0);

    /*
     * Make a second pass through the arguments, appending all the
     * strings to the object.
     */

    TCL_VARARGS_START(Tcl_Obj *,arg1,argList);
    dst = BSTRING_TO_STRING(res) + oldLength;
    
    while (1) {
	string = va_arg(argList, char *);
	if (string == NULL) {
	    break;
	}
	while (*string != 0) {
	    *dst = *string;
	    dst++;
	    string++;
	}
    }

    /*
     * Add a null byte to terminate the string.  However, be careful:
     * it's possible that the object is totally empty (if it was empty
     * originally and there was nothing to append).  In this case dst is
     * NULL; just leave everything alone.
     */

    if (dst != NULL) {
	*dst = 0;
    }
    va_end(argList);

    CELL_SET((obj_t)objPtr, res);
}


/*****************************************************************************/
/* Doubles */

void
Tcl_SetDoubleObj(objPtr, dblValue)
    register Tcl_Obj *objPtr;	/* Object whose internal rep to init. */
    register double dblValue;	/* Double used to set the object's value. */
{
  CELL_SET((obj_t) objPtr, DOUBLE_TO_REAL(dblValue));
}

int
Tcl_GetDoubleFromObj(interp, objPtr, dblPtr)
    Tcl_Interp *interp; 	/* Used for error reporting if not NULL. */
    register Tcl_Obj *objPtr;	/* The object from which to get a double. */
    register double *dblPtr;	/* Place to store resulting double. */
{
  obj_t data = ((obj_t) objPtr);

  if (!data) 
    data = DOUBLE_TO_REAL((double) 0.0);

  if (INTEGERP(data))
    data = exact__inexact_122___r4_numbers_6_5(data);
  if (REALP(data)) {
    *dblPtr = REAL_TO_DOUBLE(data);
    return TCL_OK;
  }
  else {
    if (interp != NULL) {
      Tcl_ResetResult(interp);
      Tcl_AppendToObj(Tcl_GetObjResult(interp), 
		      "expected floating-point number", -1);
    }
    return TCL_ERROR;
  }  
}



/*****************************************************************************/
/* Longs */

void
Tcl_SetLongObj(objPtr, longValue)
    register Tcl_Obj *objPtr;	/* Object whose internal rep to init. */
    register long longValue;	/* Long integer used to initialize the
				 * object's value. */
{
  CELL_SET((obj_t) objPtr , BINT(longValue));
}

int
Tcl_GetLongFromObj(interp, objPtr, longPtr)
    Tcl_Interp *interp; 	/* Used for error reporting if not NULL. */
    register Tcl_Obj *objPtr;	/* The object from which to get a long. */
    register long *longPtr;	/* Place to store resulting long. */
{
  obj_t data = CELL_REF((obj_t) objPtr);
  assert(CELLP((obj_t)objPtr));

  if (!data)
    *longPtr = 0L;
  else
    *longPtr = CINT(data);

  if (*longPtr == LONG_MIN) {
    Tcl_ResetResult(interp);
    Tcl_AppendToObj(Tcl_GetObjResult(interp), 
		    "expected integer number", -1);
    return TCL_ERROR;
  }
  else 
    return TCL_OK;
}

/*****************************************************************************/
/* Integers (as longs in SCM) */

Tcl_Obj *
Tcl_NewIntObj(intValue)
    register int intValue;	/* Int used to initialize the new object. */
{
  return (Tcl_Obj *)MAKE_CELL(BINT(intValue));
}


void
Tcl_SetIntObj(objPtr, intValue)
    register Tcl_Obj *objPtr;	/* Object whose internal rep to init. */
    register int intValue;	/* Int integer used to initialize the
				 * object's value. */
{
  CELL_SET((obj_t) objPtr, BINT(intValue));
}

int
Tcl_GetIntFromObj(interp, objPtr, intPtr)
    Tcl_Interp *interp; 	/* Used for error reporting if not NULL. */
    register Tcl_Obj *objPtr;	/* The object from which to get a int. */
    register int *intPtr;	/* Place to store resulting int. */
{
  obj_t data = CELL_REF((obj_t) objPtr);
  assert(CELLP((obj_t)objPtr));
  
  if (!data)
    *intPtr = 0;
  else
    *intPtr = CINT(data);

  if (*intPtr == LONG_MIN) {
    Tcl_ResetResult(interp);
    Tcl_AppendToObj(Tcl_GetObjResult(interp), 
		    "expected integer number", -1);
    return TCL_ERROR;
  }
  else 
    return TCL_OK;
}

/*****************************************************************************/
/* Booleans */
void
Tcl_SetBooleanObj(objPtr, boolValue)
    register Tcl_Obj *objPtr;	/* Object whose internal rep to init. */
    register int boolValue;	/* Boolean used to set object's value. */
{
  CELL_SET((obj_t) objPtr, boolValue? BTRUE: BFALSE);
}

int
Tcl_GetBooleanFromObj(interp, objPtr, boolPtr)
    Tcl_Interp *interp; 	/* Used for error reporting if not NULL. */
    register Tcl_Obj *objPtr;	/* The object from which to get boolean. */
    register int *boolPtr;	/* Place to store resulting boolean. */
{
  obj_t data = CELL_REF((obj_t) objPtr);
  assert(CELLP((obj_t)objPtr));

  /* Accept various form of boolean "yes" 1 #t */
  if (!data) 
    *boolPtr = 0;
  else
    if (STRINGP(data))
      Tcl_GetBoolean(interp, BSTRING_TO_STRING(data), boolPtr);
    else
      if (INTEGERP(data))
	*boolPtr = CINT(data) != 0;
      else 
	*boolPtr = (data != BFALSE);
  return TCL_OK;
}
  
Tcl_Obj *
Tcl_NewBooleanObj(boolValue)
    register int boolValue;	/* Boolean used to initialize new object. */
{
  return (Tcl_Obj *)MAKE_CELL( boolValue? BTRUE: BFALSE );
}


/*****************************************************************************/
/* Indexes */
int
Tcl_GetIndexFromObj(interp, objPtr, tablePtr, msg, flags, indexPtr)
    Tcl_Interp *interp; 	/* Used for error reporting if not NULL. */
    Tcl_Obj *objPtr;		/* Object containing the string to lookup. */
    char **tablePtr;		/* Array of strings to compare against the
				 * value of objPtr; last entry must be NULL
				 * and there must not be duplicate entries. */
    char *msg;			/* Identifying word to use in error messages. */
    int flags;			/* 0 or TCL_EXACT */
    int *indexPtr;		/* Place to store resulting integer index. */
{
    int index, length, i, numAbbrev;
    char *key, *p1, *p2, **entryPtr;
    Tcl_Obj *resultPtr;

    /*
     * Lookup the value of the object in the table.  Accept unique
     * abbreviations unless TCL_EXACT is set in flags.
     */

    key = Tcl_GetStringFromObj(objPtr, &length);
    index = -1;
    numAbbrev = 0;
    for (entryPtr = tablePtr, i = 0; *entryPtr != NULL; entryPtr++, i++) {
	for (p1 = key, p2 = *entryPtr; *p1 == *p2; p1++, p2++) {
	    if (*p1 == 0) {
		index = i;
		goto done;
	    }
	}
	if (*p1 == 0) {
	    /*
	     * The value is an abbreviation for this entry.  Continue
	     * checking other entries to make sure it's unique.  If we
	     * get more than one unique abbreviation, keep searching to
	     * see if there is an exact match, but remember the number
	     * of unique abbreviations and don't allow either.
	     */

	    numAbbrev++;
	    index = i;
	}
    }
    if ((flags & TCL_EXACT) || (numAbbrev != 1)) {
	goto error;
    }

    done:
    *indexPtr = index;
    return TCL_OK;

    error:
    if (interp != NULL) {
	resultPtr = Tcl_GetObjResult(interp);
	Tcl_AppendStringsToObj(resultPtr,
		(numAbbrev > 1) ? "ambiguous " : "bad ", msg, " \"",
		key, "\": must be ", *tablePtr, (char *) NULL);
	for (entryPtr = tablePtr+1; *entryPtr != NULL; entryPtr++) {
	    if (entryPtr[1] == NULL) {
		Tcl_AppendStringsToObj(resultPtr, ", or ", *entryPtr,
				       (char *) NULL);
	    } else {
		Tcl_AppendStringsToObj(resultPtr, ", ", *entryPtr,
				       (char *) NULL);
	    }
	}
    }
    return TCL_ERROR;
}  
  
/*
 *----------------------------------------------------------------------
 *
 * Tcl_WrongNumArgs --
 *
 *	This procedure generates a "wrong # args" error message in an
 *	interpreter.  It is used as a utility function by many command
 *	procedures.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	An error message is generated in interp's result object to
 *	indicate that a command was invoked with the wrong number of
 *	arguments.  The message has the form
 *		wrong # args: should be "foo bar additional stuff"
 *	where "foo" and "bar" are the initial objects in objv (objc
 *	determines how many of these are printed) and "additional stuff"
 *	is the contents of the message argument.
 *
 *----------------------------------------------------------------------
 */

void
Tcl_WrongNumArgs(interp, objc, objv, message)
    Tcl_Interp *interp;			/* Current interpreter. */
    int objc;				/* Number of arguments to print
					 * from objv. */
    Tcl_Obj *CONST objv[];		/* Initial argument objects, which
					 * should be included in the error
					 * message. */
    char *message;			/* Error message to print after the
					 * leading objects in objv. */
{
    Tcl_Obj *objPtr;
    int i;
#ifdef SCM_CODE
    int stk_hack = (objc == 2);
#endif
    
    objPtr = Tcl_GetObjResult(interp);
#ifdef SCM_CODE
    Tcl_AppendToObj(objPtr, "wrong # args: should be (", -1);
    for (i = 0; i < objc; i++) {

      if (stk_hack && i == 1) {
	/* VERY VERY HACKY: place a quote in front of the second element
	 * of the message. This is a special case for messages such as
	 *      wrong # args: should be (.but 'configure ....)
	 *  But this code is so frequent.
	 */
	Tcl_AppendStringsToObj(objPtr, "'",
			       Tcl_GetStringFromObj(objv[i], (int *) NULL), " ",
			       (char *) NULL);
      }	  
      else {
	Tcl_AppendStringsToObj(objPtr,
			       Tcl_GetStringFromObj(objv[i], (int *) NULL), " ",
			       (char *) NULL);
      }
    }
    Tcl_AppendStringsToObj(objPtr, message, ")", (char *) NULL);
#else
    Tcl_AppendToObj(objPtr, "wrong # args: should be \"", -1);
    for (i = 0; i < objc; i++) {
	Tcl_AppendStringsToObj(objPtr,
		Tcl_GetStringFromObj(objv[i], (int *) NULL), " ",
		(char *) NULL);
    }
    Tcl_AppendStringsToObj(objPtr, message, "\"", (char *) NULL);
#endif
}

/*
 *----------------------------------------------------------------------
 *
 * Tcl_TildeSubst --
 *
 *	Given a name starting with a tilde, produce a name where
 *	the tilde and following characters have been replaced by
 *	the home directory location for the named user.
 *
 * Results:
 *	The result is a pointer to a static string containing
 *	the new name.  If there was an error in processing the
 *	tilde, then an error message is left in interp->result
 *	and the return value is NULL.  The result may be stored
 *	in bufferPtr; the caller must call Tcl_DStringFree(bufferPtr)
 *	to free the name.
 *
 * Side effects:
 *	Information may be left in bufferPtr.
 *
 *----------------------------------------------------------------------
 */
char *
Tcl_TildeSubst(interp, name, bufferPtr)
    Tcl_Interp *interp;		/* Interpreter in which to store error
				 * message (if necessary). */
    char *name;			/* File name, which may begin with "~/"
				 * (to indicate current user's home directory)
				 * or "~<user>/" (to indicate any user's
				 * home directory). */
    Tcl_DString *bufferPtr;	/* May be used to hold result.  Must not hold
				 * anything at the time of the call, and need
				 * not even be initialized. */
{
    char *dir;
    register char *p;

    Tcl_DStringInit(bufferPtr);
    if (name[0] != '~') {
        Tcl_DStringAppend(bufferPtr, name, -1);
	return name;
    }

    if ((name[1] == '/') || (name[1] == '\0')) {
	dir = getenv("HOME");
	if (dir == NULL) {
	    Tcl_ResetResult(interp);
	    Tcl_AppendResult(interp, "couldn't find HOME environment ",
		    "variable to expand \"", name, "\"", (char *) NULL);
	    return NULL;
	}
	Tcl_DStringAppend(bufferPtr, dir, -1);
	Tcl_DStringAppend(bufferPtr, name+1, -1);
    } else {
	struct passwd *pwPtr;

	for (p = &name[1]; (*p != 0) && (*p != '/'); p++) {
	    /* Null body;  just find end of name. */
	}
	Tcl_DStringAppend(bufferPtr, name+1, p - (name+1));
#ifdef WIN32
	Tcl_ResetResult(interp);
	Tcl_AppendResult(interp, "user \"", bufferPtr->string,
		    "\" doesn't exist", (char *) NULL);
	return NULL;
    }
#else
	pwPtr = getpwnam(bufferPtr->string);
	if (pwPtr == NULL) {
	    endpwent();
	    Tcl_ResetResult(interp);
	    Tcl_AppendResult(interp, "user \"", bufferPtr->string,
		    "\" doesn't exist", (char *) NULL);
	    return NULL;
	}
	Tcl_DStringFree(bufferPtr);
	Tcl_DStringAppend(bufferPtr, pwPtr->pw_dir, -1);
	Tcl_DStringAppend(bufferPtr, p, -1);
	endpwent();
    }
    return bufferPtr->string;
#endif
}

/*
 *----------------------------------------------------------------------
 *
 * Tcl_GetCommandInfo --
 *
 *	Returns various information about a Tcl command.
 *
 * Results:
 *	If cmdName exists in interp, then *infoPtr is modified to
 *	hold information about cmdName and 1 is returned.  If the
 *	command doesn't exist then 0 is returned and *infoPtr isn't
 *	modified.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

int
Tcl_GetCommandInfo(interp, cmdName, infoPtr)
    Tcl_Interp *interp;			/* Interpreter in which to look
					 * for command. */
    char *cmdName;			/* Name of desired command. */
    Tcl_CmdInfo *infoPtr;		/* Where to store information about
					 * command. */
{
    obj_t v;
    struct Tk_command *p;
    
    v = tcl_lookup_command(cmdName);
    if (!PROCEDUREP(v)) return 0;

    p =  SCM_TO_TK_COMMAND( v );

    infoPtr->proc       = p->fct;
    infoPtr->clientData = p->ptr;
    infoPtr->deleteProc = p->delproc;;
    infoPtr->deleteData = NULL;
    return 1;
}

/*
 *----------------------------------------------------------------------
 *
 * Tcl_GetCommandName --
 *
 *	Given a token returned by Tcl_CreateCommand, this procedure
 *	returns the current name of the command (which may have changed
 *	due to renaming).
 *
 * Results:
 *	The return value is the name of the given command.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

char *
Tcl_GetCommandName(interp, command)
    Tcl_Interp *interp;		/* Interpreter containing the command. */
    Tcl_Command command;	/* Token for the command, returned by a
				 * previous call to Tcl_CreateCommand.
				 * The command must not have been deleted. */
{
  return ((struct Tk_command *) command)->Id;
}

/*---------------------------------------------------------------------*/
/*    int                                                              */
/*    Tcl_GlobalEval ...                                               */
/*---------------------------------------------------------------------*/
int
Tcl_GlobalEval(Tcl_Interp *interp, char *s)
{
   obj_t exp;
   
  /* 
   * If the callback is nor surrounded by parenthesis, add them. We
   * don't have parenthesis when the callback is a closure. In this
   * case, the callback is simply #p12345. Note that this allow Tk to
   * add some parameters to the callback when needed (on bindings, or
   * scrollbars for instance). To recognize such cases, we look at
   * first character: if it is not an open parenthesis, we add a pair
   * of () around the callback string
   *     
   */
  if (*s == '\0') return TCL_OK;
  if (*s != '(') {
     char *ptr;
     /* Build the command to evaluate by adding a pair of parenthesis */
     int len = strlen(s);
    
     ptr = (char *) alloca((size_t) len+3);
     sprintf(ptr, "(%s)", s);
     s = ptr;
  }

   exp = SCM_tk_string_to_scm_list( s );

   return SCM_tk_funcall( exp );
}

int Tcl_Eval(Tcl_Interp *interp, char *s)
{
  return Tcl_GlobalEval(interp, s);
}
 
/*
 *----------------------------------------------------------------------
 *
 * Tcl_VarEval --
 *
 *	Given a variable number of string arguments, concatenate them
 *	all together and execute the result as a Tcl command.
 *
 * Results:
 *	A standard Tcl return result.  An error message or other
 *	result may be left in interp->result.
 *
 * Side effects:
 *	Depends on what was done by the command.
 *
 *----------------------------------------------------------------------
 */
	/* VARARGS2 */ /* ARGSUSED */
int
Tcl_VarEval TCL_VARARGS_DEF(Tcl_Interp *,arg1)
{
    va_list argList;
    Tcl_DString buf;
    char *string;
    Tcl_Interp *interp;
    int result;

    /*
     * Copy the strings one after the other into a single larger
     * string.  Use stack-allocated space for small commands, but if
     * the command gets too large than call ckalloc to create the
     * space.
     */

    interp = TCL_VARARGS_START(Tcl_Interp *,arg1,argList);
    Tcl_DStringInit(&buf);
    while (1) {
	string = va_arg(argList, char *);
	if (string == NULL) {
	    break;
	}
	Tcl_DStringAppend(&buf, string, -1);
    }
    va_end(argList);

    result = Tcl_Eval(interp, Tcl_DStringValue(&buf));
    Tcl_DStringFree(&buf);
    return result;
}

/*---------------------------------------------------------------------*/
/*    static obj_t                                                     */
/*    SCM_tcl_list_to_scm_list ...                                     */
/*---------------------------------------------------------------------*/
static obj_t
SCM_tcl_list_to_scm_list( obj_t l ) {
   if( !l )
      return BNIL;
   if( !PAIRP( l ) )
      return CELLP( l ) ? CELL_REF( l ) : l;
   return MAKE_PAIR( SCM_tcl_list_to_scm_list( CAR( l ) ),
		     SCM_tcl_list_to_scm_list( CDR( l ) ) );
}

/*---------------------------------------------------------------------*/
/*    static obj_t                                                     */
/*    TkResult2Scheme ...                                              */
/*---------------------------------------------------------------------*/
static obj_t
TkResult2Scheme(Tcl_Interp *interp, int objproc)
{
  obj_t res;

  if (objproc) {
    obj_t data = CELL_REF((obj_t) Tcl_GetObjResult(interp));
    assert(CELLP((obj_t) Tcl_GetObjResult(interp)));
    res = data ? SCM_tcl_list_to_scm_list(data) : BNIL;
  }
  else {
    register char *s = interp->result;
    res = (*s) ? SCM_tk_string_to_scm_list(s) : BNIL;
  }
  Tcl_ResetResult(interp);

  return res;
}

/*---------------------------------------------------------------------*/
/*    obj_t                                                            */
/*    tcl_call_command ...                                             */
/*---------------------------------------------------------------------*/
obj_t
tcl_call_command( obj_t proc, obj_t lst ) {
   int tkres;
   struct Tk_command *W = SCM_TO_TK_COMMAND( proc );
   int argc = list_length( lst ) + 1;
   int i;
   int objproc;
   char **argv;

   objproc = W->objproc;
   argv = (char **)alloca( sizeof( char * ) * (argc + 1) );

   /* First initialize an argv array */
   if (objproc) {
      /* Objproc procedure (Tcl_8.0+) */
      argv[0] = (char*) Tcl_NewStringObj(W->Id, -1);
      for (i = 1; PAIRP(lst); i++, lst=CDR(lst)) {
	 argv[i] = (char*) MAKE_CELL(CAR(lst));
      }
   }
   else {
      argv[0] = W->Id;
      
      for( i = 1; PAIRP( lst ); i++, lst = CDR( lst ) ) {
	 int j;
	 
	 argv[ i ] = SCM_scm_to_tk_string( CAR( lst ) );
/* 	 printf( "argv[%d]: ", i );                                    */
/* 	 for( j = 0; (j < 10) && argv[ i ][ j ]; j++ )                 */
/* 	    printf( "%c ", argv[ i ][ j ] );                           */
/* 	 puts("");                                                     */
      }
   }
   argv[ i ] = 0;

   /* Now, call the Tk library function */
   Tcl_ResetResult(SCM_main_interp);

   tkres = W->fct( W->ptr, SCM_main_interp, argc, argv );

   /* return result as a string or "evaluated" depending of */
   /* string_result field                                   */
   if (tkres == TCL_OK)
      return TkResult2Scheme(SCM_main_interp, objproc);
   
   /* if we are here, we had an error. Signal it */
   if (objproc) {
      Tcl_Obj *mess = Tcl_GetObjResult(SCM_main_interp);
      obj_t msg;

      msg = CELL_REF((obj_t) mess);
      assert(CELLP((obj_t)mess));

      if (!msg) msg = BNIL;

      C_FAILURE( "biglook", argv[ 0 ], msg );
   }
   else
      C_FAILURE( "biglook", argv[ 0 ], string_to_bstring( SCM_main_interp->result ) );
   
   return BUNSPEC;
}

   
/******************************************************************************
 *
 * Callback management
 *
 ******************************************************************************/

static Tcl_HashTable Tk_callbacks;

void SCM_tcl_init_callbacks() {
   /* Init the callback table */
   Tcl_InitHashTable(&Tk_callbacks, TCL_STRING_KEYS);
}

int SCM_valid_callback(char *s, void **closure)
{
   /* A callback is valid iff it is of the form "#pxxxx" where xxxx is composed
    * only of hexadecimal digits.
    * Furthermore, the given address must  be a valid adress
    */
   char *p;

   if( !s )
      return 0;
   
/* printf("SCM_valid_callback: [%s]\n", s );                           */
   *closure = NULL;
   if (s[0] == '(') s++;
   if (s[0] == '#' && s[1] == 'p') {
      /* Verify that the rest of the string only contains hexadecimal digits */
      for (p = s + 2; *p && ((*p != ' ') && *p != ')'); p++)
	 if (!isxdigit(*p)) return 0;

      sscanf(s+2, "%lx", (unsigned long *) closure);
   }
   return 1;
}

/* static obj_t biglook_glob = BNIL;                                   */

void SCM_add_callback(char *key1, char *key2, char *key3, obj_t closure)
{
  Tcl_HashEntry *entry;
  Tcl_HashTable *secondary_hash_table;
  int new;
  char key[200]; /* Largely sufficient */

/*   biglook_glob = MAKE_PAIR( closure, biglook_glob );                */
/*   printf( "SCM_add_callback: %lx\n", closure );                     */
  if (*key2) {
    /* We have two keys. Use a secondary hash table */
    if (entry=Tcl_FindHashEntry(&Tk_callbacks, key1))
      /* Key already in hash table */
      secondary_hash_table = (Tcl_HashTable * ) Tcl_GetHashValue(entry);
    else {
      secondary_hash_table = (Tcl_HashTable *) GC_MALLOC(sizeof(Tcl_HashTable));
      Tcl_InitHashTable(secondary_hash_table, TCL_STRING_KEYS);
      entry = Tcl_CreateHashEntry(&Tk_callbacks, (char *) key1, &new);
      Tcl_SetHashValue(entry, secondary_hash_table);
    }
    
    /* Enter a new key (obtained from key2 and key3) in the hash table.
     * Don't worry about old value: since it is no more pointed by the 
     * hash table, it will be garbaged at next GC run
     */
    sprintf(key, "%s#%s", key2, key3);/* Create a new key from key2 and key3 */
    entry = Tcl_CreateHashEntry(secondary_hash_table, key, &new);
    Tcl_SetHashValue(entry, closure); 
  }
  else {
    /* Only one key. No need for a secondary hash table */
    entry =Tcl_CreateHashEntry(&Tk_callbacks, key1, &new);
    Tcl_SetHashValue(entry, closure);
  }
}


void SCM_delete_callback(char *key)
{
  /*
   * key is destroyed. We only need to free the entry associated to it in the 
   * Tk_callback hash table (if it exists).
   */
  Tcl_HashEntry *entry;
  Tcl_HashTable *secondary_hash_table;

  if (entry=Tcl_FindHashEntry(&Tk_callbacks, key)) {
    if (*key != '(' && (*key == 'a' && strncmp(key, "after.", 6) != 0)) {
      /* Delete the secondary hash table associated to this entry */
      secondary_hash_table = (Tcl_HashTable * ) Tcl_GetHashValue(entry);
      Tcl_DeleteHashTable(secondary_hash_table);
      GC_free(secondary_hash_table);
    }
    /* Delete the entry itself */
    Tcl_DeleteHashEntry(entry);
  }
}

