/*---------------------------------------------------------------------*/
/*   This program is free software; you can redistribute it            */
/*   and/or modify it under the terms of the GNU General Public        */
/*   License as published by the Free Software Foundation; either      */
/*   version 2 of the License, or (at your option) any later version.  */
/*                                                                     */
/*   This program is distributed in the hope that it will be useful,   */
/*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
/*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
/*   GNU General Public License for more details.                      */
/*                                                                     */
/*   You should have received a copy of the GNU General Public         */
/*   License along with this program; if not, write to the Free        */
/*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
/*   MA 02111-1307, USA.                                               */
/*---------------------------------------------------------------------*/
/*=====================================================================*/
/*    serrano/prgm/project/bglk/runtime/Clib/tk-main.c                 */
/*    -------------------------------------------------------------    */
/*    Author      :  Manuel Serrano                                    */
/*    Creation    :  Fri Oct 16 14:12:32 1998                          */
/*    Last change :  Sat May  8 11:15:51 1999 (serrano)                */
/*    -------------------------------------------------------------    */
/*    The TK initialization                                            */
/*=====================================================================*/
#include <strings.h>
#include <bigloo.h>
#include <signal.h>
#include <tkInt.h>
#include <stdio.h>

/*---------------------------------------------------------------------*/
/*    Tcl_Interp *                                                     */
/*    SCM_main_interp ...                                              */
/*    -------------------------------------------------------------    */
/*    Interpreter for this application                                 */
/*---------------------------------------------------------------------*/
Tcl_Interp *SCM_main_interp = NULL;

/*---------------------------------------------------------------------*/
/*    int                                                              */
/*    Tk_initialized ...                                               */
/*---------------------------------------------------------------------*/
int Tk_initialized = 0;
extern void Tk_main();
extern void SCM_init_pixmap();
extern int (*the_getc)(FILE *);
extern int (*the_feof)(FILE *);

/*---------------------------------------------------------------------*/
/*    Static io buffer                                                 */
/*---------------------------------------------------------------------*/
#define BUFFER_SIZE 512
#define TCL_EVENT_FLAG 0

static char buffer[ BUFFER_SIZE+1 ];
static int bufidx=0;
static int count=-1;
static int filled=0;
static int feof_stdin = 0;

/*---------------------------------------------------------------------*/
/*    int                                                              */
/*    tk_getc ...                                                      */
/*---------------------------------------------------------------------*/
int
tk_getc( FILE *f ) {

   if (bufidx < count)
      return buffer[bufidx++];
   else {
      filled = 0;
      while (!filled) {
	 Tcl_DoOneEvent(TCL_EVENT_FLAG);
	 if (Tk_GetNumMainWindows() <= 0) return EOF;
      }
   }

   if (count <= 0) {
      return EOF;
   }
   else {
      bufidx = 1;
      return *buffer;
   }
}

/*---------------------------------------------------------------------*/
/*    int                                                              */
/*    tk_feof ...                                                      */
/*---------------------------------------------------------------------*/
int
tk_feof( FILE *f ) {
   return feof( f ) || feof_stdin;
}

/*---------------------------------------------------------------------*/
/*    void                                                             */
/*    stdin_proc ...                                                   */
/*---------------------------------------------------------------------*/
void
stdin_proc() {
   feof_stdin = 0;   
   for ( ; ; ) {
      count = read( fileno( stdin ), buffer, BUFFER_SIZE );
      if( !count ) feof_stdin = 1;
      if( count != -1 || errno != EINTR ) break;
   }
   filled = 1;
}

/*---------------------------------------------------------------------*/
/*    static void                                                      */
/*    init_tk_io ...                                                   */
/*---------------------------------------------------------------------*/
static void
init_tk_io() {
   Tcl_CreateFileHandler( fileno( stdin ),
			  TCL_READABLE,
			  (Tk_FileProc *) stdin_proc, 
			  (ClientData) NULL );
   the_getc = tk_getc;
   the_feof = tk_feof;
}

/*---------------------------------------------------------------------*/
/*    int                                                              */
/*    biglook_mainloopp ...                                            */
/*---------------------------------------------------------------------*/
int biglook_mainloopp = 1;

/*---------------------------------------------------------------------*/
/*    void                                                             */
/*    biglook_mainloop ...                                             */
/*---------------------------------------------------------------------*/
static void
biglook_mainloop( void ) {
   if( biglook_mainloopp )
      Tk_MainLoop();
}

/*---------------------------------------------------------------------*/
/*    void                                                             */
/*    biglook_exit ...                                                 */
/*---------------------------------------------------------------------*/
void
biglook_exit( int status ) {
   biglook_mainloopp = 0;
   exit( status );
}

/*---------------------------------------------------------------------*/
/*    static void                                                      */
/*    init_tk_mainloop ...                                             */
/*---------------------------------------------------------------------*/
static void
init_tk_mainloop() {
   atexit( biglook_mainloop );
}

/*---------------------------------------------------------------------*/
/*    static void                                                      */
/*    tcl_main ...                                                     */
/*---------------------------------------------------------------------*/
static void tcl_main( void ) {
  SCM_main_interp = Tcl_CreateInterp();
  SCM_tcl_init_tracevar();
  SCM_tcl_init_callbacks();
}

/*---------------------------------------------------------------------*/
/*    void                                                             */
/*    Tk_main ...                                                      */
/*---------------------------------------------------------------------*/
void Tk_main( int main_loop_p,
              char *argv0,
	      int synchronize,
	      char *name,
	      char *Xdisplay,
	      char *geometry ) {
   char *p, *Class;
   int argc, code;
   char *args[ 20 ];  

   /* tk io initialization */
   init_tk_io();

   /* tk main loop registration (only for application that needs it). */
   if( main_loop_p )
      init_tk_mainloop();
   
   tcl_main();
   
   /* Initialize the Tk application and arrange to map the main window */
   /* after the startup script has been executed, if any.  This way    */
   /* the script can withdraw the window so it isn't ever mapped at all*/
   Class = "BGlk";

   /* Find the name of the application */
   if (name == NULL)
      name = Class;

   args[0] = "toplevel";
   args[1] = ".";
   args[2] = "-class";
   args[3] = Class;
   argc    = 4;

   if (Xdisplay != NULL) {
      args[argc] = "-screen";
      args[argc+1] = Xdisplay;
      argc += 2;
   }

   args[argc] = NULL;
  
   Tcl_ResetResult( SCM_main_interp );

   code = TkCreateFrame( (ClientData) NULL,
			 SCM_main_interp,
			 argc,
			 args,
			 1,
			 name );
   
   if( code != TCL_OK ) {
      panic( SCM_main_interp->result );
   }
  
#ifndef WIN32
   if( synchronize )
      XSynchronize( Tk_Display( Tk_MainWindow( SCM_main_interp ) ), 1 );
#endif
   
   /* Set the geometry of the main window, if requested. */
   if( *geometry ) {
      args[0] = "wm";
      args[1] = "geometry";
      args[2] = ".";
      args[3] = geometry;
      argc    = 4;

      Tk_WmCmd( Tk_MainWindow( SCM_main_interp ),
		SCM_main_interp,
		argc,
		args );
   }
   code = TkpInit( SCM_main_interp );
  
   if( code == TCL_OK )
      Tk_initialized = 1;

   /* we setup the root window */
   SCM_root_window = tcl_lookup_command( "." );

   /* Initialize commands which are now in Tcl */
   Tcl_CreateCommand( SCM_main_interp, "after",  Tcl_AfterCmd, NULL, NULL );

   /* Pixmap initialization */
   SCM_init_pixmap();
}

   

