/*---------------------------------------------------------------------*/
/*   This program is free software; you can redistribute it            */
/*   and/or modify it under the terms of the GNU General Public        */
/*   License as published by the Free Software Foundation; either      */
/*   version 2 of the License, or (at your option) any later version.  */
/*                                                                     */
/*   This program is distributed in the hope that it will be useful,   */
/*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
/*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
/*   GNU General Public License for more details.                      */
/*                                                                     */
/*   You should have received a copy of the GNU General Public         */
/*   License along with this program; if not, write to the Free        */
/*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
/*   MA 02111-1307, USA.                                               */
/*---------------------------------------------------------------------*/
/*=====================================================================*/
/*    serrano/prgm/project/bglk/runtime/Include/tclglue.h              */
/*    -------------------------------------------------------------    */
/*    Author      :  Manuel Serrano                                    */
/*    Creation    :  Tue Oct 13 15:33:18 1998                          */
/*    Last change :  Thu Apr 29 17:22:20 1999 (serrano)                */
/*    -------------------------------------------------------------    */
/*    The Bigloo/Tcl connection.                                       */
/*=====================================================================*/
#ifndef _TCLGLUE_H
#define _TCLGLUE_H
#include <tclInt.h>

/*---------------------------------------------------------------------*/
/*    importations ...                                                 */
/*---------------------------------------------------------------------*/
/*--- misc Bigloo functions -------------------------------------------*/
extern int GC_size();
extern obj_t reverse__39___r4_pairs_and_lists_6_3( obj_t );
extern obj_t append_2_18___r4_pairs_and_lists_6_3( obj_t, obj_t );
extern obj_t exact__inexact_122___r4_numbers_6_5( obj_t );
extern obj_t make_string( long, unsigned char );
extern obj_t c_constant_string_to_string( char * );
extern obj_t string_to_bstring_len( char *, int );
extern obj_t string_to_bstring();
extern obj_t string_to_keyword( char * );

/*--- __scmglue bindings ---------------------------------------------*/
/* Scheme <-> Tk converstion */
extern obj_t SCM_tk_string_to_scm_list( char * );
extern obj_t SCM_tk_string_to_scm( char * );
extern char *SCM_scm_to_tk_string( obj_t );
extern char *SCM_procedure_to_tk_string( obj_t );
extern char *SCM_stringify( char *s, int );

/* tk command */
extern obj_t tcl_lookup_command( char * );
extern obj_t tcl_call_command( obj_t, obj_t );
extern obj_t SCM_last_Tk_as_scheme(void);

/*--- tcl-var bindings ------------------------------------------------*/
/* variable handling */
extern char *SCM_tcl_getvar( char *, char *);
extern char *SCM_tcl_setvar( char *, char *, int , char * );
extern char *SCM_tcl_setvar2( char *, char *, char *, int, char * );
extern char *SCM_tcl_getvar2( char *, char *, char * );

/* variable evaluation */
extern int Tcl_GlobalEval(Tcl_Interp *interp, char *s);
extern int Tcl_Eval(Tcl_Interp *interp, char *s);

/*--- tk-main bindings ------------------------------------------------*/
/* global variable */
extern Tcl_Interp *SCM_main_interp;
extern obj_t SCM_root_window;

/*--- tclglue bindings ------------------------------------------------*/
/* misc */
extern char *Tcl_posix_error( long );
extern int SCM_valid_callback( char *, void ** );
void SCM_add_callback( char *, char *, char *, obj_t );
extern void SCM_tcl_init_callbacks();

/* initialization */
extern int tcl_init_glue();

/*--- Menu ------------------------------------------------------------*/
extern int SCM_post_cascade( char *, int, int );

/*---------------------------------------------------------------------*/
/*    Variable handling                                                */
/*---------------------------------------------------------------------*/
#define Tcl_GetVar( interp, varName, flags) \
  SCM_tcl_getvar(varName, 0L)
#define Tcl_GetVar2(interp, name1, name2, flags) \
  SCM_tcl_getvar2(name1, name2, 0L)

#define Tcl_SetVar( interp, varName, newValue, flags) \
  SCM_tcl_setvar( varName, newValue, flags, 0L)
#define Tcl_SetVar2(interp, name1, name2, newValue, flags) \
  SCM_tcl_setvar2( name1, name2, newValue, flags, 0L)

/*---------------------------------------------------------------------*/
/*    Standard TK ios                                                  */
/*---------------------------------------------------------------------*/
#define SCM_stdin  stdin
#define SCM_stdout stdout
#define SCM_stderr stderr

/*---------------------------------------------------------------------*/
/*    TK command ...                                                   */
/*---------------------------------------------------------------------*/
struct Tk_command {
   /* pointer associated to the widget command */
   ClientData ptr;
   /* Tk lib function associated to widget     */
   Tcl_CmdProc *fct;
   /* procedure to call when cmd is destroyed  */
   Tcl_CmdDeleteProc *delproc;
   /* value to pass to delproc                 */
   ClientData deldata;
   /* 1 if command has already been deleted    */
   char deleted;
   /* 1 if command is an tcl_Obj procedure     */
   char objproc;
   /* that value is used within  TK_COMMANDP   */
   obj_t marker;
   /* the associated tk-object (facultative)   */
   obj_t object;
   /* must be last field                       */
   char Id[1];		   	
};

/*---------------------------------------------------------------------*/
/*    Posix erros                                                      */
/*---------------------------------------------------------------------*/
#define Tcl_PosixError( interp )     \
   tcl_posix_error( (long)interp )

/*---------------------------------------------------------------------*/
/*    Tk obj <-> Bigloo obj machinery                                  */
/*---------------------------------------------------------------------*/
#define SCM_TO_TK_COMMAND( obj )     \
   ((struct Tk_command *)(&PROCEDURE_REF( BREF( obj ), 0 )))

#define TK_COMMANDP( obj )           \
  (PROCEDUREP( obj ) &&              \
   ((function_t)PROCEDURE_VA_ENTRY( obj ) == (function_t)(&tcl_call_command)))

#define TK_COMMAND_ID( obj )         \
   SCM_TO_TK_COMMAND( obj )->Id

#define TK_COMMAND_OBJECT( obj )  \
   SCM_TO_TK_COMMAND( obj )->object

#define TK_COMMAND_OBJECT_SET( obj, val )  \
   (SCM_TO_TK_COMMAND( obj )->object = (obj_t)val)

#define SCM_CREATE_TCL_OBJECT( obj ) \
   ((Tcl_Obj *)MAKE_CELL( obj ))

#define SCM_NIL() BNIL

#define SCM_KEYWORD_OBJ( obj )       \
  SCM_CREATE_TCL_OBJECT( string_to_keyword( obj ) )
#endif
