;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bglk/runtime/Llib/focus.scm                 */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Tue Oct 20 17:39:15 1998                          */
;*    Last change :  Fri Apr  9 09:57:43 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The focus facility                                               */
;*    -------------------------------------------------------------    */
;*    Source documentation:                                            */
;*       @path ../../manual/focus.texi@                                */
;*       @node Focus@                                                  */
;*    Example:                                                         */
;*       @path ../../demos/focus/focus.scm@                            */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_focus
   
   (include "Llib/biglook.sch")
   
   (option  (loadq "Llib/biglook.init"))
   
   (import  (__biglook_glue      "Llib/scmglue.scm")
	    (__biglook_misc      "Llib/misc.scm")
	    (__biglook_tk-object "Llib/tk-object.scm"))
   
   (export  (focus-next w)
	    (focus-prev w)
	    (focus-follows-mouse)))

;*---------------------------------------------------------------------*/
;*    Global parameters                                                */
;*---------------------------------------------------------------------*/
(define (all-children w)
   (let ((res '()))
      (for-each (lambda (x)
		   (unless (equal? x (winfo 'toplevel x))
		      ;; x is not a toplevel
		      (set! res (append res (all-children x)))))
		(winfo 'children w))
      (cons w res)))

(define (focusable? w)
   (let ((focus #t)
	 (value #f))
      
      (if (winfo 'viewable w)
	  (begin
	     ;; 1. See if this window tells something in its :takefocus option
	     (if (not (catch (set! value (tk-get w :takefocus))))
		 ;; widget has :takefocus option
		 ;; Remark: :takefocus option must be a closure. But original 
		 ;; Tcl/Tk code can define the focus action as "0" or "1".
		 ;; Those values can be hard-coded in the Tk library. So take them
		 ;; into account
		 (cond 
		    ((boolean? value) (set! focus value))
		    ((procedure? value) (set! focus (value w)))))
	     
	     ;; 2. See if the window is not disabled
	     (if focus
		 (if (not (catch (set! value (tk-get w :state))))
		     ;; widget has a :state option
		     (set! focus (not (equal? value "disabled")))))
	     
	     ;; 3. Claim that the window is focuable only if it exist some
	     ;;    keyboard binding associated to it
	     (if focus
		 (let ((p (open-output-string)))
		    (display (bind w) p)
		    (display (bind (winfo 'class w)) p)
		    (let ((s (close-output-port p)))
		       (set! focus
			     (string-case s
				((: (* all) "Key") #t)
				((: (* all) "Focus") #t)
				(else #f)))))))
	  ;; w is not visible
	  (set! focus #f))
      focus))

(define (find-next w l)
   (let* ((len   (length l))
	  (index (- len (length (member w l)))))
      
      ;; index is the position of the current widget in l
      (let loop ((i (modulo (+ index 1) len)))
	 (if (= i index)
	     ;; not found, return w
	     w
	     (let ((widget (list-ref l i)))
		(if (focusable? widget)
		    widget
		    (loop (modulo (+ i 1) len))))))))

  
(define focus-next 
   (lambda (w)
      (let ((all (all-children (winfo 'toplevel w))))
	 (if (= (length all) 1)
	     (car all)
	     (find-next w all)))))

(define focus-prev
   (lambda (w)
      (let ((all (all-children (winfo 'toplevel w))))
	 (if (= (length all) 1)
	     (car all)
	     (find-next w (reverse all))))))

(define (focus-follows-mouse)
   (add-binding! "all" "<Enter>" (WW d)
		 (if (or (equal? d "NotifyAncestor")
			 (equal? d "NotifyNonlinear")
			 (equal? d "NotifyInferior"))
		     (focus WW))))


 


