;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bglk/runtime/Llib/font-chooser.scm          */
;*    -------------------------------------------------------------    */
;*    Author      :  Erick Gallesio [eg@unice.fr]                      */
;*    Creation    :  Thu Apr 29 17:18:16 1999                          */
;*    Last change :  Fri May 14 15:32:35 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    Font chooser                                                     */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_font-chooser
   
   (include "Llib/biglook.sch")
   
   (import  (__biglook_glue      "Llib/scmglue.scm")
	    (__biglook_misc      "Llib/misc.scm")
	    (__biglook_key       "Llib/key.scm")
	    (__biglook_tk-object "Llib/tk-object.scm")
	    (__biglook_frame     "Lwidget/frame.scm")
	    (__biglook_entry     "Lwidget/entry.scm")
	    (__biglook_button    "Lwidget/button.scm")
	    (__biglook_scrollbar "Lwidget/scrollbar.scm")
	    (__biglook_listbox   "Lwidget/listbox.scm")
	    (__biglook_combobox  "Lwidget/combobox.scm"))
   
   (export  (make-font-chooser . fnt)))

;*---------------------------------------------------------------------*/
;*    font ...                                                         */
;*---------------------------------------------------------------------*/
(define font (tk-lookup-command "font"))

;*---------------------------------------------------------------------*/
;*    Global variables                                                 */
;*---------------------------------------------------------------------*/
(define *fc:font*      	 #f)	;;; the prototype font 
(key-value-set! "fc:lock" #f)   ;;; to grab the window while choosing font

;*---------------------------------------------------------------------*/
;*    Utilities                                                        */
;*---------------------------------------------------------------------*/
(define (font-wait-result chooser)
   (let ((cur-grab (grab 'current chooser))
	 (old-font (font 'actual *fc:font*))
	 (pretty   (lambda ()
		      (append 
		       (list (font 'conf *fc:font* :family)
			     (font 'conf *fc:font* :size)
			     (string->symbol (font 'conf *fc:font* :weight))
			     (string->symbol (font 'conf *fc:font* :slant)))
		       (if (font 'conf *fc:font* :underline)
			   '(underline)
			   '())
		       (if (font 'conf *fc:font* :overstrike)
			   '(overstrike)
			   '())))))
      (tkwait 'visibility chooser)
      (grab 'set chooser) 
      (tkwait 'variable "fc:lock") 
      (and cur-grab (grab 'set cur-grab))
      
      ;; Compute result
      (case (key-value "fc:lock")
	 ((ok)      (destroy chooser) 
		    ;; return a pretty result
		    (pretty))
	 ((cancel)  (destroy chooser) 
		    ;; restore the font we have when entering the widget
		    (apply font 'configure *fc:font* old-font)
		    #f))))


(define (%make-font-chooser fnt)
  ;;
  ;; Some utilities
  ;;
  (define (toggle-weight)
    (font 'configure *fc:font* :weight
	  (if (equal? (font 'conf *fc:font* :weight) "normal")
	      "bold"
	      "normal")))

  (define (toggle-slant)
    (font 'configure *fc:font* :slant
	  (if (equal? (font 'conf *fc:font* :slant) "roman")
	      "italic"
	      "roman")))

  (define (toggle-underline)
    (font 'configure *fc:font* :underline
	  (not (font 'configure *fc:font* :underline))))

  (define (toggle-overstrike)
    (font 'configure *fc:font* :overstrike
	  (not (font 'configure *fc:font* :overstrike))))

  ;;
  ;; Top frame building
  ;;
  (define (make-top-frame parent)
    (let* ((fonts   (sort (font 'families) string<?))
	   (f       (instantiate::frame
		       (parent parent)
		       (relief "groove")
		       (border-width 2)))
	   (family  (instantiate::label
		       (parent f)
		       (text "Font Family:")))
	   (choice1 (instantiate::combobox
		       (parent f)
		       (values fonts)
		       (value (font 'configure *fc:font* :family))
		       (command (lambda (v) 
				   (font 'conf *fc:font* :family v)))))
	   (size    (instantiate::label
		       (parent f)
		       (text "Font Size:")))
	   (choice2 (instantiate::combobox
		       (parent f)
		       (values '(8 10 12 14 20 24 36 48))
		       (width 3)
		       (string-value #f)
		       (value (font 'configure *fc:font* :size))
		       (command (lambda (v) 
				   (font 'conf *fc:font* :size v)))))
	   (bold    (instantiate::check-button
		       (parent f)
		       (text "B")
		       (width 3)
		       (font (font 'create :weight 'bold))
		       (indicator-on #f)
		       (command toggle-weight)))
	   (italic  (instantiate::check-button
		       (parent f)
		       (text "i")
		       (width 3)
		       (font (font 'create :slant 'italic))
		       (indicator-on #f)
		       (command toggle-slant)))
	   (under   (instantiate::check-button
		       (parent f)
		       (text "U")
		       (width 3)
		       (font (font 'create :underline #t))
		       (indicator-on #f )
		       (command toggle-underline)))
	   (over    (instantiate::check-button
		       (parent f)
		       (text "O")
		       (width 3)
		       (font (font 'create :overstrike #t))
		       (indicator-on #f)
		       (command toggle-overstrike))))

      ;; See the  buttons that must be toggled
      (if (equal? (font 'conf *fc:font* :weight) "bold")   (toggle-weight))
      (if (equal? (font 'conf *fc:font* :slant)  "italic") (toggle-slant))
      (if (font 'conf *fc:font* :underline)		   (toggle-underline))
      (if (font 'conf *fc:font* :overstrike) 		   (toggle-overstrike))


      ;; Change binding of "Font size" box  to allow direct manipulation
      (bind (combobox-entry choice2) "<Return>" 
	    (lambda () (font 'conf *fc:font* :size (combobox-value choice2))))


      ;; disable family combobox
      (combobox-state-set! choice1 "disabled")
      
      ;; Pack everybody
      (pack family choice1 size choice2 :side 'left :padx 2)
      (pack bold under italic over :fill 'y :side 'left :padx 2 :pady 2)
      
      f))

  ;;;
  ;;; Center part of the widget (the sample)
  ;;;
  (define (make-sample parent)
    (instantiate::label
       (parent parent)
       (font *fc:font*)
       (text (string-append #"ABCDEFGHIJKLMNOPQRSTUVWXYZ\n"
			    #"abcdefghijklmnopqrstuvwxyz\n"
			    #"0123456789~`!@#$%^&*()_-+=\n"
			    "{}[]:;\"'<>,.?/"))))


  ;;;
  ;;; Bottom part of the widget (the closing buttons)
  ;;;
  (define (make-buttons parent) 
    (let* ((f      (instantiate::frame
		      (parent parent)
		      (relief "groove")
		      (border-width 2)))
	   (sel    (instantiate::button
		      (parent f)
		      (text "Select")
		      (command (lambda () (key-value-set! "fc:lock" 'ok)))))
	   (cancel (instantiate::button
		      (parent f)
		      (text "Cancel")
		      (command (lambda () (key-value-set! "fc:lock" 'cancel))))))
      (wm 'protocol parent "WM_DELETE_WINDOW"
	  (lambda() (key-value-set! "fc:lock" 'cancel)))
      (pack sel cancel :side 'left :padx 2 :pady 2)
      f))
  
  ;;; 
  ;;; Start of %make-font-chooser
  ;;;
  (let* ((t   (instantiate::top-level
		 (title "Font chooser ...")
		 (class "FontChooser")
		 (geometry "500x300")))

	 (top (make-top-frame t))
	 (txt (make-sample t))
	 (but (make-buttons t)))
    ; The internal frame
    (pack top :fill 'x :expand #f)
    (pack txt :fill 'both :expand #t)
    (pack but :fill 'x :expand #f :side 'bottom)
    t))

;=============================================================================
;
; make-font-chooser (the only exported procedure)
;
;=============================================================================
(define (make-font-chooser . fnt)
   (if (not *fc:font*)
       (begin
	  ;; If this is the first call to this function.
	  ;; Create the prototype font
	  ;; with a plausible font
	  (let ((tmp (instantiate::button)))
	     (set! *fc:font*
		   (apply font 'create (font 'actual (button-font tmp))))
	     (destroy tmp))))
   
   (if (not (null? fnt))
       (apply font 'configure *fc:font* (font 'actual (car fnt))))
   
   ;; Call the chooser box
   (font-wait-result (%make-font-chooser *fc:font*)))
