;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bglk/runtime/Llib/font.scm                  */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Fri May 14 15:04:13 1999                          */
;*    Last change :  Fri Jun 18 11:51:55 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    Font management                                                  */
;*    -------------------------------------------------------------    */
;*    Source documentation:                                            */
;*       @path ../../manual/font.texi@                                 */
;*       @node Font@                                                   */
;*    Example:                                                         */
;*       @path ../../demos/font/font.scm@                              */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_font

   (include "Llib/biglook.sch")
   
   (option  (loadq "Llib/biglook.init"))
   
   (import  (__biglook_glue "Llib/scmglue.scm")
	    (__biglook_misc "Llib/misc.scm"))
   
   (export  ;; font
            (class font
	       ;;intializer
	       (font-init)
	       ;; id 
	       (id read-only (default (symbol->string (gensym))))
	       ;; value
	       (value
		(get (lambda (o)
			(with-access::font o (id)
			   (append 
			    (list (tk-font 'conf id :family)
				  (tk-font 'conf id :size)
				  (string->symbol (tk-font 'conf id :weight))
				  (string->symbol (tk-font 'conf id :slant)))
			    (if (tk-font 'conf id :underline)
				'(underline)
				'())
			    (if (tk-font 'conf id :overstrike)
				'(overstrike)
				'())))))
		(set (lambda (o v)
			(with-access::font o (id)
			   (apply tk-font 'configure id (tk-font 'actual v))
			   o))))
	       ;; Font virtual slots
	       (family
		(get (lambda (o)
			(tk-font 'configure (font-id o) :family)))
		(set (lambda (o v)
			(tk-font 'configure (font-id o) :family v))))
	       (size
		(get (lambda (o)
			(tk-font 'configure (font-id o) :size)))
		(set (lambda (o v)
			(tk-font 'configure (font-id o) :size v))))
	       (weight
		(get (lambda (o)
			(tk-font 'configure (font-id o) :weight)))
		(set (lambda (o v)
			(tk-font 'configure (font-id o) :weight v))))
	       (slant
		(get (lambda (o)
			(tk-font 'configure (font-id o) :slant)))
		(set (lambda (o v)
			(tk-font 'configure (font-id o) :slant v))))
	       (underline
		(get (lambda (o)
			(tk-font 'configure (font-id o) :underline)))
		(set (lambda (o v)
			(tk-font 'configure (font-id o) :underline v))))
	       (overstrike
		(get (lambda (o)
			(tk-font 'configure (font-id o) :overstrike)))
		(set (lambda (o v)
			(tk-font 'configure (font-id o) :overstrike v)))))

	    (font-init ::font)

	    (measure ::font ::bstring . window)
	    (get-all-font-names)))

;*---------------------------------------------------------------------*/
;*    tk-font ...                                                      */
;*---------------------------------------------------------------------*/
(define tk-font (tk-lookup-command "font"))

;*---------------------------------------------------------------------*/
;*    font-init ...                                                    */
;*---------------------------------------------------------------------*/
(define (font-init font::font)
   (with-access::font font (id)
      (tk-font 'create id)))

;*---------------------------------------------------------------------*/
;*    measure ...                                                      */
;*    -------------------------------------------------------------    */
;*    Measures the amount of spaece the string TEXT would use in       */
;*    given FONT.                                                      */
;*---------------------------------------------------------------------*/
(define (measure font::font text::bstring . window)
   (if (pair? window)
       (tk-font 'measure font text :displayof (car window))
       (tk-font 'measure font text)))

;*---------------------------------------------------------------------*/
;*    get-all-font-names ...                                           */
;*    -------------------------------------------------------------    */
;*    Returns the list of all fonts currently in use.                  */
;*---------------------------------------------------------------------*/
(define (get-all-font-names)
   (tk-font 'names))
