;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bglk/runtime/Llib/image.scm                 */
;*    -------------------------------------------------------------    */
;*    Author      :  Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>      */
;*    Creation    :  15-Oct-1996 14:25                                 */
;*    Last change :  Fri Jun  4 20:11:25 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    image.stk -- Images functions for Biglook                        */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_image
   
   (include "Llib/biglook.sch")
   
   (option  (loadq "Llib/biglook.init"))
   
   (import  (__biglook_glue      "Llib/scmglue.scm")
	    (__biglook_misc      "Llib/misc.scm")
	    (__biglook_tk-object "Llib/tk-object.scm"))

   (extern  (include "biglook_cfg.h")
	    (macro library-directory::string "BIGLOOK_LIBRARY_DIRECTORY"))
   
   (export  (class image::tk-object
	       (image-init)
 	       (file::bstring read-only)
	       (type
		(get (lambda (o) (tk-image 'type (image-id o))))
		read-only)
	       (width
		(get (lambda (o) (tk-image 'width (image-id o))))
		read-only)
	       (height
		(get (lambda (o) (tk-image 'height (image-id o))))
		read-only))
	    
	    (image-init   ::image)
	    (find-image   ::bstring)
	    (change-image ::bstring . opt)
	    (delete-image ::bstring)
	    tk-image
	    
	    *image-path*))

;*---------------------------------------------------------------------*/
;*    image-init ...                                                   */
;*---------------------------------------------------------------------*/
(define (image-init o)
   (with-access::image o (id eid file)
      (let ((img (internal-make-image file)))
	 (set! eid img)
	 (set! id img))))

;*---------------------------------------------------------------------*/
;*    find-image ...                                                   */
;*---------------------------------------------------------------------*/
(define (find-image key)
   (let ((cell (get-hash key image-cache)))
      (if (pair? cell)
	  (cdr cell)
	  #f)))
  
;*---------------------------------------------------------------------*/
;*    change-image ...                                                 */
;*---------------------------------------------------------------------*/
(define (change-image key . opt)
   (let ((img (find-image key)))
      (if img
	  (create key opt)
	  (error "change-image" "no image with name" key)))))

;*---------------------------------------------------------------------*/
;*    delete-image ...                                                 */
;*---------------------------------------------------------------------*/
(define (delete-image key)
   (let ((img (find-image key)))
      (if img
	  (begin
	     (image 'delete key)
	     (rem-key-hash! key image-cache))
	  (error "delete-image" "no image with name" key))))

;*---------------------------------------------------------------------*/
;*    tk-image ...                                                     */
;*---------------------------------------------------------------------*/
(define tk-image (tk-lookup-command "image"))

;*---------------------------------------------------------------------*/
;*    *image-path* ...                                                 */
;*---------------------------------------------------------------------*/
(define *image-path*
   (cons* ""
	  "."
	  (make-file-name library-directory "images")
	  (build-path-from-shell-variable "BGLK_IMAGE_PATH")))

;*---------------------------------------------------------------------*/
;*    build-path-from-shell-variable ...                               */
;*---------------------------------------------------------------------*/
(define (build-path-from-shell-variable var)
   (let ((val (getenv var)))
      (if (string? val)
	  (string-case val
	     ((+ (out #\:))
	      (let* ((str (the-string))
		     (res (ignore)))
		 (cons str res)))
	     (#\:
	      (ignore))
	     (else
	      '()))
	  '())))

;*---------------------------------------------------------------------*/
;*    image-cache ...                                                  */
;*    -------------------------------------------------------------    */
;*    Images are cached in a local cache variable. Each image is       */
;*    given a key so that multple use of the same image share the      */
;*    same copy                                                        */
;*                                                                     */
;*---------------------------------------------------------------------*/
(define image-cache
   (make-hash-table 1024
		    (lambda (o) (string->0..2^x-1 o 10))
		    (lambda (x) (car x))
		    string=?
		    64))

;*---------------------------------------------------------------------*/
;*    mk-image ...                                                     */
;*---------------------------------------------------------------------*/
(define (mk-image nape opt)
   (let ((img #f))
      ;; Try to create a bitmap then a pixmap else a photo image.
      (and (catch (set! img (apply tk-image 'cre 'photo  opt)))
	   (catch (set! img (apply tk-image 'cre 'pixmap opt)))
	   (catch (set! img (apply tk-image 'cre 'bitmap opt))))
      (if (string? img)
	  (tk-lookup-command img)
	  #f)))

;*---------------------------------------------------------------------*/
;*    create ...                                                       */
;*---------------------------------------------------------------------*/
(define (create key opt)
   (let ((img #f))
      (if (null? opt)
	  ;; No option given. Take the key as the file name
	  (let loop ((l *image-path*))
	     (if (null? l)
		 (set! img (mk-image key (list :data key)))
		 (let ((f (string-append (car l) "/" key)))
		    (if (file-exists? f)
			(set! img (mk-image key (list :file f))))
		    (unless img (loop (cdr l))))))
	  ;. Use option given
	  (set! img (mk-image key opt)))
      (when img
	 ;; Found an image; Register it
	 (put-hash! (cons key img) image-cache)
	 img)))

;*---------------------------------------------------------------------*/
;*    internal-make-image ...                                          */
;*---------------------------------------------------------------------*/
(define (internal-make-image key . opt)
   (let ((img (or (find-image key) (create key opt))))
      (if (not (tk-command? img))
	  (error "make-image" "Can't make image" key)
	  img)))




