;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bglk/runtime/Llib/misc.scm                  */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Sat Oct 17 06:37:21 1998                          */
;*    Last change :  Fri Jun  4 16:55:24 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The Tk bindings                                                  */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_misc
   
   (include "Llib/biglook.sch")
   
   (option  (loadq "Llib/biglook.init"))
   
   (import (__biglook_glue  "Llib/scmglue.scm")
	   (__biglook_focus "Llib/focus.scm"))

   (use    (__biglook_tk-object "Llib/tk-object.scm"))

   (extern (include "biglook_cfg.h")
	   (macro _biglook-version::string   "BIGLOOK_VERSION")
	   (macro _biglook-exit::int (::int) "biglook_exit")
	   (export procedure->tk-string "SCM_procedure_to_tk_string"))

   (export (biglook-exit ::int)
	   (biglook-args? ::bstring)
	   the-biglook-args-parse-usage
	   (biglook-args-parse-usage ::bool)
	   *biglook-version*
	   *tk-strict-motif*
	   *after-id*
	   *x*
	   *y*
	   (cancel-repeat)
	   (tab-to-window w)
	   (make-widget-constructor::procedure ::bstring)
           after
	   bell
	   bindtags
	   clipboard
	   destroy
	   event
	   focus
	   grab
	   grid
	   option
	   pack
	   place
	   selection
	   tk-old
	   tkwait
	   update
	   winfo
	   wm
	   (root)
	   (procedure->tk-string::string ::procedure)))

;*---------------------------------------------------------------------*/
;*    biglook-exit ...                                                 */
;*---------------------------------------------------------------------*/
(define (biglook-exit status)
   (_biglook-exit status))

;*---------------------------------------------------------------------*/
;*    biglook-args? ...                                                */
;*    -------------------------------------------------------------    */
;*    The list of command line options is taken from the regular       */
;*    command line option parsing.                                     */
;*    -------------------------------------------------------------    */
;*    @label Biglook command line args@                                */
;*    @ref scmglue.scm:Biglook command line args@                      */
;*---------------------------------------------------------------------*/
(define (biglook-args? arg)
   (member arg
	   '("-help" "-name" "-display" "-geometry" "-sync" "-no-event-loop")))

;*---------------------------------------------------------------------*/
;*    the-biglook-args-parse-usage ...                                 */
;*---------------------------------------------------------------------*/
(define the-biglook-args-parse-usage the-biglook-args-parse-usage)

;*---------------------------------------------------------------------*/
;*    biglook-args-parse-usage ...                                     */
;*---------------------------------------------------------------------*/
(define (biglook-args-parse-usage opt)
   (if (procedure? the-biglook-args-parse-usage)
       (the-biglook-args-parse-usage opt)
       opt))
	 
;*---------------------------------------------------------------------*/
;*    Global parameters ...                                            */
;*---------------------------------------------------------------------*/
(define *biglook-version* _biglook-version)
(define *tk-strict-motif* #t)
(define *after-id*        "")
(define *x*               0)
(define *y*               0)

;*---------------------------------------------------------------------*/
;*    cancel-repeat ...                                                */
;*    -------------------------------------------------------------    */
;*    This procedure is invoked to cancel an auto-repeat action        */
;*    described by *after-id*. It's used by several widgets to         */
;*    auto-scroll the widget when the mouse is dragged out of the      */
;*    widget with a button pressed.                                    */
;*---------------------------------------------------------------------*/
(define (cancel-repeat)
   (after 'cancel *after-id*)
   (set! *after-id* ""))

;*---------------------------------------------------------------------*/
;*    tab-to-window ...                                                */
;*---------------------------------------------------------------------*/
;; This procedure moves the focus to the given widget.  If the widget
;; is an entry, it selects the entire contents of the widget.
(define (tab-to-window w)
   (if (string=? (winfo 'class w) "Entry")
       (begin
	  (w 'select 'range 0 'end)
	  (w 'icur 'end)))
   (focus w))

;*---------------------------------------------------------------------*/
;*    parse-widget-args ...                                            */
;*---------------------------------------------------------------------*/
(define parse-widget-args
   (let ((count 0))
      (lambda (name args)
	 (define (make-widget-name prefix)
	    (set! count (+fx count 1))
	    (string-append prefix "." name (integer->string count)))
	 (let loop ((largs    args)
		    (pre-args '()))
	    (if (null? largs)
		(values (make-widget-name "") (reverse! pre-args))
		(match-case largs
		   ((:parent ?parent . ?rest)
		    (values (make-widget-name (tk-command-id parent))
			    (append (reverse! pre-args) rest)))
		   ((:id ?id . ?rest)
		    (values id
			    (append (reverse! pre-args) rest)))
		   ((?key ?val . ?rest)
		    (loop rest (cons* val key pre-args)))
		   (else
		    (error "parse-widget-args" "Illegal arguments" args))))))))
   
;*---------------------------------------------------------------------*/
;*    make-widget-constructor ...                                      */
;*---------------------------------------------------------------------*/
(define (make-widget-constructor widget)
   (let ((proc (tk-lookup-command widget)))
      (lambda _args
	 (multiple-value-bind (name args)
	    (parse-widget-args widget _args)
	    (tk-lookup-command
	     (string-downcase (symbol->string (apply proc name args))))))))
       
;*---------------------------------------------------------------------*/
;*    Global Tk bindings ...                                           */
;*---------------------------------------------------------------------*/
(define after        (tk-lookup-command "after"))
(define bell         (tk-lookup-command "bell"))
;; bind (see tk-object.scm)
(define bindtags     (tk-lookup-command "bindtags"))
(define clipboard    (tk-lookup-command "clipboard"))
(define destroy      (tk-lookup-command "destroy"))
(define event        (tk-lookup-command "event"))
(define focus        (tk-lookup-command "focus"))
;; font (see font.scm)
(define grab         (tk-lookup-command "grab"))
(define grid         (tk-lookup-command "grid"))
;; image (see image.scm)
;; lower (see lower.scm)
(define option       (tk-lookup-command "option"))
(define pack         (tk-lookup-command "pack"))
(define place        (tk-lookup-command "place"))
;; raise (see tk-object.scm)
(define selection    (tk-lookup-command "selection"))
(define tk-old       (tk-lookup-command "tk"))
(define tkwait       (tk-lookup-command "tkwait"))
(define update       (tk-lookup-command "update"))
(define winfo        (tk-lookup-command "winfo"))
(define wm           (tk-lookup-command "wm"))
;; button (see button.scm)
;; canvas (see canvas.scm)
;; check-button (see button.scm)
;; entry (see entry.scm)
;; frame (see frame.scm)
;; label (see label.scm)
;; list-box (see listbox.scm)
;; menu (see menu.scm)
;; menu-button (see menu.scm)
;; message (see message.scm)
;; radio-button (see button.scm)
;; scale (see scale.scm)
;; scrollbar (see scrollbar.scm)
;; text (see text.scm)
;; top-level (see frame.scm)

;*---------------------------------------------------------------------*/
;*    root ...                                                         */
;*---------------------------------------------------------------------*/
(define (root)
   *root*)
   
;*---------------------------------------------------------------------*/
;*    procedure->tk-string ...                                         */
;*---------------------------------------------------------------------*/
(define (procedure->tk-string::string obj::procedure)
   (let ((res (make-string 10)))
      (sprintf res "#p%lx" obj)
      res))

;*---------------------------------------------------------------------*/
;*    Default bindings                                                 */
;*---------------------------------------------------------------------*/
;; Keyboard traversal
(add-binding! "all" "<Tab>" 	  (WW) (tab-to-window (focus-next WW)))
(add-binding! "all" "<Shift-Tab>" (WW) (tab-to-window (focus-prev WW)))

;; Cut/Copy/Paste virtual events
(event 'add "<<Cut>>"   "<Control-Key-x>" "<Key-F20>")
(event 'add "<<Copy>>"  "<Control-Key-c>" "<Key-F16>")
(event 'add "<<Paste>>" "<Control-Key-v>" "<Key-F18>")
(event 'add "<<PasteSelection>>" "<ButtonRelease-2>" )

