;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bglk/runtime/Llib/palette.scm               */
;*    -------------------------------------------------------------    */
;*    Author      :  Erick Gallesio  - I3S-CNRS/ESSI <eg@unice.fr>     */
;*    Creation    :  4-Aug-1995 11:24                                  */
;*    Last change :  Fri Jun 18 12:23:35 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    Palette -- procedures that change the color palette used by Tk   */
;*    -------------------------------------------------------------    */
;*    Source documentations:                                           */
;*       @path ../../manual/palette.texi@                              */
;*       @node Palette@                                                */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_palette
   
   (include "Llib/biglook.sch")
   
   (extern  (macro sprintf-color::int (::string ::string ::int ::int ::int)
		   "sprintf"))
   
   (option  (loadq "Llib/biglook.init"))
   
   (import  (__biglook_glue      "Llib/scmglue.scm")
	    (__biglook_misc      "Llib/misc.scm")
	    (__biglook_key       "Llib/key.scm")
	    (__biglook_tk-object "Llib/tk-object.scm")
	    (__biglook_button    "Lwidget/button.scm")
	    (__biglook_entry     "Lwidget/entry.scm")
	    (__biglook_frame     "Lwidget/frame.scm")
	    (__biglook_scrollbar "Lwidget/scrollbar.scm"))
   
   (export  (set-palette! . args)
	    (set-palette-bisque!)
	    (make-rgb-color r::int g::int b::int)
	    (make-lighter-color color)
	    (make-darker-color color)))
	     
;*---------------------------------------------------------------------*/
;*    dotimes ...                                                      */
;*---------------------------------------------------------------------*/
(define-macro (dotimes binding . body)
   (if (list? binding)
       ;; binding is a list
       (let ((var   #f) (count #f) (result #f))
	  (case (length binding)
	     ((2)  (set! var    (car binding))
		   (set! count  (cadr binding)))
	     ((3)  (set! var    (car binding))
		   (set! count  (cadr binding))
		   (set! result (caddr binding)))
	     (else (error "dotimes" "bad binding construct" binding)))
	  `(do ((,var 0 (+ ,var 1)))
		 ((= ,var ,count) ,result)
	      ,@body))
       ;; binding is ill-formed
       (error "dotimes" "binding is not a list: " binding)))

;;
;; make-lighter-color
;;
(define (make-lighter-color color)
   ;; Make a lighter color: to do this, round each color component
   ;; up by 15% or 1/3 of the way to full white, whichever is greater.
   (let ((rgb   (winfo 'rgb *root* color))
	 (light (make-vector 3)))
      (dotimes (i 3)
	       (let* ((c    (/ (list-ref rgb i) 256))
		      (inc1 (* c 0.15))
		      (inc2 (/ (- 255 c) 3)))
		  (set! c (+ c (max inc1 inc2)))
		  (vector-set! light i (inexact->exact (min c 255)))))
      (make-rgb-color (vector-ref light 0) 
		      (vector-ref light 1) 
		      (vector-ref light 2))))

;;
;; make-darker-color
;;
(define (make-darker-color color)
   (let ((rgb (winfo 'rgb *root* color)))
      (make-rgb-color (/fx (* 9 (list-ref rgb 0)) 2560)
		      (/fx (* 9 (list-ref rgb 1)) 2560)
		      (/fx (* 9 (list-ref rgb 2)) 2560))))

 
;*---------------------------------------------------------------------*/
;*    palette hash tables ...                                          */
;*---------------------------------------------------------------------*/
(define (make-palette-hash-table)
   (list 'palette-hash-table))

(define (palette-hash-table? table)
   (and (pair? table) (eq? (car table) 'palette-hash-table)))

(define (hash-table-for-each table fun)
   (for-each (lambda (cell)
		(fun (car cell) (cdr cell)))
	     (cdr table)))

(define (hash-table-put! table key value)
   (let ((old (assoc key (cdr table))))
      (if (pair? old)
	  (set-cdr! old value)
	  (set-cdr! table (cons (cons key value) (cdr table))))))

(define (hash-table-get table key . default)
   (let ((old (assoc key (cdr table))))
      (cond
	 ((pair? old)
	  (cdr old))
	 ((pair? default)
	  (car default))
	 (else
	  (error "hash-table-get" "Unbound key" (cons key table))))))

;; Tk palette hash table
(define tk-palette #f)

(define (make-rgb-color r g b)
   (let ((new-string (make-string 7)))
      (sprintf-color new-string
		     "#%02x%02x%02x"
		     (min (inexact->exact r) 255)
		     (min (inexact->exact g) 255)
		     (min (inexact->exact b) 255))
      new-string))
   
(define (Tk-recolor-tree w colors)
   (hash-table-for-each colors
			(lambda (db-opt db-value)
			   (let ((opt (string->keyword
				       (string-append
					":"
					(string-upcase db-opt))))
				 (val #f))
			      (unless (catch (set! val (tk-get w opt)))
				 (if (equal? val
					     (hash-table-get tk-palette
							     db-opt))
				     (tk-set! w opt db-value))))))
   
   ;; Do the same job for all the children
   (for-each (lambda (child) (Tk-recolor-tree child colors))
	     (winfo 'children w)))

;*---------------------------------------------------------------------*/
;*    set-palette! ...                                                 */
;*    -------------------------------------------------------------    */
;*    Changes the default color scheme for a Tk application by setting */
;*    default colors in the option database and by modifying all of    */
;*    the color options for existing widgets that have the default     */
;*    value.                                                           */
;*                                                                     */
;*    The arguments consist of either a single color name, which       */
;*    will be used as the new background color (all other colors will  */
;*    be computed from this) or an even number of values consisting of */
;*    option names and values.  The name for an option is the one used */
;*    for the option database, such as activeForeground, not           */
;*    -activeforeground.                                               */
;*---------------------------------------------------------------------*/
(define (set-palette! . args)
   (let ((new (make-palette-hash-table)))
      ;; Create a hash table that has the complete new palette.
      ;; If some colors aren't specified, compute them from
      ;; other colors that are specified.
      (if (= (length args) 1)
	  (hash-table-put! new "background" (car args))
	  ;; place all given colors in the "new" hashatable
	  (for-each (lambda (x) (hash-table-put! new (car x) (cadr x)))
		    args))
      
      (unless (hash-table-get new "background" #f)
	 (error "palette" "you must specify a background color" new))
      
      (unless (hash-table-get new "foreground" #f)
	 (hash-table-put! new "foreground" "black"))
      
      (let* ((back      (hash-table-get new "background"))
	     (fore      (hash-table-get new "foreground"))
	     (bg        (winfo 'rgb *root* back))
	     (fg        (winfo 'rgb *root* fore))
	     (darker-bg (make-rgb-color (/fx (* 9 (list-ref bg 0)) 2560)
					(/fx (* 9 (list-ref bg 1)) 2560)
					(/fx (* 9 (list-ref bg 2)) 2560))))
	 (for-each (lambda (x)
		      (unless (hash-table-get new x #f)
			 (hash-table-put! new x fore)))
		   (list "activeForeground" "insertBackground" 
			 "selectForeground" "highlightColor"))
	 
	 (unless (hash-table-get new "disabledForeground" #f)
	    (hash-table-put! new 
			     "disabledForeground" 
			     (make-rgb-color
			      (+fx (* 3 (car   bg)) (/fx (car   fg) 1024))
			      (+fx (* 3 (cadr  bg)) (/fx (cadr  fg) 1024))
			      (+fx (* 3 (caddr bg)) (/fx (caddr fg) 1024)))))
	 
	 (unless (hash-table-get new "highlightBackground" #f)
	    (hash-table-put! new "highlightBackground" back))
	 
	 (unless (hash-table-get new "activeBackground" #f)
	    ;; Pick a default active background that is lighter than the
	    ;; normal background.  To do this, round each color component
	    ;; up by 15% or 1/3 of the way to full white, whichever is
	    ;; greater.
	    
	    (let ((light (make-vector 3)))
	       (dotimes (i 3)
			(let* ((c    (/ (list-ref bg i) 256))
			       (inc1 (* c 0.15))
			       (inc2 (/ (- 255 c) 3)))
			   (set! c (+ c (max inc1 inc2)))
			   (vector-set! light i (min c 255))))
	       (hash-table-put! new 
				"activeBackground" 
				(make-rgb-color (vector-ref light 0)
						(vector-ref light 1)
						(vector-ref light 2)))))
	 
	 (unless (hash-table-get new "selectBackground" #f)
	    (hash-table-put! new "selectBackground" darker-bg))
	 
	 (unless (hash-table-get new "troughColor" #f)
	    (hash-table-put! new "troughColor" darker-bg))
	 
	 (unless (hash-table-get new "selectColor" #f)
	    (hash-table-put! new "selectColor" "#b03060"))
	 
	 ;; Walk the widget hierarchy, recoloring all existing windows.
	 ;; Before doing this, make sure that the tk-palette variable holds
	 ;; the default values of all options, so that Tk-recolor-tree can
	 ;; be sure to only change options that have their default values.
	 ;; If the variable exists, then it is already correct
	 ;; (it was created the last time this procedure was invoked).
	 ;; If the variable doesn't exist, fill it in using the
	 ;; defaults from a few widgets.
	 
	 (unless (palette-hash-table? tk-palette)
	    (let ((c (instantiate::check-button))
		  (e (instantiate::entry))
		  (s (instantiate::scrollbar)))
	       
	       (set! tk-palette (make-palette-hash-table))
	       
	       (hash-table-put! tk-palette "activeBackground"
				(cadddr (tk c 'configure
					    :activebackground)))
	       (hash-table-put! tk-palette "activeForeground"
				(cadddr (tk c 'configure
					    :activeforeground)))
	       (hash-table-put! tk-palette "background"
				(cadddr (tk c 'configure
					    :background)))
	       (hash-table-put! tk-palette "disabledForeground"
				(cadddr (tk c 'configure
					    :disabledforeground)))
	       (hash-table-put! tk-palette "foreground"
				(cadddr (tk c 'configure
					    :foreground)))
	       (hash-table-put! tk-palette "highlightBackground"
				(cadddr (tk c 'configure
					    :highlightbackground)))
	       (hash-table-put! tk-palette "highlightColor"
				(cadddr (tk c 'configure
					    :highlightcolor)))
	       (hash-table-put! tk-palette "insertBackground"
				(cadddr (tk e 'configure
					    :insertbackground)))
	       (hash-table-put! tk-palette "selectColor"
				(cadddr (tk c 'configure
					    :selectcolor)))
	       (hash-table-put! tk-palette "selectBackground"
				(cadddr (tk e 'configure
					    :selectbackground)))
	       (hash-table-put! tk-palette "selectForeground"
				(cadddr (tk e 'configure
					    :selectforeground)))
	       (hash-table-put! tk-palette "troughColor"
				(cadddr (tk s 'configure
					    :troughcolor)))
	       
	       ;; Destroy temporary widgets
	       (destroy c e s)))
	 
	 (Tk-recolor-tree *root* new)
	 
	 ;; Change the option database so that future windows will get the
	 ;; same colors. Save the options in the global variable
	 ;; tk-palette, for use the next time we change the options. 
	 (hash-table-for-each new (lambda (x y)
				     (option 'add
					     (string-append "*" x)
					     (hash-table-get new x))
				     (hash-table-put! tk-palette x y))))))

;*---------------------------------------------------------------------*/
;*    set-palette-bisque! ...                                          */
;*    -------------------------------------------------------------    */
;*    Reset the Tk color palette to the old "bisque" colors.           */
;*---------------------------------------------------------------------*/
(define (set-palette-bisque!)
   (set-palette! '("activeBackground"    "#e6ceb1")
		 '("activeForeground"    "black")
		 '("background" 	   "#ffe4c4")
		 '("disabledForeground"  "#b0b0b0")
		 '("foreground" 	   "black")
		 '("highlightBackground" "#ffe4c4")
		 '("highlightColor" 	   "black")
		 '("insertBackground"    "black")
		 '("selectColor" 	   "#b03060")
		 '("selectBackground"    "#e6ceb1")
		 '("selectForeground"    "black")
		 '("troughColor" 	   "#cdb79e")))
