;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bglk/runtime/Llib/scmglue.scm               */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Fri Oct 16 13:40:51 1998                          */
;*    Last change :  Fri May 14 18:15:05 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The connection to TK. This is the Scheme part of the connection, */
;*    that is, to a large extent, Scheme library functions             */
;*    implementing the connection.                                     */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_glue
   
   (include "Llib/args.sch"
	    "Llib/biglook.sch")
   
   (extern (include "tclglue.h")
	   (include "tcl.h")
	   
	   ;; C importations
	   (macro tk-command?           (::obj)                "TK_COMMANDP")
	   (macro tk-command-id::string (::obj)                "TK_COMMAND_ID")
	   
	   (macro printf::int  (::string ::obj)                "printf")
	   (macro sprintf::int (::string ::string ::procedure) "sprintf")
	   (macro sprintf%g::int (::string ::string ::double)  "sprintf")
	   (macro strcmp::bool (::string ::string)             "strcmp")
	   
	   (tk-main::int (::bool ::string ::int ::string ::string ::string) "Tk_main")
	   (_tk-mainloop::int ()                "Tk_MainLoop")
	   
	   (tk-lookup-command::obj (::string)   "tcl_lookup_command")
	   
	   (errno::int                          "errno")
	   (strerror::string (::int)            "strerror")
	   (macro tcl-error::int                "TCL_ERROR")
	   (macro tcl-ok::int                   "TCL_OK")
	   (macro tcl-break::int                "TCL_BREAK")
	   
	   (string-undotify!::string (::string) "SCM_string_undotify")
	   
	   ;; Bigloo exportations
	   (export tcl-add-error-info           "Tcl_AddErrorInfo")
	   (export tcl-posix-error              "tcl_posix_error")
	   (export tcl-is-safe?                 "Tcl_IsSafe")
	   
	   (export tk-funcall                   "SCM_tk_funcall")
	   
	   (export tk-string->scm-list          "SCM_tk_string_to_scm_list")
	   (export tk-string->scm               "SCM_tk_string_to_scm")
	   (export scm->tk-string               "SCM_scm_to_tk_string")
	   
	   (export *root*                       "SCM_root_window")
	   
	   (export post-cascade   "SCM_post_cascade")
	   (export unpost-cascade "SCM_unpost_cascade")
	   (export tk-menu-dup    "tk_menu_dup"))
   
   (use    (__biglook_misc      "Llib/misc.scm")
	   (__biglook_tk-object "Llib/tk-object.scm")
	   (__biglook_font      "Llib/font.scm"))
   
   (export *root*

	   (tk-string->scm-list          ::string)
	   (tk-string->scm               ::string)
	   (scm->tk-string::string       ::obj)
	   
	   (tcl-add-error-info::int      ::long ::string)
	   (tcl-posix-error::string      ::long)
	   (tcl-is-safe?::bool           ::long)
	   (tk-funcall::long             ::obj)
	   
	   (post-cascade::int menu-name::string x::int y::int)
	   (unpost-cascade::int menu-name::string)
	   (tk-menu-dup::obj ::string ::string ::string)
	   
	   (inline tk-mainloop)))

;*---------------------------------------------------------------------*/
;*    *no-main-loop-apps* ...                                          */
;*    -------------------------------------------------------------    */
;*    The name of the applications that do not requires the tk main    */
;*    loop.                                                            */
;*---------------------------------------------------------------------*/
(define *no-main-loop-apps* '("biglook"))
   
;*---------------------------------------------------------------------*/
;*    *root*                                                           */
;*    -------------------------------------------------------------    */
;*    The TK root window                                               */
;*---------------------------------------------------------------------*/
(define *root* *root*)

;*---------------------------------------------------------------------*/
;*    *error-info* ...                                                 */
;*---------------------------------------------------------------------*/
(define *error-info* "")

;*---------------------------------------------------------------------*/
;*    tcl-add-error-info ...                                           */
;*---------------------------------------------------------------------*/
(define (tcl-add-error-info interp message)
   (set! *error-info* (string-append message *error-info*))
   0)

;*---------------------------------------------------------------------*/
;*    tcl-posix-error ...                                              */
;*---------------------------------------------------------------------*/
(define (tcl-posix-error interp)
   (strerror errno))

;*---------------------------------------------------------------------*/
;*    tcl-is-safe? ...                                                 */
;*---------------------------------------------------------------------*/
(define (tcl-is-safe? interp)
   #f)
   
;*---------------------------------------------------------------------*/
;*    tk-string->scm ...                                               */
;*---------------------------------------------------------------------*/
(define (tk-string->scm s::string)
;*    (print "tk-string->scm: " s)                                     */
   (if (not (strcmp s "."))
       *root*
       (let* ((port (open-input-c-string s))
	      (exp  (read port)))
	  (close-input-port port)
	  exp)))
   
;*---------------------------------------------------------------------*/
;*    tk-string->scm-list ...                                          */
;*---------------------------------------------------------------------*/
(define (tk-string->scm-list s::string)
;*    (display "tk-string->scm-list: ")                                */
;*    (write s)                                                        */
;*    (newline)                                                        */
   (if (not (strcmp s "."))
       *root*
       ;; in a first step, we scan the string to replace the dot character
       ;; with the bang character. That way, the reader won't fail reading
       ;; tk result
       (let* ((port (open-input-c-string (string-undotify! s)))
	      (exp  (let ((exp (read port)))
		       (if (eq? exp '!) 
			   *root*
			   exp))))
	  (if (eof-object? exp)
	      #unspecified
	      (let ((next-exp (read port)))
		 (if (eof-object? next-exp)
		     (begin
			(close-input-port port)
			exp)
		     (let loop ((exp    next-exp)
				(result (list exp)))
			(cond
			   ((eof-object? exp)
			    (reverse! result))
			   (else
			    (loop (read port)
				  (cons (if (eq? exp '!) *root* exp)
					result)))))))))))

;*---------------------------------------------------------------------*/
;*    real->tk-string ...                                              */
;*---------------------------------------------------------------------*/
(define (real->tk-string::string obj::real)
   (let ((res (make-string 40)))
      (sprintf%g res "%g" obj)
      res))

;*---------------------------------------------------------------------*/
;*    scm->tk-string ...                                               */
;*---------------------------------------------------------------------*/
(define (scm->tk-string obj)
;*    (display "obj->tk-string: ")                                     */
;*    (write obj)                */
;*    (newline)                                                        */
   (define (keyword->tk-string obj)
      (let ((cell (getprop obj 'tk-value)))
	 (if cell
	     cell
	     (let ((res (string-downcase (keyword->string obj))))
		(string-set! res 0 #\-)
		(putprop! obj 'tk-value res)
		res))))
   (define (fixnum->tk-string obj)
      (case obj
	 ((-1) "-1")
	 ((0) "0")
	 ((1) "1")
	 ((2) "2")
	 ((3) "3")
	 ((4) "4")
	 ((5) "5")
	 ((6) "6")
	 ((7) "7")
	 ((8) "8")
	 ((9) "9")
	 ((10) "10")
	 (else
	  (integer->string obj))))
   (define (proc->tk-string obj)
      (if (tk-command? obj)
	  (tk-command-id obj)
	  (procedure->tk-string obj)))
   (define (symbol->tk-string obj)
      (let ((cell (getprop obj 'tk-value)))
	 (if cell
	     cell
	     (let ((res (string-downcase (symbol->string obj))))
		(putprop! obj 'tk-value res)
		res))))
   (define (circular? obj)
      (class? obj))
   (define (list->tk-string obj)
      (let ((port (open-output-string)))
	 (display (map scm->tk-string obj) port)
	 (close-output-port port)))
   (define (else->tk-string obj)
      (let ((port (open-output-string)))
	 (if (circular? obj)
	     (error "biglook" "Illegal circular object" obj)
	     (begin
		(display obj port)
		(close-output-port port)))))
   (cond
      ((keyword? obj)         (keyword->tk-string obj))
      ((fixnum? obj)          (fixnum->tk-string obj))
      ((procedure? obj)       (proc->tk-string obj))
      ((symbol? obj)          (symbol->tk-string obj))
      ((string? obj)          obj)
      ((eq? obj #unspecified) "")
      ((eq? obj #t)           "#t")
      ((eq? obj #f)           "#f")
      ((tk-object? obj)       (proc->tk-string (tk-object-eid obj)))
      ((font? obj)            (font-id obj))
      ((flonum? obj)          (real->tk-string obj))
      ((pair? obj)            (list->tk-string obj))
      (else                   (else->tk-string obj))))

;*---------------------------------------------------------------------*/
;*    tk-funcall ...                                                   */
;*---------------------------------------------------------------------*/
(define (tk-funcall exp)
   (let ((value (match-case exp
		   (()
		    (error "tk-funcall" "Illegal form" '()))
		   ((?fun)
		    (if (and (procedure? fun) (correct-arity? fun 0))
			(fun)
			(error "tk-funcall" "Illegal call for" exp)))
		   ((?fun ?arg1)
		    (if (and (procedure? fun) (correct-arity? fun 1))
			(fun arg1)
			(error "tk-funcall" "Illegal arity for" exp)))
		   ((?fun ?arg1 ?arg2)
		    (if (and (procedure? fun) (correct-arity? fun 2))
			(fun arg1 arg2)
			(error "tk-funcall" "Illegal arity for" exp)))
		   ((?fun ?arg1 ?arg2 ?arg3)
		    (if (and (procedure? fun) (correct-arity? fun 3))
			(fun arg1 arg2 arg3)
			(error "tk-funcall" "Illegal arity for" exp)))
		   ((?fun ?arg1 ?arg2 ?arg3 ?arg4)
		    (if (and (procedure? fun) (correct-arity? fun 4))
			(fun arg1 arg2 arg3 arg4)
			(error "tk-funcall" "Illegal arity for" exp)))
		   ((?fun . ?rest)
		    (if (and (procedure? fun)
			     (correct-arity? fun (length rest)))
			(apply fun rest)
			(error "tk-funcall" "Illegal arity for" exp))))))
      (if (eq? value 'break)
	  tcl-break
	  tcl-ok)))
	   
;*---------------------------------------------------------------------*/
;*    tk-mainloop ...                                                  */
;*---------------------------------------------------------------------*/
(define-inline (tk-mainloop)
   (_tk-mainloop))
 
;*---------------------------------------------------------------------*/
;*    The tcl initialization                                           */
;*    -------------------------------------------------------------    */
;*    If one option is added, it has to be reported in the             */
;*    BIGLOOK-ARGS? function.                                          */
;*    -------------------------------------------------------------    */
;*    @label Biglook command line args@                                */
;*    @ref misc.scm:Biglook command line args@                         */
;*---------------------------------------------------------------------*/
(let* ((args      (command-line))
       (argv0     (car args))
       (sync      0)
       (name      "")
       (display   (or (getenv "DISPLAY") ""))
       (geometry  "")
       (mainloop? (not (member (basename argv0) *no-main-loop-apps*))))
   (args-parse (cdr args)
      (section "Biglook")
      (("-name" ?nm (synopsis "Name to use for application"))
       (set! name nm))
      (("-display" ?value (synopsis "Display to use"))
       (set! display value))
      (("-geometry" ?geo (synopsis "Initial geometry for window"))
       (set! geometry geo))
      (("-sync" (synopsis "Use synchronous mode for display server"))
       (set! sync #t))
      (("-no-event-loop" (synopsis "Do not spawn event loop"))
       (set! mainloop? #f))
      (else
       (set! the-biglook-args-parse-usage args-parse-usage)))
   (tk-main mainloop? argv0 sync name display geometry))

;*---------------------------------------------------------------------*/
;*    post-cascade ...                                                 */
;*---------------------------------------------------------------------*/
(define (post-cascade menu-name x y)
   (let ((menu (tk-lookup-command menu-name)))
      (if (not (tk-command? menu))
	  (error "post-cascade" "Can't find cascade menu" menu-name))
      (try (begin
	      (menu 'post x y)
	      tcl-ok)
	   (lambda (escape obj proc msg)
	      (notify-error obj proc msg)
	      (escape tcl-error)))))

;*---------------------------------------------------------------------*/
;*    unpost-cascade ...                                               */
;*---------------------------------------------------------------------*/
(define (unpost-cascade menu-name)
   (let ((menu (tk-lookup-command menu-name)))
      (if (not (tk-command? menu))
	  (error "post-cascade" "Can't find cascade menu" menu-name))
      (try (begin
	      (menu 'unpost)
	      tcl-ok)
	   (lambda (escape obj proc msg)
	      (notify-error obj proc msg)
	      (escape tcl-error)))))

;*---------------------------------------------------------------------*/
;*    tk-menu-dup ...                                                  */
;*---------------------------------------------------------------------*/
(define (tk-menu-dup src dst type)  ;; duplicate src menu into dst
   (let ((src (tk-lookup-command src))
	 (menu (make-widget-constructor "menu")))
      (let ((args '()))
	 (for-each
	  (lambda (option)
	     (unless (or (= (length option) 2) (equal? (car option) :type))
		(set! args `(,(car option) ,(list-ref option 4) ,@args))))
	  (src 'configure))
	 (let ((dst (apply menu :id dst :type type args)))
	    (let ((last (src 'index "last")))
	       (unless (equal? last "none")
		  (let loop ((i (if (tk-get src :tearoff) 1 0)))
		     (when (<= i last)
			(let ((args '())
			      (type (src 'type i)))
			   (for-each
			    (lambda (option)
			       (set! args 
				     `(,(car option) ,(list-ref option 4) ,@args)))
			    (src 'entryconfigure i))
			   (apply dst 'add type args)
			   (loop (+ i 1)))))))
	    ;; Duplicate the binding tags and bindings from the source menu.
	    ;
	    ;    regsub -all . $src {\\&} quotedSrc
	    ;    regsub -all . $dst {\\&} quotedDst
	    ;    regsub -all $quotedSrc [bindtags $src] $dst x
	    ;    bindtags $dst $x
	    ;    foreach event [bind $src] {
	    ;	   regsub -all $quotedSrc [bind $src $event] $dst x
	    ;	   bind $dst $event $x
	    ;    }
	    ;
	    ; Is it really useful? Should we duplicate bindings on the copy?
	    ; Furthermore, most of the time this code should do nothing (even if
	    ; necessary for completude).
	    ; Eventually translate this Tcl code in STk but don't use regexp
	    ; since they could not be compiled for STk
	    
	    ;; Return dst as result
	    dst))))

