;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bglk/runtime/Llib/tk-object.scm             */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Tue Dec 15 10:40:37 1998                          */
;*    Last change :  Wed Jun  9 13:09:21 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The general TK object class                                      */
;*    -------------------------------------------------------------    */
;*    Source documentations:                                           */
;*       @path ../../manual/bind.texi@                                 */
;*       @path ../../manual/lower.texi@                                */
;*       @path ../../manual/raise.texi@                                */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_tk-object
   
   (extern (include "tclglue.h")
	   (include "tcl.h")
	   (macro _tk-command-object::obj (::obj)
		  "TK_COMMAND_OBJECT") 
	   (macro _tk-command-object-set!::obj (::obj ::tk-object)
		  "TK_COMMAND_OBJECT_SET"))
   
   (import (__biglook_glue "Llib/scmglue.scm"))

   (use    (__biglook_misc "Llib/misc.scm"))
   
   (export (id->tk-object::obj ::obj)
	   (tk-command->tk-object::tk-object ::procedure)
	   (tk-command-object-set!::obj ::obj ::tk-object)
	   
	   (class tk-object
	      (id::procedure  (default list))
	      (eid::procedure (default list))
	      (parent         (default #unspecified)))
	   
	   ;; general tk widgets
	   (class widget::tk-object)
	   
	   ;; tk non composite widget
	   (class simple-widget::widget
	      ;; background
	      (background::bstring
	       (get (lambda (o)
		       (tk o 'cget :background)))
	       (set (lambda (o v)
		       (tk o 'configure :background v))))
	      ;; border-width
	      (border-width::int
	       (get (lambda (o)
		       (tk o 'cget :borderwidth)))
	       (set (lambda (o v)
		       (tk o 'configure :borderwidth v))))
	      ;; cursor
	      (cursor::bstring
	       (get (lambda (o)
		       (tk o 'cget :cursor)))
	       (set (lambda (o v)
		       (tk o 'configure :cursor v))))
	      ;; highlight-thickness
	      (highlight-thickness::int
	       (get (lambda (o)
		       (tk o 'cget :highlightthickness)))
	       (set (lambda (o v)
		       (tk o 'configure :highlightthickness v))))
	      ;; relief
	      (relief::bstring
	       (get (lambda (o)
		       (tk o 'cget :relief)))
	       (set (lambda (o v)
		       (tk o 'configure :relief v))))
	      ;; take-focus
	      (take-focus::bool
	       (get (lambda (o)
		       (tk o 'cget :takefocus)))
	       (set (lambda (o v)
		       (tk o 'configure :takefocus v)))))
	   
	   (simple-widget-init object::tk-object ::procedure)
	   
	   (inline tk   ::tk-object . l)
	   (inline tk-1 ::tk-object a1)
	   (inline tk-2 ::tk-object a1 a2)
	   (inline tk-3 ::tk-object a1 a2 a3)
	   (inline tk-4 ::tk-object a1 a2 a3 a4)
	   
	   (generic bind ::obj . args)
	   (generic tag-bind ::tk-object ::bstring . args)
	   (generic lower ::obj . below)
	   (generic raise ::obj . below)
	   (flash ::obj)
	   (invoke ::obj)))

;*---------------------------------------------------------------------*/
;*    id->tk-object ...                                                */
;*---------------------------------------------------------------------*/
(define (id->tk-object name)
   (let* ((str (cond
		  ((symbol? name)
		   (string-downcase (symbol->string name)))
		  ((string? name)
		   name)
		  (else
		   (error "id->tk-object" "Illegal identifier" name))))
	  (obj (tk-lookup-command str)))
      (if (tk-command? obj)
	  (_tk-command-object obj)
	  (error "id->tk-object" "Illegal tk command" obj))))
 
;*---------------------------------------------------------------------*/
;*    tk-command-object-set! ...                                       */
;*---------------------------------------------------------------------*/
(define (tk-command-object-set! obj tk-object)
   (if (tk-command? obj)
       (_tk-command-object-set! obj tk-object)
       (error "tk-command-object-set!" "Illegal tk-command" obj)))

;*---------------------------------------------------------------------*/
;*    tk-command->tk-object ...                                        */
;*---------------------------------------------------------------------*/
(define (tk-command->tk-object::tk-object obj::procedure)
   (if (tk-command? obj)
       (_tk-command-object obj)
       (error "tk-command->tk-object" "Illegal tk command" obj)))
       
;*---------------------------------------------------------------------*/
;*    simple-widget-init ...                                           */
;*    -------------------------------------------------------------    */
;*    All simple (non composite) widgets call that function to be      */
;*    properly initialized.                                            */
;*---------------------------------------------------------------------*/
(define (simple-widget-init object::tk-object tk-alloc)
   (with-access::tk-object object (id eid parent)
      (let ((w (if (eq? parent #unspecified)
		   (tk-alloc)
		   (tk-alloc :parent (tk-object-id parent)))))
	 (set! id w)
	 (set! eid w)
	 ;; we make the connection between the tk object and
	 ;; the biglook command in order to implement the
	 ;; id->tk-object facility.
	 (tk-command-object-set! eid object))))

;*---------------------------------------------------------------------*/
;*    tk ...                                                           */
;*    -------------------------------------------------------------    */
;*    General tk caller. All tk slots may be access via that           */
;*    function.                                                        */
;*---------------------------------------------------------------------*/
(define-inline (tk tok::tk-object . args)
   (with-access::tk-object tok (id)
      (apply id args)))

;*---------------------------------------------------------------------*/
;*    tk-1 ...                                                         */
;*---------------------------------------------------------------------*/
(define-inline (tk-1 tok::tk-object arg1)
   (with-access::tk-object tok (id)
      (id arg1)))
   
;*---------------------------------------------------------------------*/
;*    tk-2 ...                                                         */
;*---------------------------------------------------------------------*/
(define-inline (tk-2 tok::tk-object arg1 arg2)
   (with-access::tk-object tok (id)
      (id arg1 arg2)))
   
;*---------------------------------------------------------------------*/
;*    tk-3 ...                                                         */
;*---------------------------------------------------------------------*/
(define-inline (tk-3 tok::tk-object arg1 arg2 arg3)
   (with-access::tk-object tok (id)
      (id arg1 arg2 arg3)))
   
;*---------------------------------------------------------------------*/
;*    tk-4 ...                                                         */
;*---------------------------------------------------------------------*/
(define-inline (tk-4 tok::tk-object arg1 arg2 arg3 arg4)
   (with-access::tk-object tok (id)
      (id arg1 arg2 arg3 arg4)))

;*---------------------------------------------------------------------*/
;*    tk-bind ...                                                      */
;*---------------------------------------------------------------------*/
(define tk-bind (tk-lookup-command "bind"))

;*---------------------------------------------------------------------*/
;*    @deffn bind@ ::obj ...                                           */
;*---------------------------------------------------------------------*/
(define-generic (bind obj::obj . args)
   (apply tk-bind obj args)) 

;*---------------------------------------------------------------------*/
;*    bind ::tk-object ...                                             */
;*---------------------------------------------------------------------*/
(define-method (bind obj::tk-object . args)
   (apply tk-bind (tk-object-id obj) args))

;*---------------------------------------------------------------------*/
;*    @deffn tag-bind@ ::obj ...                                       */
;*---------------------------------------------------------------------*/
(define-generic (tag-bind obj::tk-object key::bstring . args))

;*---------------------------------------------------------------------*/
;*    tk-lower ...                                                     */
;*---------------------------------------------------------------------*/
(define tk-lower (tk-lookup-command "lower"))

;*---------------------------------------------------------------------*/
;*    @deffn lower@ ::obj ...                                          */
;*---------------------------------------------------------------------*/
(define-generic (lower obj::obj . below)
   (apply tk-lower obj below))

;*---------------------------------------------------------------------*/
;*    tk-raise ...                                                     */
;*---------------------------------------------------------------------*/
(define tk-raise (tk-lookup-command "raise"))

;*---------------------------------------------------------------------*/
;*    @deffn raise@ ::obj ...                                          */
;*---------------------------------------------------------------------*/
(define-generic (raise obj::obj . below)
   (apply tk-raise obj below))
   
;*---------------------------------------------------------------------*/
;*    flash ...                                                        */
;*---------------------------------------------------------------------*/
(define (flash obj::obj)
   (tk obj 'flash))

;*---------------------------------------------------------------------*/
;*    invoke ...                                                       */
;*---------------------------------------------------------------------*/
(define (invoke obj::obj)
   (tk obj 'invoke))

