;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bglk/runtime/Lwidget/button.scm             */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Tue Oct 20 10:39:28 1998                          */
;*    Last change :  Thu Jun 10 14:50:15 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The Button widgets                                               */
;*    -------------------------------------------------------------    */
;*    Source documentations:                                           */
;*       @path ../../manual/label.texi@                                */
;*       @path ../../manual/button.texi@                               */
;*       @path ../../manual/checkbutton.texi@                          */
;*       @path ../../manual/radiobutton.texi@                          */
;*       @node Label@                                                  */
;*       @node Button@                                                 */
;*       @node Checkbutton@                                            */
;*       @node Radiobutton@                                            */
;*    Examples:                                                        */
;*       @path ../../demos/label/label.scm@                            */
;*       @path ../../demos/button/button.scm@                          */
;*       @path ../../demos/checkbutton/checkbutton.scm@                */
;*       @path ../../demos/radiobutton/radiobutton.scm@                */
;*=====================================================================*/
  
;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_button
   
   (include "Llib/biglook.sch")
   
   (option  (loadq "Llib/biglook.init"))
   
   (import  (__biglook_glue      "Llib/scmglue.scm")
	    (__biglook_misc      "Llib/misc.scm")
	    (__biglook_key       "Llib/key.scm")
	    (__biglook_tk-object "Llib/tk-object.scm")
	    (__biglook_frame     "Lwidget/frame.scm"))
   
   (export  ;; tk label
            (class label::simple-widget
	       (label-init)
	       ;; anchor
	       (anchor
		(get (lambda (o)
			(tk o 'cget :anchor)))
		(set (lambda (o v)
		 	(tk o 'configure :anchor v))))
	       ;; font
	       (font
		(get (lambda (o)
			(tk o 'cget :font)))
		(set (lambda (o v)
			(tk o 'configure :font v))))
	       ;; foreground
	       (foreground
		(get (lambda (o)
			(tk o 'cget :foreground)))
		(set (lambda (o v)
			(tk o 'configure :foreground v))))
	       ;; image
	       (image
		(get (lambda (o)
			(tk o 'cget :image)))
		(set (lambda (o v)
			(tk o 'configure :image v))))
	       ;; justify
	       (justify
		(get (lambda (o)
			(tk o 'cget :justify)))
		(set (lambda (o v)
			(tk o 'configure :justify v))))
	       ;; pad-x
	       (pad-x
		(get (lambda (o)
			(tk o 'cget :padx)))
		(set (lambda (o v)
			(tk o 'configure :padx v))))
	       ;; pad-y
	       (pad-y
		(get (lambda (o)
			(tk o 'cget :pady)))
		(set (lambda (o v)
			(tk o 'configure :pady v))))
	       ;; text
	       (text
		(get (lambda (o)
			(tk o 'cget :text)))
		(set (lambda (o v)
			(tk o 'configure :text v))))
	       ;; underline
	       (underline
		(get (lambda (o)
			(tk o 'cget :underline)))
		(set (lambda (o v)
			(tk o 'configure :underline v))))
	       ;; wrap-length
	       (wrap-length
		(get (lambda (o)
			(tk o 'cget :wraplength)))
		(set (lambda (o v)
			(tk o 'configure :wraplength v))))
	       ;; width
	       (width
		(get (lambda (o)
			(tk o 'cget :width)))
		(set (lambda (o v)
			(tk o 'configure :width v))))
	       ;; height
	       (height
		(get (lambda (o)
			(tk o 'cget :height)))
		(set (lambda (o v)
			(tk o 'configure :height v)))))

	    ;; help balloon
	    (class help-balloon::label
	       ;; labeled entry constructor
	       (help-balloon-init)
	       ;; associated widgets
	       (frame (default #unspecified))
	       (entry (default #unspecified))
	       (label (default #unspecified))
	       ;; class
	       (class::bstring read-only (default "HelpBallon"))
	       ;; delay
	       (delay::int (default 500)))

            ;; simple buttons
	    (class button::label
	       ;; button initialization
	       (button-init)
	       ;; internal balloon field
	       (%balloon (default #unspecified))
	       ;; active-background
	       (active-background
		(get (lambda (o)
			(tk o 'cget :activebackground)))
		(set (lambda (o v)
			(tk o 'configure :activebackground v))))
	       ;; active-foreground
	       (active-foreground
		(get (lambda (o)
			(tk o 'cget :activeforeground)))
		(set (lambda (o v)
			(tk o 'configure :activeforeground v))))
	       ;; command
	       (command
		(get (lambda (o)
			(tk o 'cget :command)))
		(set (lambda (o v)
			(if (and (procedure? v) (correct-arity? v 0))
			    (tk o 'configure :command v)
			    (error "button-command-set!"
				   "Illegal procedure"
				   v)))))
	       ;; disabled-foreground
	       (disabled-foreground
		(get (lambda (o)
			(tk o 'cget :disabledf)))
		(set (lambda (o v)
			(tk o 'configure :disabledf v))))
	       ;; state
	       (state
		(get (lambda (o)
			(tk o 'cget :state)))
		(set (lambda (o v)
			(tk o 'configure :state v))))
	       ;; balloon
 	       (balloon
		(get (lambda (o)
			(with-access::button o (%balloon)
			   (if (help-balloon? %balloon)
			       (help-balloon-text %balloon)
			       #f))))
		(set (lambda (o v::bstring)
			(with-access::button o (%balloon)
			   (if (help-balloon? %balloon)
			       (help-balloon-text-set! %balloon v)
			       (let ((balloon (instantiate::help-balloon
						 (text v))))
				  (button-%balloon-set! o balloon)
				  (add-balloon! balloon o)))))))
	       ;; balloon-background
	       (balloon-background
		(get (lambda (o)
			(with-access::button o (%balloon)
			   (if (help-balloon? %balloon)
			       (help-balloon-background %balloon)
			       #f))))
		(set (lambda (o v)
			(with-access::button o (%balloon)
			   (if (help-balloon? %balloon)
			       (help-balloon-background-set! %balloon v)))))))
	    
	    ;; complex buttons
	    (class complex-button::label
	       ;; active-background
	       (active-background
		(get (lambda (o)
			(tk o 'cget :activebackground)))
		(set (lambda (o v)
			(tk o 'configure :activebackground v))))
	       ;; active-foreground
	       (active-foreground
		(get (lambda (o)
			(tk o 'cget :activeforeground)))
		(set (lambda (o v)
			(tk o 'configure :activeforeground v))))
	       ;; indicator-on
	       (indicator-on
		(get (lambda (o)
			(tk o 'cget :indicatoron)))
		(set (lambda (o v)
			(tk o 'configure :indicatoron v))))
	       ;; select-color
	       (select-color::bstring
		(get (lambda (o)
			(tk o 'cget :selectcolor)))
		(set (lambda (o v)
			(tk o 'configure :selectcolor v))))
	       ;; select-image
	       (select-image::bstring
		(get (lambda (o)
			(tk o 'cget :selectimage)))
		(set (lambda (o v)
			(tk o 'configure :selectimage v))))
	       ;; command
	       (command
		(get (lambda (o)
			(tk o 'cget :command)))
		(set (lambda (o v)
			(if (and (procedure? v) (correct-arity? v 0))
			    (tk o 'configure :command v)
			    (error "complex-button-set!"
				   "Illegal procedure"
				   v))))))
	    
	    ;; check buttons
	    (class check-button::complex-button
	       (check-button-init)
	       ;; key
	       (key::bstring read-only (default (symbol->string (gensym))))
 	       ;; value
	       (value
		read-only
		(get (lambda (o)
			(with-access::check-button o (key)
			   (key-value key))))))
	    
	    ;; radio buttons
	    (class radio-button::complex-button
	       (radio-button-init)
	       ;; string-value
	       (string-value
		(get (lambda (o)
			(tk o 'cget :stringvalue)))
		(set (lambda (o v)
			(tk o 'configure :stringvalue v))))
	       ;; key
	       (key::bstring
		(get (lambda (o)
			(tk o 'cget :variable)))
		(set (lambda (o v)
			(tk o 'configure :variable v))))
	       ;; value
	       (value
		(get (lambda (o)
			(tk o 'cget :value)))
		(set (lambda (o v)
			(tk o 'configure :value v)))))

	    ;; constructors
	    label-init
	    button-init
	    check-button-init
	    radio-button-init

	    ;; balloon
	    (help-balloon-init ::help-balloon)
	    (add-balloon!      ::help-balloon who)))

;*---------------------------------------------------------------------*/
;*    label-init ...                                                   */
;*---------------------------------------------------------------------*/
(define label-init
   (let ((label (make-widget-constructor "label")))
      (lambda (o)
	 (simple-widget-init o label))))

;*---------------------------------------------------------------------*/
;*    button-init ...                                                  */
;*---------------------------------------------------------------------*/
(define button-init
   (let ((button (make-widget-constructor "button")))
      (lambda (o) (simple-widget-init o button))))

;*---------------------------------------------------------------------*/
;*    check-button-init ...                                            */
;*---------------------------------------------------------------------*/
(define check-button-init 
   (let ((check-button (make-widget-constructor "checkbutton")))
      (lambda (o)
	 (with-access::check-button o (key value)
	    (simple-widget-init o check-button)
	    (key-value-set! key #f)
	    (tk o 'configure :variable key)))))

;*---------------------------------------------------------------------*/
;*    radio-button-init ...                                            */
;*---------------------------------------------------------------------*/
(define radio-button-init
   (let ((radio-button (make-widget-constructor "radiobutton")))
      (lambda (o) (simple-widget-init o radio-button))))

;*---------------------------------------------------------------------*/
;*    global variables                                                 */
;*---------------------------------------------------------------------*/
(define window        #f)
(define button-window '())
(define relief	      "sunken")

;*---------------------------------------------------------------------*/
;*    button-enter ...                                                 */
;*    -------------------------------------------------------------    */
;*    The procedure below is invoked when the mouse pointer enters a   */
;*    button widget.  It records the button we're in and changes the   */
;*    state of the button to active unless the button is disabled.     */
;*---------------------------------------------------------------------*/
(define (button-enter WW)
   (unless (equal? (tk-get WW :state) "disabled")
	   (tk-set! WW :state "active")
	   (if (equal? button-window WW)
	       (tk-set! WW :state "active" :relief "sunken")))
   (set! window WW))

;*---------------------------------------------------------------------*/
;*    button-leave ...                                                 */
;*    -------------------------------------------------------------    */
;*    The procedure below is invoked when the mouse pointer leaves a   */
;*    button widget. It changes the state of the button back to        */
;*    inactive. If we're leaving the button window with a mouse button */
;*    pressed (:button-window == WW), restore the relief of the        */
;*    button too.                                                      */
;*---------------------------------------------------------------------*/
(define (button-leave WW)
   (unless (equal? (tk-get WW :state) "disabled")
	   (tk-set! WW :state "normal"))
   (if (equal? button-window WW)
       (tk-set! WW :relief relief))
   (set! window #f))

;*---------------------------------------------------------------------*/
;*    button-down ...                                                  */
;*    -------------------------------------------------------------    */
;*    The procedure below is invoked when the mouse button is pressed  */
;*    in a button widget. It records the fact that the mouse is in the */
;*    button, saves the button's relief so it can be restored later,   */
;*    and changes the relief to sunken.                                */
;*---------------------------------------------------------------------*/
(define (button-down WW)
   (set! relief (tk-get WW :relief))
   (unless (equal? (tk-get WW :state) "disabled")
	   (set! button-window WW)
	   (tk-set! WW :relief "sunken")))

;*---------------------------------------------------------------------*/
;*    button-up ...                                                    */
;*    -------------------------------------------------------------    */
;*    The procedure below is invoked when the mouse button is released */
;*    in a button widget.  It restores the button's relief and invokes */
;*    the command as long as the mouse hasn't left the button.         */
;*---------------------------------------------------------------------*/
(define (button-up WW)
   (when (equal? button-window WW)
	 (set! button-window "")
	 (tk-set! WW :relief relief)
	 (when (and (equal? WW window)
		    (not (equal? (tk-get WW :state) "disabled")))
	       (WW 'invoke))))

;*---------------------------------------------------------------------*/
;*    button-invoke ...                                                */
;*    -------------------------------------------------------------    */
;*    The procedure below is called when a button is invoked through   */
;*    the keyboard.  It simulate a press of the button via the mouse.  */
;*---------------------------------------------------------------------*/
(define (button-invoke WW)
  (unless (equal? (tk-get WW :state) "disabled")
     (let ((old-relief (tk-get WW :relief))
	   (old-state  (tk-get WW :state)))
       (tk-set! WW :state "active" :relief "sunken")
       (update 'idletasks)
       (after 100)
       (tk-set! WW :state old-state :relief old-relief)
       (WW 'invoke))))

;*---------------------------------------------------------------------*/
;*    R&C-invoke ...                                                   */
;*    -------------------------------------------------------------    */
;*    The procedure below is invoked when the mouse button is pressed  */
;*    in a checkbutton or radiobutton widget, or when the widget is    */
;*    invoked through the keyboard.  It invokes the widget if it isn't */
;*    disabled.                                                        */
;*---------------------------------------------------------------------*/
(define (R&C-invoke WW . cmd)
   (unless (equal? (tk-get WW :state) "disabled")
	   (WW (if (null? cmd) 'invoke (car cmd)))))

;*---------------------------------------------------------------------*/
;*    Class bindings for various flavors of button widgets. :window    */
;*    keeps track of the button containing the mouse, and :relief      */
;*    saves the original relief of the button so it can be restored    */
;*    when the mouse button is released.                               */
;*---------------------------------------------------------------------*/
;; button
(remove-binding! "Checkbutton" "<FocusIn>")
(add-binding! "Button" "<Enter>" (WW)
	      (button-enter WW))
(add-binding! "Button" "<Leave>" (WW)
	      (button-leave WW))
(add-binding! "Button" "<1>" (WW)
	      (button-down WW))
(add-binding! "Button" "<ButtonRelease-1>" (WW)
	      (button-up WW))
(add-binding! "Button" "<space>" (WW)
	      (button-invoke WW))

;; check-button
(remove-binding! "Checkbutton" "<FocusIn>")
(add-binding! "Checkbutton" "<Return>" (WW)
	      (unless *tk-strict-motif* (R&C-invoke WW)))
(add-binding! "Checkbutton" "<1>" (WW)
	      (R&C-invoke WW))
(add-binding! "Checkbutton" "<Enter>" (WW)
	      (button-enter WW))
(add-binding! "Checkbutton" "<Leave>" (WW)
	      (button-leave WW))
(add-binding! "Checkbutton" "<space>" (WW)
	      (R&C-invoke WW)))

;; radio-button
(remove-binding! "Radiobutton" "<FocusIn>")
(add-binding! "Radiobutton" "<Return>" (WW)
	      (unless *tk-strict-motif*
		      (R&C-invoke WW)))
(add-binding! "Radiobutton" "<1>" (WW)
	      (R&C-invoke WW))
(add-binding! "Radiobutton" "<Enter>" (WW)
	      (button-enter WW))
(add-binding! "Radiobutton" "<Leave>" (WW)
	      (button-leave WW))
(add-binding! "Radiobutton" "<space>" (WW)
	      (R&C-invoke WW))

;*---------------------------------------------------------------------*/
;*    help-balloon-init ...                                            */
;*---------------------------------------------------------------------*/
(define (help-balloon-init self::help-balloon)
   (with-access::help-balloon self (parent eid id label frame)
      (set! parent (instantiate::top-level))
      (set! frame parent)
      (set! label (instantiate::label
		     (parent parent)
		     (background "LemonChiffon")))
      (make-transient parent)
      (label-init self)
      ;; pack the label in the parent window
      (pack (label-id label) :expand #t :fill "both")
      ;; set true slots
      (set! id (label-id label))))

;*---------------------------------------------------------------------*/
;*    add-balloon! ...                                                 */
;*---------------------------------------------------------------------*/
(define (add-balloon! self::help-balloon who)
   (define handler #f)
   
   (define (display W)
      (after 'cancel handler)
      (let ((delay (help-balloon-delay self)))
	 (if (>= delay 0)
	     (set! handler
		   (after delay
			  (lambda ()
			     (let ((height (winfo 'height W))
				   (pos-y  (winfo 'rooty  W)))
				(with-access::help-balloon self (frame text)
				   ;; place the balloon just outside the widget
				   (top-level-geometry-set!
				    frame
				    (string-append "+"
						   (number->string
						    (winfo 'pointerx W))
						   "+"
						   (number->string
						    (+ pos-y height 2))))
				   ;; Deiconify
				   (deiconify frame)
				   (raise frame)))))))))
   
   (define (delete)
      (withdraw (help-balloon-frame self))
      (after 'cancel handler))
   
   ;; Associate new bindings to a "ballooned" widget
   (add-binding! who "<Enter>" () (display who))
   (add-binding! who "<Leave>" () (delete)))

;*---------------------------------------------------------------------*/
;*    X11 resources                                                    */
;*---------------------------------------------------------------------*/
(option 'add "*HelpBalloon*Background"  	"#ffffb0" "widgetDefault")
(option 'add "*HelpBalloon*Font"        	"fixed"   "widgetDefault")
(option 'add "*HelpBalloon*Relief"		"flat"    "widgetDefault")
(option 'add "*HelpBalloon*HighlightThickness"	0 	  "widgetDefault")

