;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bglk/runtime/Lwidget/canvas.scm             */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Tue Dec 22 08:16:58 1998                          */
;*    Last change :  Fri Jun 11 15:24:13 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    Canvas class definition                                          */
;*    -------------------------------------------------------------    */
;*    Source documentation:                                            */
;*       @path ../../manual/canvas.texi@                               */
;*       @node Canvas@                                                 */
;*    Example:                                                         */
;*       @path ../../demos/canvas/canvas.scm@                          */
;*       @path ../../demos/dnd/dnd.scm@                                */
;*       @path ../../demos/bindcanvas/bindcanvas.scm@                  */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_canvas
   
   (include "Llib/biglook.sch")
   
   (option  (loadq "Llib/biglook.init"))
   
   (import  (__biglook_glue      "Llib/scmglue.scm")
	    (__biglook_misc      "Llib/misc.scm")
	    (__biglook_key  "Llib/key.scm")
	    (__biglook_tk-object "Llib/tk-object.scm"))
   
   (export  (class canvas::simple-widget
	       ;; initialization
	       (canvas-init)
	       ;; width
	       (width
		(get (lambda (o)
			(tk o 'cget :width)))
		(set (lambda (o v)
			(tk o 'configure :width v))))
	       ;; height
	       (height
		(get (lambda (o)
			(tk o 'cget :height)))
		(set (lambda (o v)
			(tk o 'configure :height v))))
	       ;; x-scroll-command
	       (x-scroll-command
		(get (lambda (o)
			(tk o 'cget :xscrollcommand)))
		(set (lambda (o v)
			(tk o 'configure :xscrollcommand v))))
	       ;; y-scroll-command
	       (y-scroll-command
		(get (lambda (o)
			(tk o 'cget :yscrollcommand)))
		(set (lambda (o v)
			(tk o 'configure :yscrollcommand v))))
	       ;; insert-background
	       (insert-background
		(get (lambda (o)
			(tk o 'cget :insertbackground)))
		(set (lambda (o v)
			(tk o 'configure :insertbackground v))))
	       ;; insert-border-width
	       (insert-border-width
		(get (lambda (o)
			(tk o 'cget :insertborderwidth)))
		(set (lambda (o v)
			(tk o 'configure :insertborderwidth v))))
	       ;; insert-off-time
	       (insert-off-time
		(get (lambda (o)
			(tk o 'cget :insertofftime)))
		(set (lambda (o v)
			(tk o 'configure :insertofftime v))))
	       ;; insert-on-time
	       (insert-on-time
		(get (lambda (o)
			(tk o 'cget :insertontime)))
		(set (lambda (o v)
			(tk o 'configure :insertontime v))))
	       ;; insert-width
	       (insert-width
		(get (lambda (o)
			(tk o 'cget :insertwidth)))
		(set (lambda (o v)
			(tk o 'configure :insertwidth v))))
	       ;; select-background
	       (select-background
		(get (lambda (o)
			(tk o 'cget :selectbackground)))
		(set (lambda (o v)
			(tk o 'configure :selectbackground v))))
	       ;; select-foreground
	       (select-foreground
		(get (lambda (o)
			(tk o 'cget :selectforeground)))
		(set (lambda (o v)
			(tk o 'configure :selectforeground v))))
	       ;; select-border-width
	       (select-border-width
		(get (lambda (o)
			(tk o 'cget :selectborderwidth)))
		(set (lambda (o v)
			(tk o 'configure :selectborderwidth v))))
	       ;; items
	       (items (default '()))
	       ;; close-enough
	       (close-enough
		(get (lambda (o)
			(tk o 'cget :closeenough)))
		(set (lambda (o v)
			(tk o 'configure :closeenough v))))
	       ;; confine
	       (confine
		(get (lambda (o)
			(tk o 'cget :confine)))
		(set (lambda (o v)
			(tk o 'configure :confine v))))
	       ;; x-scroll-increment
	       (x-scroll-increment
		(get (lambda (o)
			(tk o 'cget :xscrollincrement)))
		(set (lambda (o v)
			(tk o 'configure :xscrollincrement v))))
	       ;; y-scroll-increment
	       (y-scroll-increment
		(get (lambda (o)
			(tk o 'cget :yscrollincrement)))
		(set (lambda (o v)
			(tk o 'configure :yscrollincrement v))))
	       ;; scroll-region
	       (scroll-region
		(get (lambda (o)
			(tk o 'cget :scrollregion)))
		(set (lambda (o v)
	 		(tk o 'configure :scrollregion v)))))
	    
	    canvas-init
	    bind-for-dragging!
	    (find-items self::canvas . args)
	    (canvas-delete self::canvas . args)
	    (bounding-box self::canvas tag)))

;*---------------------------------------------------------------------*/
;*    canvas-init ...                                                  */
;*---------------------------------------------------------------------*/
(define canvas-init
   (let ((canvas (make-widget-constructor "canvas")))
      (lambda (self) (simple-widget-init self canvas))))

;*---------------------------------------------------------------------*/
;*    cid->instance ...                                                */
;*---------------------------------------------------------------------*/
(define (cid->instance self::canvas id)
   (with-access::canvas self (items)
      (let ((cell (assq id items)))
	 (if (pair? cell)
	     (cdr cell)
	     #f))))

;*---------------------------------------------------------------------*/
;*    tag-value ...                                                    */
;*---------------------------------------------------------------------*/
(define (tag-value object)
   (if (or (symbol? object) (integer? object) (string? object))
       object
       (error "tag-value" "Object is not contained in a canvas" object)))

;*---------------------------------------------------------------------*/
;*    add-tag ...                                                      */
;*---------------------------------------------------------------------*/
(define (add-tag self::canvas tag . args)
   (apply tk self 'addtag tag args))

;*---------------------------------------------------------------------*/
;*    bounding-box ...                                                 */
;*---------------------------------------------------------------------*/
(define (bounding-box self::canvas tag)
   (tk self 'bbox (tag-value tag)))

;*---------------------------------------------------------------------*/
;*    canvas-bind ...                                                  */
;*---------------------------------------------------------------------*/
(define (canvas-bind self::canvas tag-or-id button args-str::bstring proc)
   (with-access::canvas self (id)
      (let ((proc-str (procedure->tk-string proc)))
	 (id 'bind
	     (tag-value tag-or-id)
	     button
	     (string-append "(" proc-str " " args-str ")")))))
   
;*---------------------------------------------------------------------*/
;*    canvas-x ...                                                     */
;*---------------------------------------------------------------------*/
(define (canvas-x self::canvas screenx . args)
   (apply tk self 'canvasx screenx args))

;*---------------------------------------------------------------------*/
;*    canvas-y ...                                                     */
;*---------------------------------------------------------------------*/
(define (canvas-y self::canvas screeny . args)
   (apply tk self 'canvasx screeny args))

;*---------------------------------------------------------------------*/
;*    coords ...                                                       */
;*---------------------------------------------------------------------*/
(define (coords self::canvas tag-or-id)
   (tk self 'coords (tag-value tag-or-id)))

;*---------------------------------------------------------------------*/
;*    canvas-delete ...                                                */
;*---------------------------------------------------------------------*/
(define (canvas-delete self::canvas . args)
   (apply tk self 'delete (map tag-value args)))

;*---------------------------------------------------------------------*/
;*    delete-chars ...                                                 */
;*---------------------------------------------------------------------*/
(define (delete-chars self::canvas tag-or-id first . last)
   (apply tk self 'dchars (tag-value tag-or-id) first last))

;*---------------------------------------------------------------------*/
;*    delete-tag ...                                                   */
;*---------------------------------------------------------------------*/
(define (delete-tag self::canvas tag-or-id . tag-to-delete)
   (apply tk self 'dtag (tag-value tag-or-id) tag-to-delete))

;*---------------------------------------------------------------------*/
;*    find-items ...                                                   */
;*---------------------------------------------------------------------*/
(define (find-items self::canvas . args)
   (let ((result (apply tk self 'find args)))
      (if (list? result)
	  (map (lambda (x) (cid->instance self x)) result)
	  (cid->instance self result))))

;*---------------------------------------------------------------------*/
;*    item-with-focus ...                                              */
;*---------------------------------------------------------------------*/
(define (item-with-focus self::canvas)
   (cid->instance self ((canvas-id self) 'focus)))

;*---------------------------------------------------------------------*/
;*    focus ...                                                        */
;*---------------------------------------------------------------------*/
;* (define (focus self::canvas tag-or-id)                              */
;*    (tk self 'focus (tag-value tag-or-id)))                          */

;*---------------------------------------------------------------------*/
;*    get-tags ...                                                     */
;*---------------------------------------------------------------------*/
(define (get-tags self::canvas tag-or-id)
   (tk self 'gettags (tag-value tag-or-id)))

;*---------------------------------------------------------------------*/
;*    icursor ...                                                      */
;*---------------------------------------------------------------------*/
(define (icursor self::canvas tag-or-id index)
   (tk self 'icursor (tag-value tag-or-id) index))

;*---------------------------------------------------------------------*/
;*    text-index ...                                                   */
;*---------------------------------------------------------------------*/
(define (text-index self::canvas tag-or-id index)
   (tk self 'index (tag-value tag-or-id) index))

;*---------------------------------------------------------------------*/
;*    text-insert ...                                                  */
;*---------------------------------------------------------------------*/
(define (text-insert self::canvas tag-or-id before string)
   (tk self 'insert (tag-value tag-or-id) before string))

;*---------------------------------------------------------------------*/
;*    item-configure ...                                               */
;*---------------------------------------------------------------------*/
(define (item-configure self::canvas tag-or-id . args)
   (apply tk self 'itemconfigure (tag-value tag-or-id) args))

;*---------------------------------------------------------------------*/
;*    tag-bind ::canvas ...                                            */
;*---------------------------------------------------------------------*/
(define-method (tag-bind obj::canvas key::bstring . args)
   (apply tk obj 'bind key args))
   
;*---------------------------------------------------------------------*/
;*    lower ...                                                        */
;*---------------------------------------------------------------------*/
(define-method (lower self::canvas . args)
   (match-case args
      ((?tag-or-id . ?below)
       (apply tk self 'lower (tag-value tag-or-id) (map tag-value below)))
      (()
       (call-next-method))
      (else
       (error "lower ::canvas" "Illegal argument" args))))

;*---------------------------------------------------------------------*/
;*    raise ...                                                        */
;*---------------------------------------------------------------------*/
(define-method (raise self::canvas . args)
   (match-case args
      ((?tag-or-id . ?above)
       (apply tk self 'raise (tag-value tag-or-id) (map tag-value above)))
      (()
       (call-next-method))
      (else
       (error "raise ::canvas" "Illegal argument" args))))

;*---------------------------------------------------------------------*/
;*    move ...                                                         */
;*---------------------------------------------------------------------*/
(define (move self::canvas tag-or-id x y)
   (tk self 'move (tag-value tag-or-id) x y))

;*---------------------------------------------------------------------*/
;*    rescale ...                                                      */
;*---------------------------------------------------------------------*/
(define (rescale self::canvas tag-or-id x y xs ys)
   (tk self 'scale (tag-value tag-or-id) x y xs ys))

;*---------------------------------------------------------------------*/
;*    scan ...                                                         */
;*---------------------------------------------------------------------*/
(define (scan self::canvas option x y)
   (tk self 'scan option x y))

;*---------------------------------------------------------------------*/
;*    text-selection ...                                               */
;*---------------------------------------------------------------------*/
(define (text-selection self::canvas . args)
   (apply tk self 'select args))

;*---------------------------------------------------------------------*/
;*    x-view ...                                                       */
;*---------------------------------------------------------------------*/
(define (x-view self::canvas . args)
   (apply tk self 'xview args))

;*---------------------------------------------------------------------*/
;*    x-view-scroll-units ...                                          */
;*---------------------------------------------------------------------*/
(define (x-view-scroll-units self::canvas num)
   (tk self 'xview 'scroll num 'units))

;*---------------------------------------------------------------------*/
;*    x-view-scroll-pages ...                                          */
;*---------------------------------------------------------------------*/
(define (x-view-scroll-pages self::canvas num)
   (tk self 'scroll num 'pages))

;*---------------------------------------------------------------------*/
;*    x-view-moveto ...                                                */
;*---------------------------------------------------------------------*/
(define (x-view-moveto self::canvas fraction)
   (tk self 'yview 'moveto fraction))

;*---------------------------------------------------------------------*/
;*    y-view ...                                                       */
;*---------------------------------------------------------------------*/
(define (y-view self::canvas . args)
   (apply tk self 'yview args))

;*---------------------------------------------------------------------*/
;*    y-view-scroll-units ...                                          */
;*---------------------------------------------------------------------*/
(define (y-view-scroll-units self::canvas num)
   (tk self 'yview 'scroll num 'units))

;*---------------------------------------------------------------------*/
;*    y-view-scroll-pages ...                                          */
;*---------------------------------------------------------------------*/
(define (y-view-scroll-pages self::canvas num)
   (tk self 'scroll num 'pages))

;*---------------------------------------------------------------------*/
;*    y-view-moveto ...                                                */
;*---------------------------------------------------------------------*/
(define (y-view-moveto self::canvas fraction)
   (tk self 'yview 'moveto fraction))

;*---------------------------------------------------------------------*/
;*    get-keyword ...                                                  */
;*---------------------------------------------------------------------*/
(define (get-keyword kwd lst default)
   (let ((cell (member kwd lst)))
      (if (pair? cell)
	  (cadr cell)
	  default)))

;*---------------------------------------------------------------------*/
;*    bind-for-dragging! ...                                           */
;*---------------------------------------------------------------------*/
(define bind-for-dragging!
   (let ((last-x            0)
	 (last-y            0)
	 (instance-selected '()))
      
      (define (start-drag instance x y closure tag)
	 (let ((tag (or tag (car (tk instance 'find 'with 'current)))))
	    (delete-tag instance 'selected)
	    (add-tag instance 'selected 'with tag)
	    (raise instance 'selected)
	    (set! last-x x)
	    (set! last-y y)
	    (set! instance-selected (cid->instance instance tag))
	    ;; Apply user :start hook
	    (if closure
		(closure instance-selected x y))))
  
      (define (motion-drag instance x y before after)
	 (let ((continue #t))
	    ;; Apply user :before-motion hook
	    (if before 
		(set! continue (before instance-selected x y)))
	    (if continue
		(begin
		   (move instance 'selected (- x last-x) (- y last-y))
		   (set! last-x x)
		   (set! last-y y)
		   ;; Apply user :after-motion hook
		   (if after 
		       (after instance-selected x y))))))

      (define (fast-motion-drag instance x y)
	 (move instance 'selected (- x last-x) (- y last-y))
	 (set! last-x x)
	 (set! last-y y))
  
      (define (stop-drag instance x y closure)
	 (delete-tag instance 'selected)
	 ;; Apply user :stop hook
	 (if closure 
	     (closure instance-selected x y)))

      (define (exp->string exp)
	 (cond
	    ((string? exp)
	     exp)
	    ((number? exp)
	     (number->string exp))
	    ((symbol? exp)
	     (symbol->string exp))
	    (else
	     (error "exp->string" "Don't know how to convert" exp))))
	     
      (define (binding-format fmt exp1 exp2)
	 (let ((str1 (exp->string exp1))
	       (str2 (exp->string exp2)))
	    (string-append "<" str1 fmt str2 ">")))

      (define (inner-bind-for-dragging! self::canvas . args)
	 (let* ((who    (tag-value (get-keyword :tag args 'all)))
		(but    (get-keyword :button args 1))
		(mod    (get-keyword :modifier args ""))
		(alone  (get-keyword :only-current args #t)) 
		(str    (if (string=? mod "") "" (string-append mod "-")))
		(start  (get-keyword :start args #f))
		(before (get-keyword :before-motion args #f))
		(motion (get-keyword :motion args #f))
		(after  (get-keyword :after-motion args motion))
		(stop   (get-keyword :stop args #f))
		(id     (canvas-id self)))
	    
	    ;; Start binding
	    (canvas-bind self who
			 (binding-format "ButtonPress-" str but)
			 "x y"
			 (lambda (x y)
			    (start-drag self x y start (if alone #f who))))
	    ;; Motion binding
	    (canvas-bind self who
			 (string-append "<" str "B" (exp->string but) "-Motion>")
			 "x y"
			 (if (or before after)
			     (lambda (x y) (motion-drag self x y before after))
			     ;; Provide a faster motion handler
			     (lambda (x y) (fast-motion-drag self x y))))
	    ;; Stop binding
	    (canvas-bind self who
			 (binding-format "ButtonRelease-" str but)
			 "x y"
			 (lambda (x y) (stop-drag self x y stop)))))

      inner-bind-for-dragging!))
   

   
