;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bglk/runtime/Lwidget/canvitem.scm           */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Tue Dec 22 08:16:58 1998                          */
;*    Last change :  Fri May 28 11:26:06 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    Canvas class definition                                          */
;*    -------------------------------------------------------------    */
;*    Source documentation:                                            */
;*       @path ../../manual/canvas.texi@                               */
;*       @node canvas@                                                 */
;*    Example:                                                         */
;*       @path ../../demos/dnd/dnd.scm@                                */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_canvitem
   
   (include "Llib/biglook.sch")
   
   (option  (loadq "Llib/biglook.init"))
   
   (import  (__biglook_glue      "Llib/scmglue.scm")
	    (__biglook_misc      "Llib/misc.scm")
	    (__biglook_key  "Llib/key.scm")
	    (__biglook_tk-object "Llib/tk-object.scm")
	    (__biglook_canvas    "Lwidget/canvas.scm"))
   
   (export  (class canvas-item::tk-object
	       ;; cid
	       (cid (default #unspecified)))

	    (class canvas-figure::canvas-item
	       ;; initialization
	       (canvas-figure-init)
	       ;; init-coords slots
	       (init-coords read-only (default '()))
	       ;; tags
	       (tags
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :tags)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :tags v))))
	       ;; coords
	       (coords
		(get (lambda (o)
			(tk o 'coords (canvas-figure-cid o))))
		(set (lambda (o v)
			(apply tk o 'coords (canvas-figure-cid o) v)))))

	    (class composite-item::canvas-item)

	    (class canvas-group::composite-item
	       ;; initialization
	       (canvas-group-init))

	    (class arc::canvas-figure
	       ;; extent
	       (extent
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :extent)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :extent v))))
	       ;; fill
	       (fill
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :fill)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :fill v))))
	       ;; outline
	       (outline
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :outline)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :outline v))))
	       ;; start
	       (start
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :start)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :start v))))
	       ;; stipple
	       (stipple
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :stipple)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :stipple v))))
	       ;; style
	       (style
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :style)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :style v))))
	       ;; width
	       (width
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :width)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :width v)))))

	    (class bitmap-item::canvas-figure
	       ;; anchor
	       (anchor
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :anchor)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :anchor v))))
	       ;; background
	       (background
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :background)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :background v))))
	       ;; bitmap-name
	       (bitmap
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :bitmap)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :bitmap v))))
	       ;; foreground
	       (foreground
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :foreground)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :foreground v)))))

	    (class image-item::canvas-figure
	       ;; anchor
	       (anchor
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :anchor)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :anchor v))))
	       ;; image
	       (image
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :image)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :image v)))))

	    (class line::canvas-figure
	       ;; arrow
	       (arrow
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :arrow)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :arrow v))))
	       ;; arrow-shape
	       (arrow-shape
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :arrowshape)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :arrowshape v))))
	       ;; cap-style
	       (cap-style
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :capstyle)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :capstyle v))))
	       ;; fill
	       (fill
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :fill)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :fill v))))
	       ;; join-style
	       (join-style
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :joinstyle)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :joinstyle v))))
	       ;; smooth
	       (smooth
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :smooth)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :smooth v))))
	       ;; spline-steps
	       (spline-steps
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :splinesteps)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :splinesteps v))))
	       ;; stipple
	       (stipple
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :stipple)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :stipple v))))
	       ;; width
	       (width
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :width)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :width v)))))

	    (class oval::canvas-figure
	       ;; fill
	       (fill
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :fill)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :fill v))))
	       ;; outline
	       (outline
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :outline)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :outline v))))
	       ;; stipple
	       (stipple
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :stipple)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :stipple v))))
	       ;; width
	       (width
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :width)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :width v)))))

	    (class polygon::canvas-figure
	       ;; fill
	       (fill
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :fill)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :fill v))))
	       ;; outline
	       (outline
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :outline)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :outline v))))
	       ;; smooth
	       (smooth
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :smooth)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :smooth v))))
	       ;; spline-steps
	       (spline-steps
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :splinesteps)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :splinesteps v))))
	       ;; stipple
	       (stipple
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :stipple)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :stipple v))))
	       ;; width
	       (width
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :width)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :width v)))))

	    (class rectangle::canvas-figure
	       ;; fill
	       (fill
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :fill)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :fill v))))
	       ;; outline
	       (outline
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :outline)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :outline v))))
	       ;; stipple
	       (stipple
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :stipple)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :stipple v))))
	       ;; width
	       (width
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :width)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :width v)))))

	    (class text-item::canvas-figure
	       ;; anchor
	       (anchor
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :anchor)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :anchor v))))
	       ;; fill
	       (fill
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :fill)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :fill v))))
	       ;; font
	       (font
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :font)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :font v))))
	       ;; justify
	       (justify
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :justify)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :justify v))))
	       ;; stipple
	       (stipple
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :stipple)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :stipple v))))
	       ;; text
	       (text
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :text)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :text v))))
	       ;; width
	       (width
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :width)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :width v)))))

	    (class canvas-window::canvas-figure
	       ;; anchor
	       (anchor
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :anchor)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :anchor v))))
	       ;; height
	       (height
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :height)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :height v))))
	       ;; width
	       (width
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :width)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :width v))))
	       ;; window
	       (window
		(get (lambda (o)
			(tk o 'itemcget (canvas-item-cid o) :window)))
		(set (lambda (o v)
			(tk o 'itemconf (canvas-item-cid o) :window v)))))

	    (canvas-figure-init ::canvas-figure)
	    (canvas-group-init  ::canvas-group)

	    (destroy-canvas-item! self::canvas-item)
	    (destroy-composite-item! self::composite-item)))


;*---------------------------------------------------------------------*/
;*    canvas-item-init ...                                             */
;*---------------------------------------------------------------------*/
(define (canvas-item-init self::canvas-item item-initializer)
   (with-access::canvas-item self (id eid parent)
      (if (not (canvas? parent))
	  (error "canvas-item-init" "Parent is not a canvas" parent)
	  (begin
	     (set! id  (canvas-id parent))
	     (set! eid (canvas-id parent))
	     (let ((cid (item-initializer self (canvas-id parent))))
		(canvas-item-cid-set! self cid)
		(with-access::canvas parent (items)
		   (set! items (cons (cons cid self) items))))))))
 
;*---------------------------------------------------------------------*/
;*    canvas-figure-init ...                                           */
;*---------------------------------------------------------------------*/
(define (canvas-figure-init self::canvas-figure)
   (canvas-item-init self
		     (lambda (self canv-id)
			(with-access::canvas-figure self (init-coords)
			   (apply canv-id 'create
				  (canvas-item-initializer self)
				  init-coords)))))
	  
;*---------------------------------------------------------------------*/
;*    canvas-group-init ...                                            */
;*---------------------------------------------------------------------*/
(define (canvas-group-init self::canvas-group)
   (canvas-item-init self
		     (lambda (o id) (gensym "group"))))
	 
;*---------------------------------------------------------------------*/
;*    canvas-item-initializer ...                                      */
;*---------------------------------------------------------------------*/
(define (canvas-item-initializer self)
   (cond
      ((arc? self)           "arc")
      ((bitmap-item? self)   "bitmap")
      ((image-item? self)    "image")
      ((line? self)          "line")
      ((oval? self)          "oval")
      ((polygon? self)       "polygon")
      ((rectangle? self)     "rectangle")
      ((text-item? self)     "text")
      ((canvas-window? self) "window")
      (else
       (error "canvas-item-initializer"
	      "Don't know how to initialize canvas item"
	      (find-runtime-type self)))))

;*---------------------------------------------------------------------*/
;*    destroy-canvas-item! ...                                         */
;*---------------------------------------------------------------------*/
(define (destroy-canvas-item! self::canvas-item)
   (with-access::canvas-item self (parent cid)
      ;; First delete item from canvas
      (tk parent 'delete cid)
      ;; now deletes its reference in the item lists
      (with-access::canvas parent (items)
	 (let ((cell (assq cid items)))
	    (if (pair? cell)
		(set! items (remq! cell items)))))))

;*---------------------------------------------------------------------*/
;*    destroy-composite-item! ...                                      */
;*---------------------------------------------------------------------*/
(define (destroy-composite-item! self::composite-item)
   (with-access::composite-item self (parent cid)
      ;; First delete item from canvas
      (let ((all (find-items parent 'with cid)))
	 (for-each destroy all)
	 ;; now deletes its reference in the item lists
	 (with-access::canvas parent (items)
	    (let ((cell (assq cid items)))
	       (if (pair? cell)
		   (set! items (remq! cell items))))))))

;*---------------------------------------------------------------------*/
;*    bind ::canvas-item ...                                           */
;*---------------------------------------------------------------------*/
(define-method (bind obj::canvas-item . args)
   (apply tk obj 'bind (canvas-item-cid obj) args))

;*---------------------------------------------------------------------*/
;*    Example                                                          */
;*---------------------------------------------------------------------*/
;* (let* ((c (instantiate::canvas                                      */
;* 	     (height 500)                                              */
;* 	     (width 600)                                               */
;* 	     (background "red")))                                      */
;*        (p (instantiate::rectangle                                   */
;* 	     (parent c)                                                */
;* 	     (init-coords '(20 20 300 40 :fill "blue"))))              */
;*        (o (instantiate::oval                                        */
;* 	     (width 400)                                               */
;* 	     (parent c)                                                */
;* 	     (init-coords '(50 100 250 45 :fill "yellow"))))           */
;*        (l (instantiate::line                                        */
;* 	     (parent c)                                                */
;* 	     (init-coords '(250 250 300 300 :arrow "both"))))          */
;*        (t (instantiate::text-item                                   */
;* 	     (parent c)                                                */
;* 	     (init-coord '(200 210 :fill "black" :text "toto")))))     */
;*    (pack c)                                                         */
;*    (bind-for-dragging! c))                                          */
