;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bglk/runtime/Lwidget/choicebox.scm          */
;*    -------------------------------------------------------------    */
;*    Author      :  Erick Gallesio [eg@kaolin.unice.fr]               */
;*    Creation    :  22-Mar-1994 13:05                                 */
;*    Last change :  Fri Jun 18 11:00:23 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    Choice box                                                       */
;*=====================================================================*/
 
;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_choice-box
   
   (include "Llib/biglook.sch")
   
   (option  (loadq "Llib/biglook.init"))
   
   (import  (__biglook_glue          "Llib/scmglue.scm")
	    (__biglook_misc          "Llib/misc.scm")
	    (__biglook_key      "Llib/key.scm")
	    (__biglook_tk-object     "Llib/tk-object.scm")
	    (__biglook_entry         "Lwidget/entry.scm")
	    (__biglook_frame         "Lwidget/frame.scm")
	    (__biglook_button        "Lwidget/button.scm")
	    (__biglook_menu          "Lwidget/menu.scm")
	    (__biglook_labeled-entry "Lwidget/labeled-entry.scm"))
   
   (export  (class choice-box::labeled-entry
	       ;; labeled entry constructor
	       (choice-box-init)
	       ;; associated widgets
	       (lentry (default #unspecified))
	       (menu (default #unspecified))
	       (menu-button (default #unspecified))
	       ;; virtual background
	       (background
		(get (lambda (o)
			(with-access::choice-box o (frame)
			   (with-access::frame frame (background)
			      background))))
		(set (lambda (o v)
			(with-access::choice-box o (frame
						    lentry
						    menu
						    menu-button)
			   (with-access::frame frame (background)
			      (set! background v))
			   (with-access::entry lentry (background)
			      (set! background v))
			   (with-access::menu menu (background)
			      (set! background v))
			   (with-access::menu menu-button (background)
			      (set! background v))))))
	       ;; virtual border-width
	       (border-width
		(get (lambda (o)
			(with-access::choice-box o (frame)
			   (with-access::frame frame (border-width)
			      border-width))))
		(set (lambda (o v)
			(with-access::choice-box o (frame)
			   (with-access::frame frame (border-width)
			      (set! border-width v))))))
	       ;; virtual choices
	       (choices
		(get get-choices)
		(set set-choices!))
	       ;; virtual relief
	       (relief
		(get (lambda (o)
			(with-access::choice-box o (frame)
			   (with-access::frame frame (relief)
			      relief))))
		(set (lambda (o v)
			(with-access::choice-box o (frame)
			   (with-access::frame frame (relief)
			      (set! relief v)))))))
	       
	    (choice-box-init ::choice-box)
	    (generic add-choice! o::choice-box mess)
	    (get-choices cb::choice-box)
	    (set-choices! cb v)))

;*---------------------------------------------------------------------*/
;*    choice-box-init ...                                              */
;*---------------------------------------------------------------------*/
(define (choice-box-init self::choice-box)
   (with-access::choice-box self (frame parent menu menu-button id eid entry label)
      (set! frame (if (eq? parent #unspecified)
		      (instantiate::frame
			 (class "ChoiceBox"))
		      (instantiate::frame
			 (parent parent)
			 (class "ChoiceBox"))))
      (set! menu-button (instantiate::menu-button
			   (parent frame)
			   (text "")
			   (relief "flat")
			   (border-width 0)
			   (highlight-thickness 0)
			   (indicator-on #t)))
      (set! menu (instantiate::menu 
		    (parent menu-button)))
      (set! entry (instantiate::entry (parent frame)))
      (set! label (instantiate::label (parent frame)))
      (set! id (entry-id entry))
      (set! eid (frame-id frame))
      (tk-command-object-set! eid self)
      (pack label :side 'left :padx 2 :pady 2)
      (pack entry :side 'left :padx 2 :pady 2 :expand #t :fill 'x)
      (pack menu-button :side 'left)))

;*---------------------------------------------------------------------*/
;*    read-from-string ...                                             */
;*---------------------------------------------------------------------*/
(define (read-from-string item)
   (let* ((port (open-input-string item))
	  (val  (read port)))
      (close-output-port port)
      val))
 
;*---------------------------------------------------------------------*/
;*    get-choices ...                                                  */
;*---------------------------------------------------------------------*/
(define (get-choices cb::choice-box)
   ;; This is a little bit compilated because the type of the element of the 
   ;; list depends of the :string-value option
   (with-access::choice-box cb (menu)
      (letrec ((last     (menu-index menu 'last))
	       (get-item (lambda (i res)
			    (if (< i 0)
				res
				(let ((item (tk menu 'entrycget i :label)))
				   (get-item (- i 1)
					     (cons (if (choice-box-string-value cb)
						       item
						       (read-from-string item))
						   res)))))))
	 (if (equal? last "none")
	     '()
	     (get-item last '())))))

;*---------------------------------------------------------------------*/
;*    set-choices! ...                                                 */
;*---------------------------------------------------------------------*/
(define (set-choices! cb v)
   (with-access::choice-box cb (menu)
      ;; Delete the current choices
      (menu-delete! menu 0 (menu-index menu 'last))
      ;; Add the current value by suuceesive calls to add-choice
      (for-each (lambda (x) (add-choice! cb x)) v)))
   
;*---------------------------------------------------------------------*/
;*    add-choice! ::choice-box ...                                     */
;*---------------------------------------------------------------------*/
(define-generic (add-choice! o::choice-box mess))

;*---------------------------------------------------------------------*/
;*    add-choice! ::choice-box ...                                     */
;*---------------------------------------------------------------------*/
(define-method (add-choice! self::choice-box mess)
   (with-access::choice-box self (menu)
      (menu-add! menu
		 'command
		 :label mess
		 :command (lambda ()
			     (with-access::choice-box self (state value)
				(let ((s state))
				   (set! state "normal")
				   (set! value mess)
				   (set! state s)))))))

;*---------------------------------------------------------------------*/
;*    menu-index ...                                                   */
;*---------------------------------------------------------------------*/
(define (menu-index menu::menu index)
   (tk menu 'index index))

;*---------------------------------------------------------------------*/
;*    X11 resources                                                    */
;*---------------------------------------------------------------------*/
(option 'add "*ChoiceBox*Entry*Background"  "white" "widgetDefault")
(option 'add "*ChoiceBox*Entry*Relief"      "ridge" "widgetDefault")
(option 'add "*ChoiceBox*Menu*TearOff"      "false" "widgetDefault")
