;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bglk/runtime/Lwidget/combobox.scm           */
;*    -------------------------------------------------------------    */
;*    Author      :  Erick Gallesio                                    */
;*    Creation    :  Wed Apr 28 23:50:10 1999                          */
;*    Last change :  Sun May 23 17:19:58 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    Labeled entry                                                    */
;*    -------------------------------------------------------------    */
;*    Source documentation:                                            */
;*       @path ../../manual/combobox.texi@                             */
;*       @node Labeled Entry@                                          */
;*    Example:                                                         */
;*       @path ../../demos/combobox/combobox.scm@                      */
;*=====================================================================*/
 
;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_combobox
   
   (include "Llib/biglook.sch")
   
   (option  (loadq "Llib/biglook.init"))
   
   (import  (__biglook_glue      "Llib/scmglue.scm")
	    (__biglook_misc      "Llib/misc.scm")
	    (__biglook_key       "Llib/key.scm")
	    (__biglook_tk-object "Llib/tk-object.scm")
	    (__biglook_entry     "Lwidget/entry.scm")
	    (__biglook_scrollbox "Lwidget/scrollbox.scm")
	    (__biglook_listbox   "Lwidget/listbox.scm")
	    (__biglook_frame     "Lwidget/frame.scm")
	    (__biglook_image     "Llib/image.scm")
 	    (__biglook_button    "Lwidget/button.scm")
	    (__biglook_scrollbar "Lwidget/scrollbar.scm"))
   
   (export  (class combobox::entry
	       ;; combobox constructor
	       (combobox-init)
	       ;; internal state
	       (%visible (default #f))
	       ;; associated widgets
	       (frame     (default #unspecified))
	       (entry     (default #unspecified))
	       (button    (default #unspecified))
	       (top-level (default #unspecified))
	       (listbox   (default #unspecified))
	       ;; real slot
	       (command   (default ""))
	       
	       ;; virtual values
	       (values
		(get (lambda (o)
			(with-access::combobox o (listbox)
			   (with-access::listbox listbox (value)
			      value))))
		(set (lambda (o v)
			(with-access::combobox o (listbox)
			   (with-access::listbox listbox (value)
			      (set! value v))))))
	       ;; virtual background
	       (background
		(get (lambda (o)
			(with-access::combobox o (frame)
			   (with-access::frame frame (background)
			      background))))
		(set (lambda (o v)
			(with-access::combobox o (frame entry label)
			   (with-access::frame frame (background)
			      (set! background v))
			   (with-access::button button (background)
			      (set! background v))
			   (with-access::listbox listbox (background)
			      (set! background v))))))
	       ;; virtual text background
	       (text-background
		(get (lambda (o)
			(with-access::combobox o (entry)
			   (with-access::entry entry (background)
			      background))))
		(set (lambda (o v)
			(with-access::combobox o (entry)
			   (with-access::entry entry (background)
			      (set! background v))))))
	       ;; virtual relief
	       (relief
		(get (lambda (o)
			(with-access::combobox o (frame)
			   (with-access::frame frame (relief)
			      relief))))
		(set (lambda (o v)
			(with-access::combobox o (frame)
			   (with-access::frame frame (relief)
			      (set! relief v)))))))
	       
	    (combobox-init ::combobox)))


;*---------------------------------------------------------------------*/
;*    Global parameters                                                */
;*---------------------------------------------------------------------*/

(define *max-item-in-combo* 8)
(define *combo-down-arrow*  (instantiate::image (file "down_arrow_8.bm")))
(define *combo-up-arrow*    (instantiate::image (file "up_arrow_8.bm")))
(define *combo-focus*	    '())

;*---------------------------------------------------------------------*/
;*                                                                     */
;* THE all-in-one function which deals with opening and closing the    */
;* associated listbox                                                  */
;*                                                                     */
;*---------------------------------------------------------------------*/
(define (show/hide-combo-list combo)
  (let ((visible (combobox-%visible combo))
	(top     (combobox-top-level combo)))
    (if visible
	;; Close the listbox window
	(begin
	  (withdraw top)
	  (button-image-set! (combobox-button combo) *combo-down-arrow*))
	
	;; Open it
	(let* ((f	 (combobox-frame combo))
	       (l	 (combobox-listbox combo))
	       (posx     (winfo 'rootx f))
	       (posy     (winfo 'rooty f))
	       (height   (winfo 'height f))
	       (width    (winfo 'width f))
	       (card	 (length (combobox-values combo)))
	       (nbitems  0))
			
	  ;; Set geometry and width/height of the top-level window
	  (if (> card *max-item-in-combo*)
	      (begin
		;; Too much item in the listbox. Use a scrollbar
		(scrollbox-height-set! l (- *max-item-in-combo* 1))
		(scrollbox-v-scroll-side-set! l "right"))
	      (begin
		;; Use a listbox which is just the good size and no scrollbar
		(scrollbox-height-set! l card)
		(scrollbox-v-scroll-side-set! l #f)))

	  (update) ; to have a correct height for l (Tk problem)!!!
	  (wm 'geometry top (string-append (number->string width)
					   "x"
					   (number->string (winfo 'height l))
					   "+"
					   (number->string posx)
					   "+"
					   (number->string (+ posy height))))
	  ;; Display the listbox
	  (wm 'deiconify top)
	  (raise top)

	  ;; Associate all the bindings
	  (let* ((do-action (lambda ()
			       (let ((command (combobox-command combo)))
				  ;; If the user has specified a command, do it
				  (if (procedure? command)
				      (if (correct-arity? command 1)
					  (command (combobox-value combo))
					  (error "combobox"
						 "Illegal procedure"
						 command)))
				  ;; Release the grab set on the top-level
				  (grab 'release top)
				  (focus
				   (if (null? *combo-focus*)
				       ""
				       *combo-focus*))
				  ;; Close the associated top-level
				  (show/hide-combo-list combo))))
		 (select   (lambda ()
			      (let ((v "")
				    (s (combobox-state combo)))
				 (unless (catch (set! v (selection 'get)))
				    (combobox-state-set! combo "normal")
				    (combobox-value-set! combo v)
				    (combobox-state-set! combo s)
				    (do-action)))))
		 )

	    (add-binding! top "<Leave>" ()
		    (add-binding! top "<Any-ButtonRelease>" ()
			   (do-action)))

	    (add-binding! top "<Enter>" ()
		 (let ((lb (scrollbox-listbox l)))
		   (remove-binding! top "<Any-ButtonRelease>")
		   (set! *combo-focus* (focus :displayof top))
		   (focus lb)
		   (add-binding! lb "<Any-ButtonRelease>" () (select))
		   (add-binding! lb "<Return>" () (select)))))

	  ;; We need a global grab (otherwise displacement of the
	  ;; entry with the wm lets the listbox in place...
	  (grab :global top)

	  ;; Change the bitmap of the menu button to an up arrow
	  (button-image-set! (combobox-button combo) *combo-up-arrow*)))

    (combobox-%visible-set! combo (not visible))))

;*---------------------------------------------------------------------*/
;*    combobox-init ...                                                */
;*---------------------------------------------------------------------*/
(define (combobox-init widget::combobox)
   (with-access::combobox widget (id eid parent frame entry listbox top-level button)
      (set! frame (if (eq? parent #unspecified)
		      (instantiate::frame
			 (class "Combobox")
			 (border-width 2)
			 (relief "ridge"))
		      (instantiate::frame
			 (parent parent)
			 (class "Combobox")
			 (border-width 2)
			 (relief "ridge"))))
      (set! top-level (instantiate::top-level
		         (highlight-thickness 0)
			 (border-width 0)))
      (set! listbox  (instantiate::scrollbox
		        (parent top-level)
			(highlight-thickness 0)
			(border-width 0)
			(scrollbar-width 10)))
      (set! entry    (instantiate::entry
		        (parent frame)
			(highlight-thickness 0)
			(relief "flat")))
      (set! button   (instantiate::button
		        (parent frame)
			(highlight-thickness 0)
			(image *combo-down-arrow*)
			(border-width 1)
			(command (lambda () (show/hide-combo-list widget)))))

      (let ((do-action (lambda ()
			       (let ((command (combobox-command widget)))
				  ;; If the user has specified a command, do it
				  (if (procedure? command)
				      (if (correct-arity? command 1)
					  (command (combobox-value widget))
					  (error "combobox"
						 "Illegal procedure"
						 command)))
				  ;; Release the grab set on the top-level
				  (grab 'release top-level)
				  (focus
				   (if (null? *combo-focus*)
				       ""
				       *combo-focus*))))))
	 (add-binding! entry "<Return>" ()
		       (let ((v (entry-value (combobox-entry widget)))
			     (s (combobox-state widget)))
			  (combobox-state-set! widget "normal")
			  (combobox-value-set! widget v)
			  (combobox-state-set! widget s)
			  (do-action))))

      ;; Pack the visible parts
      (pack entry  :expand #t :fill 'x :side 'left)
      (pack button :expand #f :fill 'y :side 'left :ipadx 2)
      
      ;; Pack the listbox and make it transient
      (pack listbox :expand #t :fill "both")
      (make-transient top-level)

      ;; Initialize true slots
      (set! id	(entry-id entry))
      (set! eid (frame-id frame))

      ;; the command -> tk-object connection
      (tk-command-object-set! eid widget)))

;*---------------------------------------------------------------------*/
;*    X11 resources                                                    */
;*---------------------------------------------------------------------*/
(option 'add "*Combobox.Entry.Background" "white" "widgetDefault")

	       
