;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bglk/runtime/Lwidget/emacs.scm              */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Fri Jun  4 15:36:24 1999                          */
;*    Last change :  Sun Jun  6 16:11:09 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The emacs-like top-levels.                                       */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_emacs
   
   (import  (__biglook_glue          "Llib/scmglue.scm")
	    (__biglook_misc          "Llib/misc.scm")
	    (__biglook_tk-object     "Llib/tk-object.scm")
	    (__biglook_frame         "Lwidget/frame.scm")
	    (__biglook_button        "Lwidget/button.scm")
	    (__biglook_entry         "Lwidget/entry.scm")
	    (__biglook_toolbar       "Lwidget/toolbar.scm")
	    (__biglook_menu          "Lwidget/menu.scm"))
   
   (export  (class emacs::top-level
	       (emacs-init)
	       ;; emacs private fields
	       (%toolbar          (default #f))
	       (%menu             (default #f))
	       (%modeline         (default #f))
	       (%modeline-left    (default #unspecified))
	       (%modeline-root    (default #unspecified))
	       (%modeline-file    (default #unspecified))
	       (%modeline-mode    (default #unspecified))
	       (%modeline-right   (default #unspecified))
	       (%minibuffer-label (default #f))
	       (%minibuffer-entry (default #f))
	       ;; the background color
	       (background
		(get (lambda (o)
			(tk o 'cget :background)))
		(set (lambda (o v)
			(tk o 'configure :background v)
			(with-access::emacs o (%minibuffer-label
					       %minibuffer-entry)
			   (label-background-set! %minibuffer-label v)
			   (entry-background-set! %minibuffer-entry v)))))
	       ;; the toolbar
	       (toolbar
		(get (lambda (o) #unspecified))
		(set (lambda (o spec)
			(with-access::emacs o (%toolbar)
			   (let ((bg #f))
			      (if (toolbar? %toolbar)
				  (begin
				     (set! bg (toolbar-background %toolbar))
				     (error "Not implemented yet"
					    #unspecified
					    #unspecified)))
			      (if spec
				  (begin
				     (set! %toolbar
					   (instantiate::toolbar
					      (background (if bg bg (emacs-default-toolbar-background)))
					      (parent o)))
				     ;; we simply initialize the toolbar with
				     ;; the user specification
				     (fill-toolbar! %toolbar spec)
				     ;; we still have to pack the new toolbar
				     (let ((l (pack 'slaves o)))
					(if (pair? l)
	 				    (pack %toolbar :side 'top
						  :fill 'x
						  :before (car l))
					    (pack %toolbar :side 'top
						  :fill 'x))))))))))
	       (toolbar-background
		(get (lambda (o)
			(with-access::emacs o (%toolbar)
			   (if (toolbar? %toolbar)
			       (toolbar-background %toolbar)
			       #f))))
		(set (lambda (o val)
			(with-access::emacs o (%toolbar)
			   (if (toolbar? %toolbar)
			       (begin
				  (toolbar-background-set! %toolbar val)
				  (toolbar-components-background-set!
				   %toolbar val)))))))
	       ;; the menu
	       (menu
		(get (lambda (o) #unspecified))
		(set (lambda (o spec)
			(with-access::emacs o (%menu)
			   (let ((bg #f)
				 (fg #f))
			      (if (toolbar? %menu)
				  (begin
				     (set! bg (toolbar-background %menu))
				     (set! fg (toolbar-components-foreground
					       %menu))
				     (error "Not implemented yet"
					    #unspecified
					    #unspecified)))
			      (if spec
				  (begin
				     (set! %menu
					   (instantiate::toolbar
					      (background (if bg bg (emacs-default-menu-background)))
					      (parent o)
					      (relief "raised")
					      (border-width 2)
					      (highlight-thickness 0)))
				     ;; we simply initialize the toolbar with
				     ;; the user specification
				     (fill-toolbar! %menu spec)
				     ;; we setup the correct coloring
				     (toolbar-components-foreground-set!
				      %menu
				      (if fg
					  fg
					  (emacs-default-menu-foreground)))
				     (toolbar-components-active-foreground-set!
				      %menu
				      (if fg
					  fg
					  (emacs-default-menu-foreground)))
;* 				     (toolbar-components-active-background-set! */
;* 				      %menu                            */
;* 				      (if bg                           */
;* 					  bg                           */
;* 					  (emacs-default-menu-background))) */
				     ;; we still have to pack the new toolbar
				     (let ((l (pack 'slaves o)))
					(if (pair? l)
					    (pack %menu :side 'top
						  :fill 'x
						  :before (car l))
					    (pack %menu :side 'top
						  :fill 'x))))))))))
	       (menu-background
		(get (lambda (o)
			(with-access::emacs o (%menu)
			   (if (toolbar? %menu)
			       (toolbar-background %menu)
			       #f))))
		(set (lambda (o val)
			(with-access::emacs o (%menu)
			   (if (toolbar? %menu)
			       (begin
				  (toolbar-background-set! %menu val)
				  (toolbar-components-background-set!
				   %menu val)))))))
	       (menu-foreground
		(get (lambda (o)
			(with-access::emacs o (%menu)
			   (if (toolbar? %menu)
			       (toolbar-components-foreground %menu)
			       #f))))
		(set (lambda (o val)
			(with-access::emacs o (%menu)
			   (if (toolbar? %menu)
			       (toolbar-components-foreground-set! %menu
								   val))))))
	       (menu-font
		(get (lambda (o)
			(with-access::emacs o (%menu)
			   (if (toolbar? %menu)
			       (toolbar-components-font %menu)
			       #f))))
		(set (lambda (o val)
			(with-access::emacs o (%menu)
			   (if (toolbar? %menu)
			       (toolbar-components-font-set! %menu val))))))
	       ;; the minibuffer
	       (minibuffer-font
		(get (lambda (o)
			(with-access::emacs o (%minibuffer-label)
			      (label-font %minibuffer-label))))
		(set (lambda (o val)
			(with-access::emacs o (%minibuffer-label
					       %minibuffer-entry)
			      (label-font-set! %minibuffer-label val)
			      (entry-font-set! %minibuffer-entry val)))))
	       (minibuffer-foreground
		(get (lambda (o)
			(with-access::emacs o (%minibuffer-label)
			      (label-foreground %minibuffer-label))))
		(set (lambda (o v)
			(with-access::emacs o (%minibuffer-label
					       %minibuffer-entry)
			      (label-foreground-set! %minibuffer-label v)
			      (entry-foreground-set! %minibuffer-entry v)))))
	       ;; modeline
	       (modeline-background
		(get (lambda (o)
			(with-access::emacs o (%modeline)
			   (frame-background %modeline))))
		(set (lambda (o val)
			(with-access::emacs o (%modeline
					       %modeline-left
					       %modeline-root
					       %modeline-file
					       %modeline-mode
					       %modeline-right
					       %minibuffer-label
					       %minibuffer-entry)
			   (frame-background-set! %modeline val)
			   (label-background-set! %modeline-left val)
			   (label-background-set! %modeline-root val)
			   (label-background-set! %modeline-file val)
			   (label-background-set! %modeline-mode val)
			   (label-background-set! %modeline-right val)))))
	       ;; modeline left
	       (modeline-left-foreground
		(get (lambda (o)
			(with-access::emacs o (%modeline-left)
			   (label-foreground %modeline-left))))
		(set (lambda (o val)
			(with-access::emacs o (%modeline-left)
			   (label-foreground-set! %modeline-left val)))))
	       ;; modeline root
	       (modeline-root
		(get (lambda (o)
			(with-access::emacs o (%modeline-root)
			   (label-text %modeline-root))))
		(set (lambda (o val)
			(with-access::emacs o (%modeline-root)
			   (label-text-set! %modeline-root val)))))
	       (modeline-root-foreground
		(get (lambda (o)
			(with-access::emacs o (%modeline-root)
			   (label-foreground %modeline-root))))
		(set (lambda (o val)
			(with-access::emacs o (%modeline-root)
			   (label-foreground-set! %modeline-root val)))))
	       (modeline-root-font
		(get (lambda (o)
			(with-access::emacs o (%modeline-root)
			   (label-font %modeline-root))))
		(set (lambda (o val)
			(with-access::emacs o (%modeline-root)
			   (label-font-set! %modeline-root val)))))
	       ;; modeline file
	       (modeline-file
		(get (lambda (o)
			(with-access::emacs o (%modeline-file)
			   (label-text %modeline-file))))
		(set (lambda (o val)
			(with-access::emacs o (%modeline-file)
			   (let ((text (string-append ":  " val "      (")))
			      (label-text-set! %modeline-file text))))))
	       (modeline-file-foreground
		(get (lambda (o)
			(with-access::emacs o (%modeline-file)
			   (label-foreground %modeline-file))))
		(set (lambda (o val)
			(with-access::emacs o (%modeline-file)
			   (label-foreground-set! %modeline-file val)))))
	       (modeline-file-font
		(get (lambda (o)
			(with-access::emacs o (%modeline-file)
			   (label-font %modeline-file))))
		(set (lambda (o val)
			(with-access::emacs o (%modeline-file)
			   (label-font-set! %modeline-file val)))))
	       ;; modeline mode
	       (modeline-mode
		(get (lambda (o)
			(with-access::emacs o (%modeline-mode)
			   (label-text %modeline-mode))))
		(set (lambda (o val)
			(with-access::emacs o (%modeline-mode)
			   (label-text-set! %modeline-mode val)))))
	       (modeline-mode-foreground
		(get (lambda (o)
			(with-access::emacs o (%modeline-mode)
			   (label-foreground %modeline-mode))))
		(set (lambda (o val)
			(with-access::emacs o (%modeline-mode)
			   (label-foreground-set! %modeline-mode val)))))
	       (modeline-mode-font
		(get (lambda (o)
			(with-access::emacs o (%modeline-mode)
			   (label-font %modeline-mode))))
		(set (lambda (o val)
			(with-access::emacs o (%modeline-mode)
			   (label-font-set! %modeline-mode val)))))
	       ;; modeline right
	       (modeline-right
		(get (lambda (o)
			(with-access::emacs o (%modeline-right)
			   (label-text %modeline-right))))
		(set (lambda (o val)
			(with-access::emacs o (%modeline-right)
			   (label-text-set! %modeline-right val)))))
	       (modeline-right-foreground
		(get (lambda (o)
			(with-access::emacs o (%modeline-right)
			   (label-foreground %modeline-right))))
		(set (lambda (o val)
			(with-access::emacs o (%modeline-right)
			   (label-foreground-set! %modeline-right val)))))
	       (modeline-right-font
		(get (lambda (o)
			(with-access::emacs o (%modeline-right)
			   (label-font %modeline-right))))
		(set (lambda (o val)
			(with-access::emacs o (%modeline-right)
			   (label-font-set! %modeline-right val))))))

	    (emacs-init ::emacs)
	    (emacs-message ::emacs . obj)
	    (emacs-set-key! emacs::emacs ::bstring ::procedure)
	    (emacs-read-string ::emacs ::bstring ::procedure . obj)))
				  
;*---------------------------------------------------------------------*/
;*    emacs-init ...                                                   */
;*---------------------------------------------------------------------*/
(define (emacs-init widget)
   (with-access::emacs widget (id eid)
      (top-level-init widget)
      ;; we have to prevent inner object to make the top level to expanse
      (pack 'propagate widget #f)
      ;; the default title
      (top-level-title-set! widget "bemacs")
      ;; we create the minibuffer
      (emacs-init-minibuffer! widget)
      ;; we create an default modeline
      (emacs-init-modeline! widget)))

;*---------------------------------------------------------------------*/
;*    emacs-init-modeline! ...                                         */
;*---------------------------------------------------------------------*/
(define (emacs-init-modeline! el)
   (with-access::emacs el (%modeline
			   %modeline-left
			   %modeline-root
			   %modeline-file
			   %modeline-mode
			   %modeline-right)
      ;; the modeline frame
      (set! %modeline
	    (instantiate::frame
	       (background (emacs-default-modeline-background))
	       (parent el)))
      ;; the modline left
      (set! %modeline-left
	    (instantiate::label
	       (parent %modeline)
	       (border-width 0)
	       (font (emacs-default-modeline-font))
	       (background (emacs-default-modeline-background))
	       (foreground (emacs-default-modeline-left-foreground))
	       (text "-----")))
      (set! %modeline-root
	    (instantiate::label
	       (parent %modeline)
	       (font (emacs-default-modeline-font))
	       (background (emacs-default-modeline-background))
	       (foreground (emacs-default-modeline-root-foreground))
	       (text "bemacs")
	       (border-width 0)))
      (set! %modeline-file
	    (instantiate::label
	       (parent %modeline)
	       (font (emacs-default-modeline-font))
	       (background (emacs-default-modeline-background))
	       (foreground (emacs-default-modeline-file-foreground))
	       (border-width 0)
	       (text ":   (")))
      (set! %modeline-mode
	    (instantiate::label
	       (parent %modeline)
	       (border-width 0)
	       (font (emacs-default-modeline-font))
	       (background (emacs-default-modeline-background))
	       (foreground (emacs-default-modeline-mode-foreground))
	       (text "")))
      (set! %modeline-right
	    (instantiate::label
	       (parent %modeline)
	       (border-width 0)
	       (anchor "w")
	       (font (emacs-default-modeline-font))
	       (background (emacs-default-modeline-background))
	       (foreground (emacs-default-modeline-right-foreground))
	       (text ")------------------------------------------------------------------------------------------------------------------------------")))
      (pack %modeline-left
	    %modeline-root
	    %modeline-file
	    %modeline-mode
	    %modeline-right
	    :side 'left :padx 0 :pady 0)
      (pack %modeline :side 'bottom :pady 0 :padx 3 :fill 'x)))

;*---------------------------------------------------------------------*/
;*    emacs-init-minibuffer! ...                                       */
;*---------------------------------------------------------------------*/
(define (emacs-init-minibuffer! el)
   (with-access::emacs el (%minibuffer-label %minibuffer-entry)
      (let ((frame (instantiate::frame
		      (background   (emacs-default-background))
		      (border-width 0)
		      (parent       el))))
	 (set! %minibuffer-label
	       (instantiate::label
		  (parent       frame)
		  (background   (emacs-default-background))
		  (foreground   (emacs-default-foreground))
		  (font         (emacs-default-minibuffer-font))
		  (border-width 0)
		  (relief       "flat")
		  (width        0)
		  (text         "")))
	 (set! %minibuffer-entry
	       (instantiate::entry
		  (parent              frame)
		  (background          (emacs-default-background))
		  (foreground          (emacs-default-foreground))
		  (border-width        0)
		  (highlight-thickness 0)
		  (relief              "flat")
		  (state               'disabled)
		  (font                (emacs-default-minibuffer-font))
		  (string-value        #t)
		  (value               "")))
	 ;; we pack the all things
	 (pack %minibuffer-label :side 'left)
	 (pack %minibuffer-entry :side 'left :fill 'x :expand #t)
	 (pack frame :side 'bottom :fill 'x :pady 0))))

		
;*---------------------------------------------------------------------*/
;*    Default configuration                                            */
;*---------------------------------------------------------------------*/
(define (emacs-default-background)
   "oldlace")

(define (emacs-default-foreground)
   "black")

(define (emacs-default-toolbar-background)
   "gray")

(define (emacs-default-menu-background)
   "gray")

(define (emacs-default-menu-foreground)
   "darkmagenta")

(define (emacs-default-modeline-background)
   "gray80")

(define (emacs-default-modeline-left-foreground)
   "orange")

(define (emacs-default-modeline-root-foreground)
   "red")

(define (emacs-default-modeline-file-foreground)
   "darkmagenta")

(define (emacs-default-modeline-mode-foreground)
   "orange")

(define (emacs-default-modeline-right-foreground)
   "darkmagenta")

(define (emacs-default-menu-font)
   "lucidasanstypewriter-bold-12")

(define (emacs-default-modeline-font)
   "lucidasanstypewriter-bold-12")

(define (emacs-default-minibuffer-font)
   "lucidasanstypewriter-12")

;*---------------------------------------------------------------------*/
;*    emacs-message ...                                                */
;*---------------------------------------------------------------------*/
(define (emacs-message emacs::emacs . obj)
   (let ((port (open-output-string)))
      (for-each (lambda (o) (display o port)) obj)
      (let ((str (close-output-port port)))
	 (with-access::emacs emacs (%minibuffer-label %minibuffer-entry)
	    (with-access::entry %minibuffer-entry (state value)
	       (set! state 'normal)
	       (set! value "")
	       (set! state 'disabled))
	    (with-access::label %minibuffer-label (text)
	       (set! text str))))))

;*---------------------------------------------------------------------*/
;*    emacs-read-string ...                                            */
;*---------------------------------------------------------------------*/
(define (emacs-read-string emacs::emacs prompt::bstring k::procedure . default)
   (with-access::emacs emacs (%minibuffer-label %minibuffer-entry)
      (let ((default (if (and (pair? default) (string? (car default)))
			 (car default)
			 "")))
	 (with-access::label %minibuffer-label (text)
	    (set! text (string-append prompt ": "))
	    (with-access::entry %minibuffer-entry (state value)
	       (set! state 'normal)
	       (set! value default)
	       (focus %minibuffer-entry)
	       (bind %minibuffer-entry
		     "<Return>"
		     (lambda ()
			(let ((res (if (=fx (string-length value) 0)
				       default
				       value)))
			   (emacs-message emacs "")
			   (set! state 'disabled)
			   (k res)))))))))

;*---------------------------------------------------------------------*/
;*    emacs-set-key! ...                                               */
;*---------------------------------------------------------------------*/
(define (emacs-set-key! emacs::emacs key::bstring callback::procedure)
   (bind emacs key (lambda () (callback emacs))))
