;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bglk/runtime/Lwidget/entry.scm              */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Tue Oct 20 17:39:15 1998                          */
;*    Last change :  Fri Apr  9 09:59:24 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The entry widget                                                 */
;*    -------------------------------------------------------------    */
;*    Source documentation:                                            */
;*       @path ../../manual/entry.texi@                                */
;*       @node Entry@                                                  */
;*    Example:                                                         */
;*       @path ../../demos/entry/entry.scm@                            */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_entry
   
   (include "Llib/biglook.sch")
   
   (option  (loadq "Llib/biglook.init"))
   
   (import  (__biglook_glue      "Llib/scmglue.scm")
	    (__biglook_misc      "Llib/misc.scm")
	    (__biglook_tk-object "Llib/tk-object.scm"))
   
   (export  (class entry::simple-widget
	       ;; entry constructor
	       (entry-init)
	       ;; string-value
	       (string-value
		(get (lambda (o)
			(tk o 'cget :stringvalue)))
		(set (lambda (o v)
			(tk o 'configure :stringvalue v))))
	       ;; foreground
	       (foreground
		(get (lambda (o)
			(tk o 'cget :foreground)))
		(set (lambda (o v)
			(tk o 'configure :foreground v))))
	       ;; select-foreground
	       (select-foreground
		(get (lambda (o)
			(tk o 'cget :selectforeground)))
		(set (lambda (o v)
			(tk o 'configure :selectforeground v))))
	       ;; select-background
	       (select-background
		(get (lambda (o)
			(tk o 'cget :selectbackground)))
		(set (lambda (o v)
			(tk o 'configure :selectbackground v))))
	       ;; font
	       (font
		(get (lambda (o)
			(tk o 'cget :font)))
		(set (lambda (o v)
			(tk o 'configure :font v))))
	       ;; export-selection
	       (export-selection
		(get (lambda (o)
			(tk o 'cget :exportselection)))
		(set (lambda (o v)
			(tk o 'configure :exportselection v))))
	       ;; virtual value
	       (value
		(get (lambda (o)  
			(tk o 'get)))
		(set (lambda (o v)
			;; first delete all present chars
			(tk o 'delete 0 'end)
			;; then insert new text
			(tk o 'insert 0 v))))
	       ;; show
	       (show
		(get (lambda (o)
			(tk o 'cget :show)))
		(set (lambda (o v)
			(tk o 'configure :show v))))
	       ;; state
	       (state 
		(get (lambda (o)
			(tk o 'cget :state)))
		(set (lambda (o v)
			(tk o 'configure :state v))))
	       ;; width
	       (width
		(get (lambda (o)
			(tk o 'cget :width)))
		(set (lambda (o v)
			(tk o 'configure :width v)))))

	    entry-init))

;*---------------------------------------------------------------------*/
;*    entry-init ...                                                   */
;*---------------------------------------------------------------------*/
(define entry-init
   (let ((tk-entry (make-widget-constructor "entry")))
      (lambda (o)
	 (simple-widget-init o tk-entry))))

;*---------------------------------------------------------------------*/
;*    Global parameters                                                */
;*---------------------------------------------------------------------*/
(define press-x     0)
(define press-y     0)
(define relief      "sunken")
(define mouse-moved #f)
(define select-mode "char")

;*---------------------------------------------------------------------*/
;*    format@~A ...                                                    */
;*---------------------------------------------------------------------*/
(define (format@~A x)
   (let ((port (open-output-string)))
      (display "@" port)
      (display x port)
      (close-output-port port)))

;; ----------------------------------------------------------------------
;; Class bindings for entry widgets.
;; ----------------------------------------------------------------------
;;
;; Elements used in this file:
;;
;; *after-id* -		If non-null, it means that auto-scanning is underway
;;			and it gives the "after" id for the next auto-scan
;;			command to be executed.
;; mouse-moved -	Non-zero means the mouse has moved a significant
;;			amount since the button went down (so, for example,
;;			start dragging out a selection).
;; press-x -		X-coordinate at which the mouse button was pressed.
;; select-mode -	The style of selection currently underway:
;;			char, word, or line.
;; x, y -		Last known mouse coordinates for scanning
;;			and auto-scanning.
;;-------------------------------------------------------------------------


;;
;; Utilities
;;
(define (word-character c)
  (or (char-alphabetic? c)
      (char-numeric? c)
      (char=? c #\-)
      (char=? c #\*)))

(define (word-separator c)
  (not (word-character c)))

(define (end-of-word str index . separator)
  (let ((len (string-length str))
	(sep (if (null? separator) word-separator separator)))
    (let loop ((i index))
      (cond
         ((= len 0)			0)
         ((>= i len) 			len)
	 ((sep (string-ref str i))	(if (= index i) (+ i 1) i))
	 (ELSE				(loop (+ i 1)))))))

(define (beginning-of-word str index . separator)
  (let ((len (string-length str))
	(sep (if (null? separator) word-separator separator)))
    (let loop ((i index))
      (cond
         ((= len 0)			0)
         ((= i -1) 			0)
	 ((>= i len)			(loop (- len 1)))
	 ((sep (string-ref str i)) 	(if (= index i) i (+ i 1)))
	 (ELSE 				(loop (- i 1)))))))

;=============================================================================

;; entry-button-1 --
;; This procedure is invoked to handle button-1 presses in "Entry"
;; widgets.  It moves the insertion cursor, sets the selection anchor,
;; and claims the input focus.
;;
;; Arguments:
;; w -		The "Entry" window in which the button was pressed.
;; x -		The x-coordinate of the button press.

(define (entry-button-1 w x)
  (let ((pos (format@~A x)))
    (set! select-mode "char")
    (set! mouse-moved #f)
    (set! press-x x)
    (w 'icursor (entry-closest-gap w x))
    (w 'selection 'from 'insert)
    (if (equal? (tk-get w :state) "normal")
	(focus w))))

;; entry-mouse-select --
;; This procedure is invoked when dragging out a selection with
;; the mouse.  Depending on the selection mode (character, word,
;; line) it selects in different-sized units.  This procedure
;; ignores mouse motions initially until the mouse has moved from
;; one character to another or until there have been multiple clicks.
;;
;; Arguments:
;; w -		The "Entry" window in which the button was pressed.
;; x -		The x-coordinate of the mouse.

(define (entry-mouse-select w x)
  (let* ((cur    (entry-closest-gap w x))
	 (anchor (w 'index 'anchor)))

    (if (or (equal? cur anchor)  
	    (>= (abs (- press-x x)) 3))
	(set! mouse-moved #t))

    (cond
      ((string=? select-mode "char")
       		(if mouse-moved
		    (cond
		      ((< cur anchor) (w 'selection 'range cur anchor))
		      ((> cur anchor) (w 'selection 'range anchor cur))
		      (ELSE	      (w 'selection 'clear)))))
      ((string=? select-mode "word")
                (if (< cur (w 'index 'anchor))
		    (w 'selection 'range 
		       (beginning-of-word (w 'get) cur)
		       (end-of-word       (w 'get) (- anchor 1)))
		    (w 'selection 'range 
		       (beginning-of-word (w 'get) anchor)
		       (end-of-word       (w 'get) (- cur 1)))))
      ((string=? select-mode "line")
	    	(w 'selection 'range 0 'end)))

    (update 'idletasks)))


;; entry-auto-scan --
;; This procedure is invoked when the mouse leaves an "Entry" window
;; with button 1 down.  It scrolls the window left or right,
;; depending on where the mouse is, and reschedules itself as an
;; "after" command so that the window continues to scroll until the
;; mouse moves back into the window or the mouse button is released.
;;
;; Arguments:
;; w -	The "Entry" window.

(define (entry-auto-scan w)
  (when (winfo 'exists w)
    (let ((x *x*))
      (if (>= x  (winfo 'width w))
	  (begin
	    (w 'xview 'scroll 2 'units)
	    (entry-mouse-select w x))
	  (if (< x 0)
	      (w 'xview 'scroll -2 'units)
	      (entry-mouse-select w x)))
      
      (set! *after-id* (after 50 (lambda ()
				     (entry-auto-scan w)))))))

;; entry-key-select --
;; This procedure is invoked when stroking out selections using the
;; keyboard.  It moves the cursor to a new position, then extends
;; the selection to that position.
;;
;; Arguments:
;; w -		The "Entry" window.
;; new -	A new position for the insertion cursor (the cursor hasn't
;;		actually been moved to this position yet).

(define (entry-key-select w new)
  (if (w 'selection 'present)
      (w 'selection 'adjust new)
      (begin
	(w 'selection 'from 'insert)
	(w 'selection 'to new)))
  (w 'icursor new))

;; entry-Insert --
;; Insert a string into an "Entry" at the point of the insertion cursor.
;; If there is a selection in the "Entry", and it covers the point of the
;; insertion cursor, then delete the selection before inserting.
;;
;; Arguments:
;; w -	The "Entry" window in which to insert the string
;; s -		The string to insert (usually just a single character)

(define (entry-insert w s)
  (unless (equal? s "")
     (let ((insert (w 'index 'insert)))
       (catch
	   (if (and (<= (w 'index 'sel.first) insert)
		    (>= (w 'index 'sel.last)  insert))
	       (w 'delete 'sel.first 'sel.last))))
     (w 'insert 'insert s)
     (entry-see-insert w)))

;; entry-backspace --
;; Backspace over the character just before the insertion cursor.
;; If backspacing would move the cursor off the left edge of the
;; window, reposition the cursor at about the middle of the window.
;;
;; Arguments:
;; w -		The "Entry" window in which to backspace.

(define (entry-backspace w)
  (if (w 'selection 'present)
      (w 'delete 'sel.first 'sel.last)
      (let ((x (- (w 'index 'insert) 1)))
	(if (>= x 0) (w 'delete x))
	(when (>= (w 'index "@0") 
		  (w 'index 'insert))
	  (let* ((range (w 'xview))
		 (left  (car range))
		 (right (cadr range)))
	    (w 'xview 'moveto (- left (/ (- right left) 2.0))))))))

;; entry-see-insert --
;; Make sure that the insertion cursor is visible in the "Entry" window.
;; If not, adjust the view so that it is.
;;
;; Arguments:
;; w -		The "Entry" window.

(define (entry-see-insert w)
  (let ((c    (w 'index 'insert))
	(left (w 'index "@0")))
    
    (if (> left c)
	(w 'xview c)
	(let ((x (winfo 'width w)))
	  (while (and (<= (w 'index (format@~A x)) c)
		      (< left c))
	     (set! left (+ left 1))
	     (w 'xview left))))))

;; entry-set-cursor -
;; Move the insertion cursor to a given position in an "Entry".  Also
;; clears the selection, if there is one in the "Entry", and makes sure
;; that the insertion cursor is visible.
;;
;; Arguments:
;; w -		The "Entry" window.
;; pos -	The desired new position for the cursor in the window.

(define (entry-set-cursor w pos)
  (w 'icursor pos)
  (w 'selection 'clear)
  (entry-see-insert w))

;; entry-Transpose -
;; This procedure implements the "transpose" function for "Entry" widgets.
;; It tranposes the characters on either side of the insertion cursor,
;; unless the cursor is at the end of the line.  In this case it
;; transposes the two characters to the left of the cursor.  In either
;; case, the cursor ends up to the right of the transposed characters.
;;
;; w -		The "Entry" window.

(define (entry-transpose w)
  (let ((i (w 'index 'insert)))
    (if (< i (w 'index 'end))
	(set! i (+ i 1)))
    (let ((first (- i 2)))
      (if (>= first 0)
	  (let* ((str (w 'get))
		 (new (string (string-ref str (- i 1)) (string-ref str first))))
	    (w 'delete first i)
	    (w 'insert 'insert new)
	    (entry-see-insert w))))))

;; entry-closest-gap --
;; Given x and y coordinates, this procedure finds the closest boundary
;; between characters to the given coordinates and returns the index
;; of the character just after the boundary.
;;
;; w -		The entry window.
;; x -		X-coordinate within the window.

(define (entry-closest-gap w x)
  (let* ((pos  (w 'index (format@~A x)))
	 (bbox (w 'bbox pos)))
    (if (< [- x (list-ref bbox 0)] (/ (list-ref bbox 2) 2))
	pos
	(+ pos 1))))


;; entry-paste --
;; This procedure sets the insertion cursor to the current mouse position,
;; pastes the selection there, and sets the focus to the window.
;;
;; w -		The entry window.
;; x -		X position of the mouse.

(define (entry-paste w x)
  (w 'icursor (entry-closest-gap w x))
  (catch (w 'insert 'insert (selection 'get :displayof w)))
  (if (string=? (tk-get w :state) "normal")
      (focus w)))


;;-------------------------------------------------------------------------
;; The code below creates the default class bindings for entries.
;;-------------------------------------------------------------------------

(add-binding! "Entry" "<<Cut>>" (WW)
  (clipboard 'clear :displayof WW)
  (catch 
     (clipboard 'append :displayof WW 
		(substring (WW 'get)
			   (WW 'index 'sel.first)
			   (WW 'index 'sel.last)))
     (WW 'delete 'sel.first 'sel.last)))

(add-binding! "Entry" "<<Copy>>" (WW)
  (clipboard 'clear :displayof WW)
  (catch 
     (clipboard 'append :displayof WW 
		(substring (WW 'get)
			   (WW 'index 'sel.first)
			   (WW 'index 'sel.last)))))


(add-binding! "Entry" "<<Paste>>" (WW)
  (catch
     (catch (WW 'delete 'sel.first 'sel.last))
     (WW 'insert 'insert (selection 'get :displayof WW 
				     	  :selection "CLIPBOARD"))
     (entry-see-insert WW)))
  
(add-binding! "Entry" "<<Clear>>" (WW)
  (WW 'delete 'sel.first 'sel.last))

(add-binding! "Entry" "<<PasteSelection>>" (WW x)
  (when (or *tk-strict-Motif* (not mouse-moved))
    (entry-paste WW x)))

(add-binding! "Entry" "<1>" (WW x)
  (entry-button-1 WW x)
  (WW 'selection 'clear))

(add-binding! "Entry" "<B1-Motion>" (WW x)
  (set! x x)
  (entry-mouse-select WW x))

(add-binding! "Entry" "<Double-1>" (WW x)
  (set! select-mode "word")
  (entry-mouse-select WW x)
  (catch 
     (WW 'icursor 'sel.first)))

(add-binding! "Entry" "<Triple-1>" (WW x)
  (set! select-mode "line")
  (entry-mouse-select WW x)
  (WW 'icursor 0))

(add-binding! "Entry" "<Shift-1>" (WW x)
  (set! select-mode "char")
  (WW 'selection 'adjust (format@~A x)))

(add-binding! "Entry" "<Double-Shift-1>" (WW x)
  (set! select-mode "word")
  (entry-mouse-select WW x))

(add-binding! "Entry" "<Triple-Shift-1>" (WW x)
  (set! select-mode "line")
  (entry-mouse-select WW x))

(add-binding! "Entry" "<B1-Leave>" (WW x)
  (set! x x)
  (entry-auto-scan WW))

(add-binding! "Entry" "<B1-Enter>" ()
  (cancel-repeat))

(add-binding! "Entry" "<ButtonRelease-1>" ()
  (cancel-repeat))

(add-binding! "Entry" "<Control-1>" (WW x)
  (WW 'icursor (format@~A x)))


(add-binding! "Entry" "<Left>" (WW)
  (entry-set-cursor WW (- (WW 'index 'insert) 1)))

(add-binding! "Entry" "<Right>" (WW)
  (entry-set-cursor WW (+ (WW 'index 'insert) 1)))

(add-binding! "Entry" "<Shift-Left>" (WW)
  (entry-key-select WW (- (WW 'index 'insert) 1))
  (entry-see-insert WW))

(add-binding! "Entry" "<Shift-Right>" (WW)
  (entry-key-select WW (+ (WW 'index 'insert) 1))
  (entry-see-insert WW))

(add-binding! "Entry" "<Control-Left>" (WW)
  (entry-set-cursor WW
		       (beginning-of-word (WW 'get) 
					     (- (WW 'index 'insert) 1))))

(add-binding! "Entry" "<Control-Right>" (WW)
  (entry-set-cursor WW (end-of-word (WW 'get) (WW 'index 'insert))))

(add-binding! "Entry" "<Shift-Control-Left>" (WW)
  (entry-key-select WW
			(beginning-of-word (WW 'get) 
					     (- (WW 'index 'insert) 1)))
  (entry-see-insert WW))

(add-binding! "Entry" "<Shift-Control-Right>" (WW)
  (entry-key-select WW (end-of-word (WW 'get) (WW 'index 'insert)))
  (entry-see-insert WW))

(add-binding! "Entry" "<Home>" (WW)
  (entry-set-cursor WW 0))

(add-binding! "Entry" "<Shift-Home>" (WW)
  (entry-key-select WW 0)
  (entry-see-insert WW))

(add-binding! "Entry" "<End>" (WW)
  (entry-set-cursor WW 'end))

(add-binding! "Entry" "<Shift-End>" (WW)
  (entry-key-select WW 'end)
  (entry-see-insert WW))

(add-binding! "Entry" "<Delete>" (WW)
  (if (WW 'selection 'present)
      (WW 'delete 'sel.first 'sel.last)
      (WW 'delete 'insert)))

(add-binding! "Entry" "<BackSpace>" (WW)
  (entry-backspace WW))

(add-binding! "Entry" "<Control-space>" (WW)
  (WW 'selection 'from 'insert))

(add-binding! "Entry" "<Select>" (WW)
  (WW 'selection 'from 'insert))

(add-binding! "Entry" "<Control-Shift-space>" (WW)
  (WW 'selection 'adjust 'insert))

(add-binding! "Entry" "<Shift-Select>" (WW)
  (WW 'selection 'adjust 'insert))

(add-binding! "Entry" "<Control-slash>" (WW)
  (WW 'selection 'range 0 'end))

(add-binding! "Entry" "<Control-backslash>" (WW)
  (WW 'selection 'clear))

(add-binding! "Entry" "<KeyPress>" (WW AA)
  (entry-Insert WW AA))


;; Ignore all Alt, Meta, and Control keypresses unless explicitly bound.
;; Otherwise, if a widget binding for one of these is defined, the
;; <KeyPress> class binding will also fire and insert the character,
;; which is wrong.  Ditto for Escape, Return, and Tab.
(add-binding! "Entry" "<Alt-KeyPress>" () 'nop)
(add-binding! "Entry" "<Meta-KeyPress>" () 'nop)
(add-binding! "Entry" "<Control-KeyPress>" () 'nop)
(add-binding! "Entry" "<Escape>" () 'nop)
(add-binding! "Entry" "<Return>" () 'nop)
(add-binding! "Entry" "<KP_Enter>" () 'nop)
(add-binding! "Entry" "<Tab>" () 'nop)

;; On Windows, paste is done using Shift-Insert.  Shift-Insert already
;; generates the <<Paste>> event, so we don't need to do anything here.
(add-binding! "Entry" "<Insert>" (WW)
   (catch (entry-insert WW (selection 'get :displayof WW))))

;; Additional emacs-like bindings:

(add-binding! "Entry" "<Control-a>" (WW)
  (entry-set-cursor WW 0))

(add-binding! "Entry" "<Control-b>" (WW)
  (entry-set-cursor WW (- (WW 'index 'insert) 1)))

(add-binding! "Entry" "<Control-d>" (WW)
  (WW 'delete 'insert))

(add-binding! "Entry" "<Control-e>" (WW)
  (entry-set-cursor WW 'end))

(add-binding! "Entry" "<Control-f>" (WW)
  (entry-set-cursor WW (+ (WW 'index 'insert) 1)))

(add-binding! "Entry" "<Control-h>" (WW)
  (entry-backspace WW))

(add-binding! "Entry" "<Control-k>" (WW)
  (WW 'delete 'insert 'end))

(add-binding! "Entry" "<Control-t>" (WW)
  (entry-transpose WW))

(add-binding! "Entry" "<Meta-b>" (WW)
 (entry-set-cursor WW
		       (beginning-of-word (WW 'get) 
					     (- (WW 'index 'insert) 1))))

(add-binding! "Entry" "<Meta-d>" (WW)
  (WW 'delete 'insert (end-of-word (WW 'get) (WW 'index 'insert))))

(add-binding! "Entry" "<Meta-f>" (WW)
  (entry-set-cursor WW (end-of-word (WW 'get) (WW 'index 'insert))))

(add-binding! "Entry" "<Meta-BackSpace>" (WW)
  (WW 'delete (beginning-of-word (WW 'get) (- (WW 'index 'insert) 1))
       'insert))

(add-binding! "Entry" "<Meta-Delete>" (WW)
  (WW 'delete (beginning-of-word (WW 'get) (- (WW 'index 'insert) 1))
       'insert))

;; A few additional bindings of my own.

(add-binding! "Entry" "<Shift-2>" (WW x)
  (WW 'scan 'mark x)
  (set! x x)
  (set! mouse-moved #f))

(add-binding! "Entry" "<Shift-B2-Motion>" (WW x)
  (if (> (abs (- x x)) 2)
      (set! mouse-moved #t))
  (WW 'scan 'dragto x))

