;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bglk/runtime/Lwidget/frame.scm              */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Tue Dec 15 16:53:28 1998                          */
;*    Last change :  Fri May  7 17:13:08 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The frame implementation                                         */
;*    -------------------------------------------------------------    */
;*    Source documentations:                                           */
;*       @path ../../manual/frame.texi@                                */
;*       @node Frame@                                                  */
;*       @path ../../manual/toplevel.texi@                             */
;*       @node Toplevel@                                               */
;*    Examples:                                                        */
;*       @path ../../demos/checkbutton/checkbutton.scm@                */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_frame
   
   (import  (__biglook_glue      "Llib/scmglue.scm")
	    (__biglook_misc      "Llib/misc.scm")
	    (__biglook_tk-object "Llib/tk-object.scm"))
   
   (export  (class frame::simple-widget
	       (frame-init)
	       (width::int
		(get (lambda (o)
			(tk o 'cget :width)))
		(set (lambda (o v)
			(tk o 'configure :width v))))
	       (height::int
		(get (lambda (o)
			(tk o 'cget :height)))
		(set (lambda (o v)
			(tk o 'configure :height v))))
	       (class::bstring read-only (default "Frame"))
	       (colormap::bstring read-only (default ""))
	       (container::bbool read-only (default #f))
	       (visual::bstring read-only (default "")))
  	     
	    (class top-level::frame
	       (top-level-init)
	       (menu
		(get (lambda (o)
			((top-level-id o) 'cget :menu)))
		(set (lambda (o v)
			((top-level-id o) 'configure :menu v))))
	       (screen::bstring read-only (default (getenv "DISPLAY")))
	       (use::bstring read-only (default ""))
	       
	       (geometry
		(get (lambda (o) (wm 'geometry (tk-object-eid o))))
		(set (lambda (o v)
			(wm 'geometry (tk-object-eid o) v))))
	       (group
		(get (lambda (o) (wm 'group (tk-object-eid o))))
		(set (lambda (o v)
			(wm 'group (tk-object-eid o) v))))
	       (icon-name
		(get (lambda (o) (wm 'iconnam (tk-object-eid o))))
		(set (lambda (o v) (wm 'iconnam (tk-object-eid o) v))))
	       (icon-window
		(get (lambda (o) (wm 'iconwin (tk-object-eid o))))
		(set (lambda (o v) (wm 'iconwin (tk-object-eid o) v))))
	       (max-size
		(get (lambda (o) (wm 'max (tk-object-eid o))))
		(set (lambda (o v)
			(apply wm 'max (tk-object-eid o) v))))
	       (min-size
		(get (lambda (o) (wm 'min (tk-object-eid o))))
		(set (lambda (o v)
			(apply wm 'min (tk-object-eid o) v))))
	       (state
		read-only
		(get (lambda (o) (wm 'state (tk-object-eid o)))))
	       (title
		(get (lambda (o) (wm 'title (tk-object-eid o))))
		(set (lambda (o v) (wm 'title (tk-object-eid o) v)))))
	    
	    frame-init
	    top-level-init
	    (top-level-root)
	    (deiconify self::top-level)
	    (iconify self::top-level)
	    (withdraw self::top-level)
	    (place-toplevel self::top-level x::int y::int)
	    (make-transient self::top-level)
	    (protocol-get o::top-level str)
	    (protocol-set! o::top-level str fun)
	    (protocol-list o::top-level)))

;*---------------------------------------------------------------------*/
;*    frame-init ...                                                   */
;*---------------------------------------------------------------------*/
(define frame-init
   (let ((frame (make-widget-constructor "frame")))
      (lambda (o)
	 (simple-widget-init
	  o
	  (lambda l
	     (with-access::frame o (class colormap container visual)
		(apply frame
		       :class class
		       :container container
		       :visual visual
		       (if (>fx (string-length colormap) 0)
			   (cons* :colormap colormap l)
			   l))))))))

;*---------------------------------------------------------------------*/
;*    top-level-init ...                                               */
;*---------------------------------------------------------------------*/
(define top-level-init
   (let ((top-level (make-widget-constructor "toplevel")))
      (lambda (o)
	 (simple-widget-init
	  o
	  (lambda l
	     (with-access::frame o (class colormap container visual)
		(apply top-level
		       :class class
		       :container container
		       :visual visual
		       (if (>fx (string-length colormap) 0)
			   (cons* :colormap colormap l)
			   l))))))))

;*---------------------------------------------------------------------*/
;*    *top-level-root* ...                                             */
;*---------------------------------------------------------------------*/
(define *top-level-root*
   (let ((top (allocate-top-level)))
      (with-access::top-level top (id eid)
	 (set! id (root))
	 (set! eid (root))
	 (tk-command-object-set! eid top))))
      
;*---------------------------------------------------------------------*/
;*    top-level-root ...                                               */
;*---------------------------------------------------------------------*/
(define (top-level-root)
   *top-level-root*)
   
;*---------------------------------------------------------------------*/
;*    deiconify ...                                                    */
;*---------------------------------------------------------------------*/
(define (deiconify self::top-level)
   (with-access::top-level self (eid)
      (wm 'deiconify eid)))

;*---------------------------------------------------------------------*/
;*    iconify ...                                                      */
;*---------------------------------------------------------------------*/
(define (iconify self::top-level)
   (with-access::top-level self (eid)
      (wm 'iconify eid)))

;*---------------------------------------------------------------------*/
;*    withdraw ...                                                     */
;*---------------------------------------------------------------------*/
(define (withdraw self::top-level)
   (with-access::top-level self (eid)
      (wm 'withdraw eid)))

;*---------------------------------------------------------------------*/
;*    place-top-level ...                                              */
;*---------------------------------------------------------------------*/
(define (place-toplevel self::top-level x::int y::int)
   (with-access::top-level self (geometry)
      (set! geometry (string-append "+" (integer->string x)
				    "+" (integer->string y)))))

;*---------------------------------------------------------------------*/
;*    make-transient ...                                               */
;*---------------------------------------------------------------------*/
(define (make-transient self::top-level)
   (withdraw self)
   (wm 'over (tk-object-eid self) #t))

;*---------------------------------------------------------------------*/
;*    protocol-get ...                                                 */
;*---------------------------------------------------------------------*/
(define (protocol-get o::top-level str)
   (wm 'proto (tk-object-eid o) str))

;*---------------------------------------------------------------------*/
;*    protocol-set! ...                                                */
;*---------------------------------------------------------------------*/
(define (protocol-set! o::top-level str fun)
   (wm 'proto (tk-object-eid o) str fun))

;*---------------------------------------------------------------------*/
;*    protocol-list ...                                                */
;*---------------------------------------------------------------------*/
(define (protocol-list o::top-level)
   (wm 'proto (tk-object-eid o)))
