;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bglk/runtime/Lwidget/gauge.scm              */
;*    -------------------------------------------------------------    */
;*    Author      :  Erick Gallesio [eg@unice.fr]                      */
;*    Creation    :  17-Oct-1996 14:53                                 */
;*    Last change :  Sun May 30 07:59:01 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    Gauge composite widget                                           */
;*    -------------------------------------------------------------    */
;*    Source documentation:                                            */
;*       @path ../../manual/gauge.texi@                                */
;*       @node Gauge@                                                  */
;*       @path ../../manual/valued-gauge.texi@                         */
;*       @node Valued Gauge@                                           */
;*    Example:                                                         */
;*       @path ../../demos/gauge/gauge.scm@                            */
;*       @path ../../demos/valued-gauge/valued-gauge.scm@              */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_gauge
   
   (include "Llib/biglook.sch")
   
   (option  (loadq "Llib/biglook.init"))
   
   (import  (__biglook_glue      "Llib/scmglue.scm")
	    (__biglook_misc      "Llib/misc.scm")
	    (__biglook_key       "Llib/key.scm")
	    (__biglook_tk-object "Llib/tk-object.scm")
	    (__biglook_entry     "Lwidget/entry.scm")
	    (__biglook_frame     "Lwidget/frame.scm")
	    (__biglook_button    "Lwidget/button.scm"))
    
   (export  (class gauge::simple-widget
	       (init-gauge)
	       ;; priv-box
	       (priv-box (default #unspecified))
	       ;; priv-foreground
	       (priv-foreground (default "#3a5fcd"))
	       ;; priv-value
	       (priv-value (default 0))
	       ;; foreground
	       (foreground
		(get (lambda (o)
			(with-access::gauge o (priv-foreground)
			   priv-foreground)))
		(set (lambda (o v)
			(with-access::gauge o (priv-box priv-foreground)
			   (set! priv-foreground v)
			   (if (not (eq? priv-box #unspecified))
			       (tk o
				   'itemconfigure priv-box
				   :fill v :outline v))))))
	       ;; height
	       (height
		(get (lambda (o)
			(tk o 'cget :height)))
		(set (lambda (o v)
			(tk o 'configure :height v)))
		(default 20))
	       ;; width
	       (width
		(get (lambda (o)
			(tk o 'cget :width)))
		(set (lambda (o v)
			(tk o 'configure :width v))))
	       ;; value
	       (value
		(get (lambda (o)
			(with-access::gauge o (priv-value)
			   priv-value)))
		(set (lambda (o v)
			(with-access::gauge o (priv-value)
			   (set! priv-value v)
			   (configure-gauge o))))))

	    (class valued-gauge::gauge
	       (init-valued-gauge)
	       ;; priv-text
	       (priv-text (default ""))
	       ;; priv-text-foreground
	       (priv-text-foreground (default "black"))
	       ;; priv-text-font
	       (priv-text-font (default '(Helvetica -14 bold)))
	       ;; text-font
	       (text-font
		(get (lambda (o)
			(with-access::valued-gauge o (priv-text-font)
			   priv-text-font)))
		(set (lambda (o v)
			(with-access::valued-gauge o (priv-text-font priv-text)
			   (set! priv-text-font v)
			   (tk o
			       'itemconfigure priv-text
			       :font v)))))
	       ;; text-foreground
	       (text-foreground
		(get (lambda (o)
			(with-access::valued-gauge o (priv-text-foreground)
			   priv-text-foreground)))
		(set (lambda (o v)
			(with-access::valued-gauge o (priv-text priv-text-foreground)
			   (tk o
			       'itemconfigure priv-text
			       :fill v))))))

	    (init-gauge        ::gauge)
	    (init-valued-gauge ::valued-gauge)
	    (configure-gauge   ::gauge)))

;*---------------------------------------------------------------------*/
;*    initialize-gauge ...                                             */
;*---------------------------------------------------------------------*/
(define (initialize-gauge self::gauge)
   (with-access::gauge self (id eid foreground
				highlight-thickness relief priv-box)
      (let ((w (simple-widget-init self (make-widget-constructor "canvas"))))
	 (set! highlight-thickness 0)
	 (set! relief "raised")
	 (set! priv-box (id 'create 'rectangle 0 0 0 0
			    :outline foreground
			    :fill foreground)))))
   
;*---------------------------------------------------------------------*/
;*    init-gauge ...                                                   */
;*---------------------------------------------------------------------*/
(define (init-gauge self::gauge)
   (initialize-gauge self)
   (bind self "<Configure>" (lambda () (configure-gauge self))))

;*---------------------------------------------------------------------*/
;*    configure-gauge ...                                              */
;*---------------------------------------------------------------------*/
(define (configure-gauge self::gauge)
   (define (configure-plain-gauge)
      (with-access::gauge self (priv-box value)
	 (if (not (eq? priv-box #unspecified))
	     (let ((width  (winfo 'width  self))
		   (height (winfo 'height self)))
		(tk self 'coords priv-box
		    0 0 
		    (quotient (* width value) 100) height)))))
   (define (configure-valued-gauge)
      (configure-plain-gauge)
      (with-access::valued-gauge self (id priv-text value)
	 (if (not (eq? priv-text #unspecified))
	     (let* ((bbox  (id 'bbox priv-text))
		    (wtxt  (- (list-ref bbox 2) (list-ref bbox 0)))
		    (htxt  (- (list-ref bbox 3) (list-ref bbox 1))))
		(id 'coords priv-text
		    (/ (- (winfo 'width  self) wtxt) 2)
		    (/ (- (winfo 'height self) htxt) 2))
		(id 'itemconfigure priv-text
		    :text
		    (string-append (number->string value) "%"))))))
   (cond
      ((valued-gauge? self)
       (configure-valued-gauge))
      (else
       (configure-plain-gauge))))
	    
;*---------------------------------------------------------------------*/
;*    init-valued-gauge ...                                            */
;*---------------------------------------------------------------------*/
(define (init-valued-gauge self::valued-gauge)
   (initialize-gauge self)
   (bind self "<Configure>" (lambda () (configure-gauge self)))
   (with-access::valued-gauge self (id priv-text text-foreground text-font)
      (set! priv-text 
	    (id 'create 'text 10 10
		:anchor "nw"
		:text "XX%" :fill text-foreground
		:font text-font))))
   

	    
	       
   
