;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bglk/runtime/Lwidget/hierarchy.scm          */
;*    -------------------------------------------------------------    */
;*    Author      :  Erick Gallesio [eg@unice.fr]                      */
;*    Creation    :  19-Aug-1998 14:02                                 */
;*    Last change :  Tue Jun  8 18:46:39 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    Hierarchy management widget                                      */
;*    -------------------------------------------------------------    */
;*    Source documentations:                                           */
;*       @path ../../manual/hierarchy.texi@                            */
;*       @node hierarchy@                                              */
;*    Examples:                                                        */
;*       @path ../../demos/hierarchy/hierarchy.scm@                    */
;*       @path ../../demos/hierarchy-file/hierarchy-file.scm@          */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_hierarchy
   
   (include "Llib/biglook.sch")
   
   (option  (loadq "Llib/biglook.init"))
   
   (import  (__biglook_glue         "Llib/scmglue.scm")
	    (__biglook_misc         "Llib/misc.scm")
	    (__biglook_tk-object    "Llib/tk-object.scm")
	    (__biglook_image        "Llib/image.scm")
	    (__biglook_frame        "Lwidget/frame.scm")
	    (__biglook_text         "Lwidget/text.scm")
	    (__biglook_scrollbar    "Lwidget/scrollbar.scm")
	    (__biglook_canvas       "Lwidget/canvas.scm")
	    (__biglook_scrollcanvas "Lwidget/scrollcanvas.scm"))
   
   (export  (class hierarchy-item
	       (hierarchy-item-init)
	       ;; parent
	       (parent (default #unspecified))
	       ;; ancestor
	       (ancestor (default #unspecified))
	       ;; data
	       (data (default #unspecified))
	       ;; node?
	       (node? (default #unspecified))
	       ;; icon
	       (icon (default #f))
	       ;; children
	       (children (default '()))
	       ;; label-id
	       (label-id (default #f))
	       ;; open
	       (open (default #f))
	       ;; bindings
	       (bindings (default '())))
	    
	    (class hierarchy-tree::scrollcanvas
	       (hierarchy-tree-init)
	       ;; root
	       (root (default #f))
	       ;; items-type
	       (make-items (default (lambda (self ancestor data node? icon)
				       (instantiate::hierarchy-item
					  (parent self)
					  (ancestor ancestor)
					  (data data)
					  (node? node?)
					  (icon icon)))))
	       ;; selection
	       (selection (default #f)))
	    
	    (hierarchy-item-init self::hierarchy-item)
	    (hierarchy-tree-init self::hierarchy-tree)
	    
	    (generic label-item self::hierarchy-item)
	    (generic open-item! self::hierarchy-item)
	    (generic close-item! self::hierarchy-item)
	    (generic select-item! self::hierarchy-item)
	    (generic add-item! self::hierarchy-tree ancestor data node? . icon)
	    (generic add-leave! self::hierarchy-tree ancestor data . icon)
	    (generic set-item-background! self::hierarchy-tree item selected?)
	    (generic add-node! self::hierarchy-tree ancestor data)
	    (remove-item! self::hierarchy-item)
	    (get-hierarchy-icons)
	    (set-hierarchy-icons! lst)))

;*---------------------------------------------------------------------*/
;*    hierarchy-item-init ...                                          */
;*---------------------------------------------------------------------*/
(define (hierarchy-item-init self::hierarchy-item)
   (with-access::hierarchy-item self (ancestor)
      (if (hierarchy-item? ancestor)
	  (with-access::hierarchy-item ancestor (children)
	     (set! children (append children (list self)))))))

;*---------------------------------------------------------------------*/
;*    hierarchy-tree-init ...                                          */
;*---------------------------------------------------------------------*/
(define (hierarchy-tree-init self::hierarchy-tree)
   ;; super class initialization
   (scrollcanvas-init self)
   ;; leave class initialization
   (with-access::hierarchy-tree self (canvas h-scroll-side root)
      (canvas-background-set! canvas "white")
      (set! h-scroll-side #f)
      ;; create the root of the hierarchy
      (set! root (add-item! self #f #f #t))
      ;; refresh the widget when idle
      (after 'idle (lambda () (update-hierarchy! self)))))

;*---------------------------------------------------------------------*/
;*    Global parameters                                                */
;*---------------------------------------------------------------------*/
(define *hierarchy-font* 	'(Helvetica 12))
(define *hierarchy-y*		0)
(define *hierarchy-node*	(instantiate::image
				   (file "dir.xpm")))
(define *hierarchy-open-node*	(instantiate::image
				   (file "diropen.xpm")))
(define *hierarchy-leave*	(instantiate::image
				   (file "file.xpm")))
(define *hierarchy-box-plus*    (instantiate::image
				   (file "box-plus.xpm")))
(define *hierarchy-box-minus*   (instantiate::image
				   (file "box-minus.xpm")))

;*---------------------------------------------------------------------*/
;*    hierarchy-icons ...                                              */
;*---------------------------------------------------------------------*/
(define (get-hierarchy-icons)
   (list *hierarchy-node* *hierarchy-open-node* *hierarchy-leave*
	 *hierarchy-box-plus* *hierarchy-box-minus*))

;*---------------------------------------------------------------------*/
;*    set-hierarchy-icons! ...                                         */
;*---------------------------------------------------------------------*/
(define (set-hierarchy-icons! lst)
   (match-case lst
      ((?node ?onode ?leave)
       (set! *hierarchy-node* node)
       (set! *hierarchy-open-node* onode)
       (set! *hierarchy-leave* leave))
      ((?node ?onode ?leave ?plus ?minus)
       (set! *hierarchy-node* node)
       (set! *hierarchy-open-node* onode)
       (set! *hierarchy-leave* leave)
       (set! *hierarchy-box-plus* plus)
       (set! *hierarchy-box-minus* minus))
      (else
       (error "hierarchy-set-icons!" "Illegal icons set" lst))))
      
;*---------------------------------------------------------------------*/
;*    update-hierarchy! ...                                            */
;*---------------------------------------------------------------------*/
(define (update-hierarchy! self::hierarchy-tree)
   (with-access::hierarchy-tree self (background root selection)
      ;; Delete all items in the hierarchy
      (canvas-delete self 'all)
      ;; Display something in 0,0 otherwise the canvas
      ;; is not centered on the origin
      ;; when we finish display. Why?
      (tk self 'create 'line 0 0 0 0 :fill background)
      (set! *hierarchy-y* 0)
      (update-a-hierarchy-level! self root 10)
      ;; Update the scroll region of the canvas
      (let ((bbox (bounding-box self 'all)))
	 (if (not (null? bbox))
	     (canvas-scroll-region-set! self bbox)))
      ;; Display the selection
      (let* ((data (and selection (hierarchy-item-data selection)))
	     (proc (lambda (i)
		      (if (eq? (hierarchy-item-data i) data)
			  (set-item-background! self i #t)))))
	 (if selection
	     (walk-hierarchy self proc)))))
 
;*---------------------------------------------------------------------*/
;*    update-a-hierarchy-level! ...                                    */
;*---------------------------------------------------------------------*/
(define (update-a-hierarchy-level! self::hierarchy-tree item indent)
   (define w (hierarchy-tree-id self))
   
   ;; WARNING: We don't use Tk tags for bindings here because we use a
   ;; lot of bindings for each layer of the tree (several per item in fact). 
   ;; When using tags, Tk doesn't delete the bindings when canvas items
   ;; are deleted but when the canvas itself is deleted. This consumes a lot
   ;; of memory for big hierarchies which are opened and closed several time.
   (define (create-icon-n-label w i x y icon)
      (let* ((label    (label-item i))
	     (add-bind (lambda (who)
			  (tk self 'bind who
			      "<Double-1>" (lambda () (open-item! i)))
			  (tk self 'bind who
			      "<1>" (lambda () (select-item! i)))
			  (for-each (lambda (binding)
				       (apply tk self 'bind who binding))
				    (hierarchy-item-bindings i))))
	     (item     (w 'create 'text x y :text label :anchor 'w
			  :font *hierarchy-font*)))
	 (if icon
	     (let ((icon (w 'create 'image (- x 4) y :image icon :anchor 'w)))
		(add-bind icon)
		(w 'move item 16 0)))
	 (hierarchy-item-label-id-set! i item)
	 (add-bind item)))
   
   (define (create-box w i x y open)
      (if open
	  (let ((img (w 'create 'image x y :image *hierarchy-box-minus*)))
	     (update-a-hierarchy-level! self i (+ indent 18))
	     (tk self 'bind img "<1>" (lambda() (close-item! i))))
	  (let ((img (w 'create 'image x y :image *hierarchy-box-plus*)))
	     (tk self 'bind img "<1>" (lambda() (open-item! i))))))
   
   (define (display-item i)
      (with-access::hierarchy-item i (node? icon)
	 (let ((x (+ indent 15))
	       (y (+ (* 20 *hierarchy-y*) 20)))
	    (set! *hierarchy-y* (+ *hierarchy-y* 1))
	    (w 'create 'line indent y (+ indent 15) y :fill "gray50")
	    (create-icon-n-label w i x y icon)
	    (if node?
		(create-box w i indent y (hierarchy-item-open i))))))
   
   (let ((initial-y *hierarchy-y*)
	 (last-y    0))
      (for-each (lambda (item)
		   (set! last-y *hierarchy-y*)
		   (display-item item))
		(hierarchy-item-children item))
      ;; Draw the vertical line linking siblings
      (if (not (= initial-y *hierarchy-y*))
	  (let ((line (w 'create 'line 
			 indent (+ (* 20 initial-y) 7)
			 indent (* 20 (+ last-y 1))
			 :fill "gray50")))
	     (tk self 'lower line)))))

;*---------------------------------------------------------------------*/
;*    walk-hierarchy ...                                               */
;*---------------------------------------------------------------------*/
(define (walk-hierarchy self::hierarchy-tree proc)
   (let loop ((item (hierarchy-tree-root self)))
      (proc item)
      (if (hierarchy-item-node? item)
	  (for-each loop (hierarchy-item-children item)))))

;*---------------------------------------------------------------------*/
;*    label-item ...                                                   */
;*---------------------------------------------------------------------*/
(define-generic (label-item self::hierarchy-item))

;*---------------------------------------------------------------------*/
;*    label-item ::hierarchy-item ...                                  */
;*---------------------------------------------------------------------*/
(define-method (label-item self::hierarchy-item)
   (with-access::hierarchy-item self (data)
      data))

;*---------------------------------------------------------------------*/
;*    select-item! ::hierarchy-item ...                                */
;*---------------------------------------------------------------------*/
(define-generic (select-item! self::hierarchy-item))

;*---------------------------------------------------------------------*/
;*    select-item! ...                                                 */
;*---------------------------------------------------------------------*/
(define-method (select-item! self::hierarchy-item)
   (with-access::hierarchy-item self (parent data)
      (let* ((old-selection (hierarchy-tree-selection parent))
	     (old-data (and (hierarchy-item? old-selection)
			    (hierarchy-item-data old-selection)))
	     (scan-proc (lambda (i)
			   (if (hierarchy-item? i)
			       (let ((d (hierarchy-item-data i)))
				  (if (eq? d old-data)
				      (set-item-background! parent i #f))
				  (if (eq? d data)
				      (set-item-background! parent i #f)))))))
	 (if (not (eq? old-data data))
	     (begin
		(walk-hierarchy parent scan-proc)
		(hierarchy-tree-selection-set! parent self))))))

;*---------------------------------------------------------------------*/
;*    open-item! ::hierarchy-item ...                                  */
;*---------------------------------------------------------------------*/
(define-generic (open-item! self::hierarchy-item))

;*---------------------------------------------------------------------*/
;*    open-item! ...                                                   */
;*---------------------------------------------------------------------*/
(define-method (open-item! self::hierarchy-item)
   (with-access::hierarchy-item self (open icon parent)
      (set! open #t)
      (if (eq? icon *hierarchy-node*)
	  (set! icon *hierarchy-open-node*))
      (update-hierarchy! parent)))

;*---------------------------------------------------------------------*/
;*    close-item! ::hierarchy-item ...                                 */
;*---------------------------------------------------------------------*/
(define-generic (close-item! self::hierarchy-item))

;*---------------------------------------------------------------------*/
;*    close-item! ...                                                  */
;*---------------------------------------------------------------------*/
(define-method (close-item! self::hierarchy-item)
   (with-access::hierarchy-item self (open icon parent)
      (set! open #f)
      (if (eq? icon *hierarchy-open-node*)
	  (set! icon *hierarchy-node*))
      (update-hierarchy! parent)))

;*---------------------------------------------------------------------*/
;*    add-item! ::hierarchy-item ...                                   */
;*---------------------------------------------------------------------*/
(define-generic (add-item! self::hierarchy-tree ancestor data node? . icon))

;*---------------------------------------------------------------------*/
;*    add-item! ...                                                    */
;*---------------------------------------------------------------------*/
(define-method (add-item! self::hierarchy-tree ancestor data node? . icon)
   (with-access::hierarchy-tree self (root make-items)
      (let ((ancestor (or ancestor root))
	    (icon     (if (pair? icon) (car icon) #f)))
	 (make-items self ancestor data node? icon))))

;*---------------------------------------------------------------------*/
;*    add-leave! ::hierarchy-leave ...                                 */
;*---------------------------------------------------------------------*/
(define-generic (add-leave! self::hierarchy-tree ancestor data . icon))

;*---------------------------------------------------------------------*/
;*    add-leave! ...                                                   */
;*---------------------------------------------------------------------*/
(define-method (add-leave! self::hierarchy-tree ancestor data . icon)
   (if (pair? icon)
       (add-item! self ancestor data #f (car icon))
       (add-item! self ancestor data #f *hierarchy-leave*)))

;*---------------------------------------------------------------------*/
;*    add-node! ::hierarchy-node ...                                   */
;*---------------------------------------------------------------------*/
(define-generic (add-node! self::hierarchy-tree ancestor data))

;*---------------------------------------------------------------------*/
;*    add-node! ...                                                    */
;*---------------------------------------------------------------------*/
(define-method (add-node! self::hierarchy-tree ancestor data)
   (add-item! self ancestor data #t *hierarchy-node*))

;*---------------------------------------------------------------------*/
;*    set-item-background! ...                                         */
;*---------------------------------------------------------------------*/
(define-generic (set-item-background! self::hierarchy-tree item selected?))

;*---------------------------------------------------------------------*/
;*    set-item-background! ...                                         */
;*---------------------------------------------------------------------*/
(define-method (set-item-background! self::hierarchy-tree item selected?)
   (let ((bg   (if selected? "red" "black"))
	 (item (hierarchy-item-label-id item)))
      (tk self 'itemconfigure item :fill bg)))

;*---------------------------------------------------------------------*/
;*    remove-item! ...                                                 */
;*---------------------------------------------------------------------*/
(define (remove-item! self::hierarchy-item)
   (with-access::hierarchy-item self (ancestor parent)
      (with-access::hierarchy-item ancestor (children)
	 (set! children (remq! self children))
	 (update-hierarchy! parent))))

;*---------------------------------------------------------------------*/
;*    bind ::hierarchy-item ...                                        */
;*    -------------------------------------------------------------    */
;*    Binding a hierarchy item is a difficult operation. The problem   */
;*    comes from that when the binding is set the tk object            */
;*    representing the item does not exist yet. In consequence, the    */
;*    binding in stored inside the hierarchy-item and when a tk        */
;*    object is built for that item the bindings are set (see          */
;*    UPDATE-A-HIERARCHY-LEVEL!).                                      */
;*---------------------------------------------------------------------*/
(define-method (bind obj::hierarchy-item . args)
   (with-access::hierarchy-item obj (bindings)
      (set! bindings (cons args bindings))))


	       


      

   


		  
