;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    .../prgm/project/bglk/runtime/Lwidget/labeled-entry.scm          */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Thu Dec 17 11:45:56 1998                          */
;*    Last change :  Fri May  7 16:07:16 1999 ()                       */
;*    -------------------------------------------------------------    */
;*    Labeled entry                                                    */
;*    -------------------------------------------------------------    */
;*    Source documentation:                                            */
;*       @path ../../manual/lentry.texi@                               */
;*       @node Labeled Entry@                                          */
;*    Example:                                                         */
;*       @path ../../demos/lentry/lentry.scm@                          */
;*=====================================================================*/
 
;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_labeled-entry
   
   (include "Llib/biglook.sch")
   
   (option  (loadq "Llib/biglook.init"))
   
   (import  (__biglook_glue      "Llib/scmglue.scm")
	    (__biglook_misc      "Llib/misc.scm")
	    (__biglook_key       "Llib/key.scm")
	    (__biglook_tk-object "Llib/tk-object.scm")
	    (__biglook_entry     "Lwidget/entry.scm")
	    (__biglook_frame     "Lwidget/frame.scm")
	    (__biglook_button    "Lwidget/button.scm"))
   
   (export  (class labeled-entry::entry
	       ;; labeled entry constructor
	       (labeled-entry-init)
	       ;; associated widgets
	       (frame (default #unspecified))
	       (entry (default #unspecified))
	       (label (default #unspecified))
	       ;; virtual title
	       (title
		(get (lambda (o)
			(with-access::labeled-entry o (label)
			   (with-access::label label (text)
			      text))))
		(set (lambda (o v)
			(with-access::labeled-entry o (label)
			   (with-access::label label (text)
			      (set! text v))))))
	       ;; virtual title-width
	       (title-width
		(get (lambda (o)
			(with-access::labeled-entry o (label)
			   (with-access::label label (width)
			      width))))
		(set (lambda (o v)
			(with-access::labeled-entry o (label)
			   (with-access::label label (width)
			      (set! width v))))))
	       ;; virtual title-anchor
	       (title-anchor
		(get (lambda (o)
			(with-access::labeled-entry o (label)
			   (with-access::label label (anchor)
			      anchor))))
		(set (lambda (o v)
			(with-access::labeled-entry o (label)
			   (with-access::label label (anchor)
			      (set! anchor v))))))
	       ;; virtual background
	       (background
		(get (lambda (o)
			(with-access::labeled-entry o (frame)
			   (with-access::frame frame (background)
			      background))))
		(set (lambda (o v)
			(with-access::labeled-entry o (frame entry label)
			   (with-access::frame frame (background)
			      (set! background v))
			   (with-access::entry entry (background)
			      (set! background v))
			   (with-access::label label (background)
			      (set! background v))))))
	       ;; virtual foreground
	       (foreground
		(get (lambda (o)
			(with-access::labeled-entry o (label)
			   (with-access::label label (foreground)
			      foreground))))
		(set (lambda (o v)
			(with-access::labeled-entry o (label)
			   (with-access::label label (foreground)
			      (set! foreground v))))))
	       ;; virtual border-width
	       (border-width
		(get (lambda (o)
			(with-access::labeled-entry o (frame)
			   (with-access::frame frame (border-width)
			      border-width))))
		(set (lambda (o v)
			(with-access::labeled-entry o (frame)
			   (with-access::frame frame (border-width)
			      (set! border-width v))))))
	       ;; virtual relief
	       (relief
		(get (lambda (o)
			(with-access::labeled-entry o (frame)
			   (with-access::frame frame (relief)
			      relief))))
		(set (lambda (o v)
			(with-access::labeled-entry o (frame)
			   (with-access::frame frame (relief)
			      (set! relief v)))))))
	       
	    (labeled-entry-init ::labeled-entry)))

;*---------------------------------------------------------------------*/
;*    labeled-entry-init ...                                           */
;*---------------------------------------------------------------------*/
(define (labeled-entry-init widget::labeled-entry)
   (with-access::labeled-entry widget (id eid parent frame entry label)
      (set! frame (if (eq? parent #unspecified)
		      (instantiate::frame
			 (class "LabeledEntry"))
		      (instantiate::frame
			 (parent parent)
			 (class "LabeledEntry"))))
      (set! entry (instantiate::entry (parent frame)))
      (set! label (instantiate::label (parent frame)))
      (set! id (entry-id entry))
      (set! eid (frame-id frame))
      (tk-command-object-set! eid widget)
      (pack label :side 'left :padx 2 :pady 2)
      (pack entry :side 'right :padx 2 :pady 2 :expand #t :fill 'x)))

;*---------------------------------------------------------------------*/
;*    X11 resources                                                    */
;*---------------------------------------------------------------------*/
(option 'add "*LabeledEntry.Entry.Background"  "white"  "widgetDefault")
(option 'add "*LabeledEntry.Entry.Font"	       "fixed"  "widgetDefault")
(option 'add "*LabeledEntry.Entry.Relief"      "sunken" "widgetDefault")
	       
