;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    .../prgm/project/bglk/runtime/Lwidget/labeled-frame.scm          */
;*    -------------------------------------------------------------    */
;*    Author      :  Erick Gallesio [eg@unice.fr]                      */
;*    Creation    :  25-Oct-1996 19:31                                 */
;*    Last change :  Fri May  7 16:11:56 1999 ()                       */
;*    -------------------------------------------------------------    */
;*    Labeled frame                                                    */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_labeled-frame
   
   (include "Llib/biglook.sch")
   
   (option  (loadq "Llib/biglook.init"))
   
   (import  (__biglook_glue      "Llib/scmglue.scm")
	    (__biglook_misc      "Llib/misc.scm")
	    (__biglook_key  "Llib/key.scm")
	    (__biglook_tk-object "Llib/tk-object.scm")
	    (__biglook_entry     "Lwidget/entry.scm")
	    (__biglook_frame     "Lwidget/frame.scm")
	    (__biglook_button    "Lwidget/button.scm"))
   
   (export  (class labeled-frame::frame
	       ;; labeled entry constructor
	       (labeled-frame-init)
	       ;; associated widgets
	       (frame (default #unspecified))
	       (fill-frame (default #unspecified))
	       (box-frame (default #unspecified))
	       (label (default #unspecified))
	       ;; virtual title
	       (title
		(get (lambda (o)
			(with-access::labeled-frame o (label)
			   (with-access::label label (text)
			      text))))
		(set (lambda (o v)
			(with-access::labeled-frame o (label)
			   (with-access::label label (text)
			      (set! text v))))))
	       ;; virtual font
	       (font
		(get (lambda (o)
			(with-access::labeled-frame o (label)
			   (with-access::label label (font)
			      font))))
		(set (lambda (o v)
			(with-access::labeled-frame o (label)
			   (with-access::label label (font)
			      (set! font v))))))
	       ;; virtual background
	       (background
		(get (lambda (o)
			(with-access::labeled-frame o (frame)
			   (with-access::frame frame (background)
			      background))))
		(set (lambda (o v)
			(with-access::labeled-frame o (frame
						       label
						       fill-frame
						       box-frame)
			   (with-access::frame frame (background)
			      (set! background v))
			   (with-access::label label (background)
			      (set! background v))
			   (with-access::frame fill-frame (background)
			      (set! background v))
			   (with-access::frame box-frame (background)
			      (set! background v))))))
	       ;; virtual foreground
	       (foreground
		(get (lambda (o)
			(with-access::labeled-frame o (label)
			   (with-access::label label (foreground)
			      foreground))))
		(set (lambda (o v)
			(with-access::labeled-frame o (label)
			   (with-access::label label (foreground)
			      (set! foreground v))))))
	       ;; virtual width
	       (width
		(get (lambda (o)
			(with-access::labeled-frame o (frame)
			   (with-access::frame frame (width)
			      width))))
		(set (lambda (o v)
			(with-access::labeled-frame o (frame)
			   (with-access::frame frame (width)
			      (set! width v))))))
	       ;; virtual height
	       (height
		(get (lambda (o)
			(with-access::labeled-frame o (frame)
			   (with-access::frame frame (height)
			      height))))
		(set (lambda (o v)
			(with-access::labeled-frame o (frame)
			   (with-access::frame frame (height)
			      (set! height v)))))))
	    (labeled-frame-init ::labeled-frame)))

;*---------------------------------------------------------------------*/
;*    labeled-frame-init ...                                           */
;*---------------------------------------------------------------------*/
(define (labeled-frame-init self::labeled-frame)
   (with-access::labeled-frame self (id eid parent
					frame box-frame fill-frame
					label)
      (set! frame (if (eq? parent #unspecified)
		      (instantiate::frame
			 (class "LabeledFrame"))
		      (instantiate::frame
			 (parent parent)
			 (class "LabeledFrame"))))
      (set! box-frame (instantiate::frame (parent frame) (class "Box")))
      (set! fill-frame (instantiate::frame (parent box-frame) (relief "flat")))
      (set! label (instantiate::label (parent frame)))
      (set! id (frame-id box-frame))
      (set! eid (frame-id frame))
      (tk-command-object-set! eid self)
      ;; filler is here to avoid that the label overwrite first widget
      ;; packed in the labeled frame
      (pack box-frame :padx 10 :pady 10 :expand #t :fill "both")
      ;; Configure internal windows
      (configure-labeled-frame self box-frame fill-frame label)
      ;; Changing the text font must reconfigure the complete bwidget
      (bind label
	    "<Configure>"
	    (lambda ()
	       (configure-labeled-frame self box-frame fill-frame label)))))

;*---------------------------------------------------------------------*/
;*    configure-labeled-frame ...                                      */
;*---------------------------------------------------------------------*/
(define (configure-labeled-frame lf box filler label)
   (let ((width  (winfo 'width label))
	 (height (winfo 'height label)))
      ;; Compute a new width and height for the filler
      (frame-height-set! filler (+fx (/fx height 2) 2))
      (frame-width-set! filler width)
      (pack filler :expand #f :fill "x" :side "top")
      ;; Place the label on the frame
      (place label :in box :x 10 :y 0 :anchor "w")))

;*---------------------------------------------------------------------*/
;*    X11 resources                                                    */
;*---------------------------------------------------------------------*/
(option 'add "*LabeledFrame.Box.Relief"      "ridge" "widgetDefault")
(option 'add "*LabeledFrame.Box.BorderWidth"       2 "widgetDefault")

;*---------------------------------------------------------------------*/
;*    Example (font selector)                                          */
;*---------------------------------------------------------------------*/
;* (module font-selector                                               */
;*    (library biglook))                                               */
;*                                                                     */
;* (tk-make-variable "font")                                           */
;* (tk-make-variable "type")                                           */
;*                                                                     */
;* (define lf (instantiate::labeled-frame (title "Font")))             */
;* (pack lf :fill "both" :expand #t :side "left")                      */
;*                                                                     */
;* (for-each (lambda (x)                                               */
;* 	     (pack (instantiate::radio-button                          */
;* 		      (anchor "w")                                     */
;* 		      (parent lf)                                      */
;* 		      (variable "font")                                */
;* 		      (text x)                                         */
;* 		      (string-value #f)                                */
;* 		      (width 8)                                        */
;* 		      (font "fixed")                                   */
;* 		      (value x))                                       */
;* 		   :fill "x" :expand #f :anchor "w" :side "top"))      */
;* 	  '("10pt" "12pt" "14pt" "18pt"))                              */
;*                                                                     */
;* (define lf2 (instantiate::labeled-frame (title "Type")))            */
;* (pack lf2 :fill "both" :expand #t :side "left")                     */
;*                                                                     */
;* (for-each (lambda (x)                                               */
;* 	     (pack (instantiate::radio-button                          */
;* 		      (anchor "w")                                     */
;* 		      (parent lf2)                                     */
;* 		      (variable "type")                                */
;* 		      (text x)                                         */
;* 		      (string-value #f)                                */
;* 		      (width 15)                                       */
;* 		      (font "fixed")                                   */
;* 		      (value x))                                       */
;* 		   :fill "x" :expand #f :anchor "w" :side "top"))      */
;* 	  '("Bold" "Italic" "Normal"))                                 */
	       
