;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bglk/runtime/Lwidget/listbox.scm            */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Tue Oct 20 10:39:28 1998                          */
;*    Last change :  Fri Apr  9 10:00:04 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The List-box widget                                              */
;*    -------------------------------------------------------------    */
;*    Source documentation:                                            */
;*       @path ../../manual/listbox.texi@                              */
;*       @node Listbox@                                                */
;*    Examples:                                                        */
;*       @path ../../demos/listbox/listbox.scm@                        */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_listbox
   
   (include "Llib/biglook.sch")
   
   (option  (loadq "Llib/biglook.init"))
   
   (import  (__biglook_glue      "Llib/scmglue.scm")
	    (__biglook_misc      "Llib/misc.scm")
	    (__biglook_tk-object "Llib/tk-object.scm"))
   
   (export  (class listbox::simple-widget
	       (list-box-init)
	       ;; select-background
	       (select-background::bstring
		(get (lambda (o)
			(tk o 'cget :selectbackground)))
		(set (lambda (o v)
			(tk o 'configure :selectbackground v))))	     
	       ;; select-foreground
	       (select-foreground::bstring
		(get (lambda (o)
			(tk o 'cget :selectforeground)))
		(set (lambda (o v)
			(tk o 'configure :selectforeground v))))	     
	       ;; export-selection
	       (export-selection::bool
		(get (lambda (o)
			(tk o 'cget :exportselection)))
		(set (lambda (o v)
			(tk o 'configure :exportselection v))))	     
	       ;; font
	       (font::bstring
		(get (lambda (o)
			(tk o 'cget :font)))
		(set (lambda (o v)
			(tk o 'configure :font v))))	     
	       ;; foreground
	       (foreground::bstring
		(get (lambda (o)
			(tk o 'cget :foreground)))
		(set (lambda (o v)
			(tk o 'configure :foreground v))))	     
	       ;; width
	       (width::int
		(get (lambda (o)
			(tk o 'cget :width)))
		(set (lambda (o v)
			(tk o 'configure :width v))))	     
	       ;; height
	       (height::int
		(get (lambda (o)
			(tk o 'cget :height)))
		(set (lambda (o v)
			(tk o 'configure :height v))))	     
	       ;; select-mode
	       (select-mode::bstring
		(get (lambda (o)
			(tk o 'cget :selectmode)))
		(set (lambda (o v)
			(tk o 'configure :selectmode v))))
	       ;; value
	       (value
		(get (lambda (o)
			(tk o 'get 0 'end)))
		(set (lambda (o v)
			(tk o 'delete 0 'end)
			(apply tk o 'insert 0 v))))
	       ;; active
	       (active
		(get (lambda (o)
			(tk o 'index 'active)))
		(set (lambda (o v)
			(tk o 'activate v))))
	       ;; selection
	       (selection
		(get (lambda (o)
			(tk o 'curselection)))
		(set (lambda (o v)
			(for-each (lambda (v)
				     (tk o 'selection 'set v))
				  v)))))
	     
	    list-box-init))

;*---------------------------------------------------------------------*/
;*    list-box-init ...                                                */
;*---------------------------------------------------------------------*/
(define list-box-init
   (let ((list-box (make-widget-constructor "listbox")))
      (lambda (o)
	 (simple-widget-init o list-box))))

;*---------------------------------------------------------------------*/
;*    global variables                                                 */
;*---------------------------------------------------------------------*/
(define *listbox-selection* '())
(define *listbox-prev*       0)

;*---------------------------------------------------------------------*/
;*    format@, ...                                                     */
;*---------------------------------------------------------------------*/
(define (format@, x y)
   (let ((port (open-output-string)))
      (display "@" port)
      (display x port)
      (display "," port)
      (display y port)
      (close-output-port port)))
      
;*---------------------------------------------------------------------*/
;*    Class bindings for listbox widgets                               */
;*---------------------------------------------------------------------*/
(add-binding! "Listbox" "<1>" (WW x y)
	      ;; Note: the check for existence of %WW below is
	      ;; because this binding is sometimes invoked after
	      ;; a window has been deleted (e.g. because there is
	      ;; a double-click binding on the widget that deletes it).
	      ;; Users can put "break"s in their bindings to avoid the
	      ;; error, but this check makes that unnecessary.
	      (when (winfo 'exists WW)
		    (listbox-begin-select WW (WW 'index (format@, x y)))))

(add-binding! "Listbox" "<Double-1>" ()
	      ;; Ignore double clicks so that users can define
	      ;; their own behaviors. Among other things, this
	      ;; prevents errors if the user deletes the
	      ;; listbox on a double click.
	      #f)

(add-binding! "Listbox" "<B1-Motion>" (WW x y)
	      (set! *x* x)
	      (set! *y* y)
	      (listbox-motion WW (WW 'index (format@, x  y))))

(add-binding! "Listbox" "<ButtonRelease-1>" (WW x y)
	      (cancel-repeat)
	      (WW 'activate (format@, x  y)))

(add-binding! "Listbox" "<Shift-1>" (WW x y)
	      (listbox-begin-extend WW (WW 'index (format@, x  y))))

(add-binding! "Listbox" "<Control-1>" (WW x y)
	      (listbox-begin-toggle WW (WW 'index (format@, x  y))))

(add-binding! "Listbox" "<B1-Leave>" (WW x y)
	      (set! *x* x)
	      (set! *y* y)
	      (listbox-auto-scan WW))

(add-binding! "Listbox" "<B1-Enter>" ()
	      (cancel-repeat))

(add-binding! "Listbox" "<Up>" (WW)
	      (listbox-up-down WW -1))

(add-binding! "Listbox" "<Shift-Up>" (WW)
	      (listbox-extend-up-down WW -1))

(add-binding! "Listbox" "<Down>" (WW)
	      (listbox-up-down WW 1))

(add-binding! "Listbox" "<Shift-Down>" (WW)
	      (listbox-extend-up-down WW 1))

(add-binding! "Listbox" "<Left>" (WW)
	      (WW 'xview 'scroll -1 'units))
(add-binding! "Listbox" "<Control-Left>" (WW)
	      (WW 'xview 'scroll -1 'pages))
(add-binding! "Listbox" "<Right>" (WW)
	      (WW 'xview 'scroll  1 'units))
(add-binding! "Listbox" "<Control-Right>" (WW)
	      (WW 'xview 'scroll  1 'pages))
(add-binding! "Listbox" "<Prior>" (WW)
	      (WW 'yview 'scroll -1 'pages))
(add-binding! "Listbox" "<Next>" (WW)
	      (WW 'yview 'scroll  1 'pages))
(add-binding! "Listbox" "<Control-Prior>" (WW)
	      (WW 'xview 'scroll -1 'pages))
(add-binding! "Listbox" "<Control-Next>" (WW)
	      (WW 'xview 'scroll  1 'pages))

(add-binding! "Listbox" "<Home>" (WW)
	      (WW 'xview 'moveto 0))

(add-binding! "Listbox" "<End>" (WW)
	      (WW 'xview 'moveto 1))

(add-binding! "Listbox" "<Control-Home>" (WW)
	      (WW 'activate 0)
	      (WW 'see 0)
	      (WW 'selection 'clear 0 'end)
	      (WW 'selection 'set   0))

(add-binding! "Listbox" "<Shift-Control-Home>" (WW)
	      (listbox-data-extend WW 0))

(add-binding! "Listbox" "<Control-End>" (WW)
	      (WW 'activate 'end)
	      (WW 'see 'end)
	      (WW 'selection 'clear 0 'end)
	      (WW 'selection 'set 'end))

(add-binding! "Listbox" "<Shift-Control-End>" (WW)
	      (listbox-data-extend WW (WW 'index 'end)))

(add-binding! "Listbox" "<<Copy>>" (WW)
	      (when (equal? (selection 'own :displayof WW) WW)
		    (clipboard 'clear :displayof WW)
		    (clipboard 'append :displayof WW
			       (selection 'get :displayof WW))))

(add-binding! "Listbox" "<space>" (WW)
	      (listbox-begin-select WW (WW 'index 'active)))

(add-binding! "Listbox" "<Select>" (WW)
	      (listbox-begin-select WW (WW 'index 'active)))

(add-binding! "Listbox" "<Control-Shift-space>" (WW)
	      (listbox-begin-extend WW (WW 'index 'active)))

(add-binding! "Listbox" "<Shift-Select>" (WW)
	      (listbox-begin-extend WW (WW 'index 'active)))

(add-binding! "Listbox" "<Escape>" (WW)
	      (listbox-cancel WW))

(add-binding! "Listbox" "<Control-slash>" (WW)
	      (listbox-select-all WW))

(add-binding! "Listbox" "<Control-backslash>" (WW)
	      (unless (equal? (tk-get WW :selectmode) "browse")
		      (WW 'selection 'clear 0 'end)))

;; Additional Tk bindings that aren't part of the Motif look and feel:
(add-binding! "Listbox" "<Shift-2>" (WW x y)
	      (WW 'scan 'mark x y))

(add-binding! "Listbox" "<B2-Motion>" (WW x y)
	      (WW 'scan 'dragto x y))

;; listbox-begin-select --
;;
;; This procedure is typically invoked on button-1 presses.  It begins
;; the process of making a selection in the listbox.  Its exact behavior
;; depends on the selection mode currently in effect for the listbox;
;; see the Motif documentation for details.
;;
;; w -		The listbox widget.
;; el -		The element for the selection operation (typically the
;;		one under the pointer).  Must be in numerical form.

(define (listbox-begin-select w el)
  (if (equal? (tk-get w :selectmode) "multiple")
      (if (w 'selection 'includes el)
	  (w 'selection 'clear el)
	  (w 'selection 'set el))
      (begin
	(w 'selection 'clear 0 'end)
	(w 'selection 'set el)
	(w 'selection 'anchor el)
	(set! *listbox-selection* '())
	(set! *listbox-prev* el))))


;; listbox-Motion --
;;
;; This procedure is called to process mouse motion events while
;; button 1 is down.  It may move or extend the selection, depending
;; on the listbox's selection mode.
;;
;; w -		The listbox widget.
;; el -		The element under the pointer (must be a number).

(define (listbox-Motion w el)
   (unless (= el *listbox-prev*)
	   (let ((anchor (w 'index 'anchor))
		 (mode   (tk-get w :selectmode)))
	      (cond
		 ((string=? mode "browse")
		  (w 'selection 'clear 0 'end)
		  (w 'selection 'set el)
		  (set! *listbox-prev* el))
		 
		 ((string=? mode "extended")
		  (let ((i *listbox-prev*))
		     (if (w 'selection 'includes 'anchor)
			 (begin
			    (w 'selection 'clear i el)
			    (w 'selection 'set 'anchor el))
			 (begin
			    (w 'selection 'clear i el)
			    (w 'selection 'clear 'anchor el)))
		     (while (and (< i el) (< i anchor))
			    (if (member i *listbox-selection*)
				(w 'selection 'set i))
			    (set! i (+ i 1)))
		     (while (and (> i el) (> i anchor))
			    (if (member i *listbox-selection*)
				(w 'selection 'set i))
			    (set! i (- i 1)))
		     (set! *listbox-prev* el)))))))

;; listbox-BeginExtend --
;;
;; This procedure is typically invoked on shift-button-1 presses.  It
;; begins the process of extending a selection in the listbox.  Its
;; exact behavior depends on the selection mode currently in effect
;; for the listbox;  see the Motif documentation for details.
;;
;; w -		The listbox widget.
;; el -		The element for the selection operation (typically the
;;		one under the pointer).  Must be in numerical form.

(define (listbox-begin-extend w el)
  (when (equal? (tk-get w :selectmode) "extended")
    (if (w 'selection 'includes 'anchor)
	(listbox-motion w el)
	;; No selection yet; simulate the begin-select operation
	(listbox-begin-select w el))))


;; listbox-begin-toggle --
;;
;; This procedure is typically invoked on control-button-1 presses.  It
;; begins the process of toggling a selection in the listbox.  Its
;; exact behavior depends on the selection mode currently in effect
;; for the listbox;  see the Motif documentation for details.
;;
;; w -		The listbox widget.
;; el -		The element for the selection operation (typically the
;;		one under the pointer).  Must be in numerical form.

(define (listbox-begin-toggle w el)
  (when (equal? (tk-get w :selectmode) "extended")
     (set! *listbox-selection* (w 'curselection))
     (set! *listbox-prev* 	 el)
     (w 'selection 'anchor el)
     (if (w 'selection 'includes el)
	 (w 'selection 'clear el)
	 (w 'selection 'set   el))))


;; listbox-auto-scan --
;; This procedure is invoked when the mouse leaves an entry window
;; with button 1 down.  It scrolls the window up, down, left, or
;; right, depending on where the mouse left the window, and reschedules
;; itself as an "after" command so that the window continues to 'scroll until
;; the mouse moves back into the window or the mouse button is released.
;;
;; Arguments:
;; w -		The entry window.

(define (listbox-auto-scan w)
  (when (winfo 'exists w)
    (let* ((x    *x*)
	   (y    *y*)
	   (scan (lambda ()
		   (listbox-motion w (w 'index (format@, x y)))
		   (set! *after-id* (after 50 (lambda ()
						  (listbox-auto-scan w)))))))
      (cond
       ((>= y (winfo 'height w)) (w 'yview 'scroll +1 'units) (scan))
       ((< y 0)		     (w 'yview 'scroll -1 'units) (scan))
       ((>= x (winfo 'width w))  (w 'xview 'scroll +2 'units) (scan))
       ((< x 0)		     (w 'xview 'scroll -2 'units) (scan))))))


;; listbox-up-down --
;;
;; Moves the location cursor (active element) up or down by one element,
;; and changes the selection if we're in browse or extended selection
;; mode.
;;
;; w -		The listbox widget.
;; amount -	+1 to move down one item, -1 to move back one item.

(define (listbox-up-down w amount)
  (let ((mode (tk-get w :selectmode)))
    (w 'activate (+ (w 'index 'active) amount))
    (w 'see 'active)
    (cond 
      ((string=? mode "browse") 
                 (w 'selection 'clear 0 'end)
		 (w 'selection 'set 'active))
      ((string=? mode "extended") (w 'selection 'clear 0 'end)
				  (w 'selection 'set 'active)
				  (w 'selection 'anchor 'active)
				  (set! *listbox-prev*      (w 'index 'active))
				  (set! *listbox-selection* '())))))

;; listbox-extend-up-down --
;;
;; Does nothing unless we're in extended selection mode;  in this
;; case it moves the location cursor (active element) up or down by
;; one element, and extends the selection to that point.
;;
;; w -		The listbox widget.
;; amount -	+1 to move down one item, -1 to move back one item.

(define (listbox-extend-up-down w amount)
  (when  (equal? (tk-get w :selectmode) "extended")
     (w 'activate (+ (w 'index 'active) amount))
     (w 'see 'active)
     (listbox-motion w (w 'index 'active))))


;; listbox-data-extend
;;
;; This procedure is called for key-presses such as Shift-KEndData.
;; If the selection mode isn't multiple or extend then it does nothing.
;; Otherwise it moves the active element to el and, if we're in
;; extended mode, extends the selection to that point.
;;
;; w -		The listbox widget.
;; el -		An integer element number.

(define (listbox-data-extend w el)
  (let ((mode (tk-get w :selectmode)))
    (cond 
       ((string=? mode "extended")	(w 'activate el)
					(w 'see el)
					(if (w 'selection 'includes 'anchor)
					    (listbox-motion w el)))
       ((string=? mode "multiple")	(w 'activate el)
					(w 'see el)))))

;; listbox-cancel
;;
;; This procedure is invoked to cancel an extended selection in
;; progress.  If there is an extended selection in progress, it
;; restores all of the items between the active one and the anchor
;; to their previous selection state.
;;
;; w -		The listbox widget.

(define (listbox-cancel w)
  (when (equal? (tk-get w :selectmode) "extended")
     (let ((first (w 'index 'anchor))
	   (last  *listbox-prev*))
       (when (> first last)
	  (let ((tmp first))
	    (set! first last)
	    (set! last tmp)))
       (w 'selection 'clear first last)
       (while (<= first last)
	  (if (member first *listbox-selection*)
	      (w 'selection 'set first))
	  (set! first (+ first 1))))))

;; listbox-select-all
;;
;; This procedure is invoked to handle the "select all" operation.
;; For single and browse mode, it just selects the active element.
;; Otherwise it selects everything in the widget.
;;
;; w -		The listbox widget.

(define (listbox-select-all w)
  (let ((mode (tk-get w :selectmode)))
    (if (or (equal? mode "single") (equal? mode "browse"))
	(begin
	  (w 'selection 'clear 0 'end)
	  (w 'selection 'set 'active))
	(w 'selection 'set 0 'end))))
