;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bglk/runtime/Lwidget/notepad.scm            */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Thu Dec 17 11:45:56 1998                          */
;*    Last change :  Thu Jun 10 10:15:50 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    Notepad                                                          */
;*    -------------------------------------------------------------    */
;*    Source documentation:                                            */
;*       @path ../../manual/notepad.texi@                              */
;*       @node Notepad@                                                */
;*    Example:                                                         */
;*       @path ../../demos/notepad/notepad.scm@                        */
;*=====================================================================*/
 
;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_notepad
   
   (include "Llib/biglook.sch")
   
   (option  (loadq "Llib/biglook.init"))
   
   (import  (__biglook_glue      "Llib/scmglue.scm")
	    (__biglook_misc      "Llib/misc.scm")
	    (__biglook_tk-object "Llib/tk-object.scm")
	    (__biglook_entry     "Lwidget/entry.scm")
	    (__biglook_frame     "Lwidget/frame.scm")
	    (__biglook_button    "Lwidget/button.scm"))
   
   (export  (class notepad::simple-widget
	       ;; notepad initialization
	       (notepad-init)
	       ;; private notepad fields
	       (frame (default #unspecified))
	       (%cache (default #unspecified))
	       (%page (default #unspecified))
	       (%tabs (default #unspecified))
	       (%padding (default #unspecified))
	       (%current (default #f))
	       ;; initialization fields
	       (init-width::int read-only (default 500))
	       (init-height::int read-only (default 300))
	       ;; virtual width
	       (width
		(get (lambda (o)
			(with-access::notepad o (frame)
			   (with-access::frame frame (width)
			      width))))
		(set (lambda (o v)
			(with-access::notepad o (frame %page)
			   (with-access::frame frame (width)
			      (set! width v))
			   (with-access::frame %page (width)
			      (set! width v))))))
	       ;; virtual height
	       (height
		(get (lambda (o)
			(with-access::notepad o (frame)
			   (with-access::frame frame (height)
			      height))))
		(set (lambda (o v)
			(with-access::notepad o (frame %page)
			   (with-access::frame frame (height)
			      (set! height v))
			   (with-access::frame %page (height)
			      (set! height v))))))
	       ;; virtual background
	       (background
		(get (lambda (o)
			(with-access::notepad o (%cache)
			   (with-access::frame %cache (background)
			      background))))
		(set (lambda (o v)
			(with-access::notepad o (frame
						 %cache
						 %page
						 %tabs
						 %padding)
			   (with-access::frame frame (background)
			      (set! background v))
			   (with-access::frame %cache (background)
			      (set! background v))
			   (with-access::frame %page (background)
			      (set! background v))
			   (with-access::frame %padding (background)
			      (set! background v))
			   (for-each (lambda (b)
					(with-access::button b (background)
					   (set! background v)))
				     %tabs))
			#unspecified)))
	       ;; virtual foreground
	       (foreground
		(get (lambda (o)
			(with-access::notepad o (%tabs)
			   (with-access::button (car %tabs) (foreground)
			      foreground))))
		(set (lambda (o v)
			(with-access::notepad o (%tabs)
			   (for-each (lambda (b)
					(with-access::button b (foreground)
					   (set! foreground v)))
				     %tabs))
			#unspecified))))

	    (class notepad-tab::button
	       ;; notepad-tab initialization
	       (notepad-tab-init)
	       (action (default list))
	       (page (default #f)))

	    (notepad-init ::notepad)
	    (notepad-tab-init ::notepad-tab)))
	       
;*---------------------------------------------------------------------*/
;*    notepad-init ...                                                 */
;*---------------------------------------------------------------------*/
(define (notepad-init self::notepad)
   (with-access::notepad self (frame parent
			       init-width init-height
			       %tabs %cache %page %padding
			       id eid)
      (set! frame (if (eq? parent #unspecified)
		      (instantiate::frame
			 (highlight-thickness 0)
			 (class "NotePad"))
		      (instantiate::frame
			 (parent parent)
			 (highlight-thickness 0)
			 (class "NotePad"))))
      (let ((f (instantiate::frame
		  (parent frame)
		  (highlight-thickness 0)
		  (border-width 2)
		  (relief "raised")))
	    (c (instantiate::frame
		  (parent frame)
		  (highlight-thickness 0)
		  (border-width 0)
		  (height 9)))
	    (padding (instantiate::frame
		       (parent frame)
		       (highlight-thickness 0)
		       (border-width 0)
		       (width 10)
		       (height 9))))
	 ;; packing the frame
	 (pack f :side 'bottom :fill 'both :expand #t)
	 (pack 'propagate f #f)
	 ;; pack the padding frame
	 (pack padding :side 'left :fill 'y)
	 ;; Create a first button in the frame.
	 ;; This button is never mapped but serves 
	 ;; as a prototype for next tabs
	 (set! %tabs (list (instantiate::button (parent frame))))
	 (set! %cache c)
	 (set! %page f)
	 (set! %padding padding)
	 (set! id (frame-id frame))
	 (set! eid id)
	 (tk-command-object-set! eid self)
	 (add-binding! self "<Configure>" () (place-cache self)))))
  
;*---------------------------------------------------------------------*/
;*    notepad-tab-init ...                                             */
;*---------------------------------------------------------------------*/
(define (notepad-tab-init self::notepad-tab)
   (with-access::notepad-tab self (parent command active-background background
					  border-width)
      (if (not (notepad? parent))
	  (error "notepad-tab-init" "Parent not a ::notepad" parent)
	  (begin
	     (button-init self)
	     (set! command (lambda () (select-tab self)))
	     (set! background (notepad-background parent))
	     (set! active-background (notepad-background parent))
;* 	     (set! border-width 1)                                     */
	     (with-access::notepad parent (%tabs %current)
		;; if this is the first notepad-tab, we have to insert
		;; some pading in the parent before the button
		(set! %tabs (append %tabs (list self)))
		(pack self :side 'left :fill 'y)
		((if %current unselect-tab select-tab) self)
		(event 'generate parent "<Configure>"))))))

;*---------------------------------------------------------------------*/
;*    unselect-tab ...                                                 */
;*---------------------------------------------------------------------*/
(define (unselect-tab self::notepad-tab)
   (with-access::notepad-tab self (relief foreground page disabled-foreground)
      (pack 'forget page)
      (set! relief "sunken")
      (set! foreground disabled-foreground)))

;*---------------------------------------------------------------------*/
;*    select-tab ...                                                   */
;*---------------------------------------------------------------------*/
(define (select-tab self::notepad-tab)
   (with-access::notepad-tab self (parent relief foreground action)
      (with-access::notepad parent (%tabs %current %page)
	 (let ((proto (car %tabs)))
	    (set! relief "raised")
	    (set! foreground (button-foreground proto))
	    (if (and %current (not (eq? %current self)))
		(unselect-tab %current))
	    (set! %current self)
	    (event 'generate parent "<Configure>")
	    (action %page self)))))
	     
;*---------------------------------------------------------------------*/
;*    place-cache ...                                                  */
;*---------------------------------------------------------------------*/
(define (place-cache self::notepad)
   ;; Place the cache window (associated to the <Configure> event)
   (with-access::notepad self (%cache %current)
      (let ((bd (button-border-width %current)))
	 (update 'idle)
	 (place %cache :in %current :bordermode "inside"
		:x -1 :y  (- (- (winfo 'height %current) (* 2 bd)) 2)
		:width (- (winfo 'width  %current) (* 2 bd))
		:height (+ (* 2 bd) 1))
	 (raise %cache))))

