;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bglk/runtime/Lwidget/paned.scm              */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Mon Dec 21 14:39:02 1998                          */
;*    Last change :  Tue Jun  1 16:30:26 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    HPaned and VPaned composite widgets                              */
;*    -------------------------------------------------------------    */
;*    Source documentation:                                            */
;*       @path ../../manual/paned.texi@                                */
;*       @node Paned@                                                  */
;*    Example:                                                         */
;*       @path ../../demos/paned/paned.scm@                            */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_paned
   
   (include "Llib/biglook.sch")
   
   (option  (loadq "Llib/biglook.init"))
   
   (import  (__biglook_glue      "Llib/scmglue.scm")
	    (__biglook_misc      "Llib/misc.scm")
	    (__biglook_key       "Llib/key.scm")
	    (__biglook_tk-object "Llib/tk-object.scm")
	    (__biglook_frame     "Lwidget/frame.scm")
	    (__biglook_text      "Lwidget/text.scm")
	    (__biglook_scrollbar "Lwidget/scrollbar.scm"))
   
   (export  (class paned::widget
	       ;; frame
	       (frame (default #unspecified))
	       ;; frame-1
	       (frame-1 (default #unspecified))
	       ;; frame-2
	       (frame-2 (default #unspecified))
	       ;; grip
	       (grip (default #unspecified))
	       ;; separator
	       (separator (default #unspecified))
	       ;; fraction
	       (fraction (default 0.5))
	       ;; virtual background
	       (background
		(get (lambda (o)
			(with-access::paned o (frame)
			   (with-access::frame frame (background)
			      background))))
		(set (lambda (o v)
			(with-access::paned o (frame frame-1 frame-2
						     separator)
			   (with-access::frame frame (background)
			      (set! background v))
			   (with-access::frame frame-1 (background)
			      (set! background v))
			   (with-access::frame frame-2 (background)
			      (set! background v))
			   (with-access::frame separator (background)
			      (set! background v))))))
	       ;; virtual border-width
	       (border-width
		(get (lambda (o)
			(with-access::paned o (frame)
			   (with-access::frame frame (border-width)
			      border-width))))
		(set (lambda (o v)
			(with-access::paned o (frame)
			   (with-access::frame frame (border-width)
			      (set! border-width v))))))
	       ;; virtual relief
	       (relief
		(get (lambda (o)
			(with-access::paned o (frame)
			   (with-access::frame frame (relief)
			      relief))))
		(set (lambda (o v)
			(with-access::paned o (frame)
			   (with-access::frame frame (relief)
			      (set! relief v))))))
	       ;; virtual width
	       (width
		(get (lambda (o)
			(with-access::paned o (frame)
			   (with-access::frame frame (width)
			      width))))
		(set (lambda (o v)
			(with-access::paned o (frame)
			   (with-access::frame frame (width)
			      (set! width v))))))
	       ;; virtual height
	       (height
		(get (lambda (o)
			(with-access::paned o (frame)
			   (with-access::frame frame (height)
			      height))))
		(set (lambda (o v)
			(with-access::paned o (frame)
			   (with-access::frame frame (height)
			      (set! height v)))))))
	    
	    (class hpaned::paned
	       ;; initialization
	       (hpaned-init)
	       ;; top-frame
	       (top-frame
		(get (lambda (o)
			(paned-frame-1 o)))
		(set (lambda (o v)
			(paned-frame-1-set! o v))))
	       ;; bottom-frame
	       (bottom-frame
		(get (lambda (o)
			(paned-frame-2 o)))
		(set (lambda (o v)
			(paned-frame-2-set! o v)))))
	    
	    (class vpaned::paned
	       ;; initialization
	       (vpaned-init)
	       ;; left-frame
	       (left-frame
		(get (lambda (o)
			(paned-frame-1 o)))
		(set (lambda (o v)
			(paned-frame-1-set! o v))))
	       ;; right-frame
	       (right-frame
		(get (lambda (o)
			(paned-frame-2 o)))
		(set (lambda (o v)
			(paned-frame-2-set! o v)))))
	    
	    (paned-init  ::paned rel1 rel2 relsize1 relsize2 anchor size class)
	    (hpaned-init ::hpaned)
	    (vpaned-init ::vpaned)))

;*---------------------------------------------------------------------*/
;*    paned-init ...                                                   */
;*---------------------------------------------------------------------*/
(define (paned-init self::paned rel1 rel2 relsize1 relsize2 anchor size class)
   (with-access::paned self (frame id eid parent
				   frame-1 frame-2 grip separator fraction)
      ;; initialize true slots
      (set! frame (if (eq? parent #unspecified)
		      (instantiate::frame
			 (class  class))
		      (instantiate::frame
			 (parent parent)
			 (class  class))))
      (set! id (frame-id frame))
      (set! eid (frame-id frame))
      (tk-command-object-set! eid self)
      (set! grip (instantiate::frame
		    (parent frame)
		    (class "Grip")))
      (set! separator (instantiate::frame
			 (parent frame)
			 (class "Separator")
			 (width 4)
			 (height 4)))
      (set! frame-1 (instantiate::frame
		       (parent frame)
		       (border-width 2)
		       (relief "raised")))
      (set! frame-2 (instantiate::frame
		       (parent frame)
		       (border-width 2)
		       (relief "raised")))
      
      ;; Place the grip
      (place-grip self rel1 rel2 relsize1 relsize2 anchor)))
 
;*---------------------------------------------------------------------*/
;*    hpaned-init ...                                                  */
;*---------------------------------------------------------------------*/
(define (hpaned-init self)
   (paned-init self :relx :rely :relwidth :relheight "w" 'height "HPaned")
   (with-access::paned self (grip)
      ;; Associate bindings to the grip
      (add-binding! grip "<Button-1>" (y)
		    (start-grip  self y))
      (add-binding! grip "<B1-Motion>" (y)
		    (motion-grip self y
				 :relx :rely
				 :relwidth :relheight
				 "w" 'height))
      (add-binding! grip "<B1-ButtonRelease-1>" (y)
		    (place-grip self :relx :rely
				:relwidth :relheight
				"w"))))
      
;*---------------------------------------------------------------------*/
;*    vpaned-init ...                                                  */
;*---------------------------------------------------------------------*/
(define (vpaned-init self)
   (paned-init self :rely :relx :relheight :relwidth "n" 'width "VPaned")
   (with-access::paned self (grip)
      ;; Associate bindings to the grip 
      (add-binding! grip "<Button-1>" (x)
		    (start-grip  self x))
      (add-binding! grip "<B1-Motion>" (x)
		    (motion-grip self x
				 :rely :relx
				 :relheight :relwidth
				 "n" 'width))
      (add-binding! grip "<B1-ButtonRelease-1>" (x)
		    (place-grip self :rely :relx
				:relheight :relwidth
				"n"))))
   
;*---------------------------------------------------------------------*/
;*    drag-start ...                                                   */
;*---------------------------------------------------------------------*/
(define drag-start 0)

;*---------------------------------------------------------------------*/
;*    place-grip ...                                                   */
;*---------------------------------------------------------------------*/
(define (place-grip self::paned rel1 rel2 relsize1 relsize2 anchor)
   (with-access::paned self (fraction separator grip frame-1 frame-2)
      (place separator rel1 0 rel2 fraction :anch anchor relsize1 1)
      (place grip rel1 0.95 rel2 fraction :anchor "center")
      (place frame-1 :x 0 :y 0 relsize1 1 relsize2 fraction)
      (place frame-2 :x 0 :y 0 rel2 fraction relsize1 1
	     relsize2 (- 1.0 fraction))
      ;; Hide separator behind the bottom and top frames
      (lower separator)
      ;; Be sure to raise the grip 
      (raise grip)))

;*---------------------------------------------------------------------*/
;*    start-grip ...                                                   */
;*---------------------------------------------------------------------*/
(define (start-grip self::paned coord)
   (with-access::paned self (separator grip)
      ;; Raise separator and grip 
      (raise separator)
      (raise grip)
      (set! drag-start coord)))

;*---------------------------------------------------------------------*/
;*    motion-grip ...                                                  */
;*---------------------------------------------------------------------*/
(define (motion-grip self::paned y rel1 rel2 relsize1 relsize2 anchor size)
   (with-access::paned self (fraction separator grip frame)
      (set! fraction (max 0.0001 
			  (min 0.9999
			       (+ fraction 
				  (/ (- y drag-start) 
				     (+ 1 (winfo size frame)))))))
      (place separator rel1 0 relsize1 1 rel2 fraction :anchor anchor)
      (place grip      rel1 0.95 rel2 fraction :anchor "center")))

;*---------------------------------------------------------------------*/
;*    Resources                                                        */
;*---------------------------------------------------------------------*/
(option 'add "*Grip.Background"       "black"             "widgetDefault")
(option 'add "*Grip.Relief"           "raised"            "widgetDefault")
(option 'add "*Grip.BorderWidth"      2                   "widgetDefault")

(option 'add "*HPaned.Grip.Cursor"    "sb_v_double_arrow" "widgetDefault")
(option 'add "*HPaned.Grip.Width"     12                   "widgetDefault")
(option 'add "*HPaned.Grip.Height"    8                   "widgetDefault")
(option 'add "*VPaned.Grip.Cursor"    "sb_h_double_arrow" "widgetDefault")
(option 'add "*VPaned.Grip.Width"     8                   "widgetDefault")
(option 'add "*VPaned.Grip.Height"    12                  "widgetDefault")

(option 'add "*Separator.BorderWidth" 2                   "widgetDefault")
(option 'add "*Separator.Relief"      "solid"             "widgetDefault")
	       








			
