;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bglk/runtime/Lwidget/scale.scm              */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Tue Oct 20 17:07:59 1998                          */
;*    Last change :  Wed Jun 23 13:34:27 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The Scale widget                                                 */
;*    -------------------------------------------------------------    */
;*    Source documentations:                                           */
;*       @path ../../manual/scale.texi@                                */
;*       @node Scale@                                                  */
;*    Examples:                                                        */
;*       @path ../../demos/scale/scale.scm@                            */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_scale

   (include "Llib/biglook.sch")

   (option  (loadq "Llib/biglook.init"))
   
   (import  (__biglook_glue "Llib/scmglue.scm")
	    (__biglook_misc "Llib/misc.scm")
	    (__biglook_tk-object "Llib/tk-object.scm"))

   (export  (class scale::simple-widget
	       (scale-init)
	       ;; active-background
	       (active-background::bstring
		(get (lambda (o)
			(tk o 'cget :activebackground)))
		(set (lambda (o v)
			(tk o 'configure :activebackground v))))
	       ;; command
	       (command
		(get (lambda (o)
			(tk o 'cget :command)))
		(set (lambda (o v)
			(if (and (procedure? v) (correct-arity? v 1))
			    (tk o 'configure :command v)
			    (error "scale-command-set!"
				   "Illegal procedure"
				   v)))))
	       ;; font
	       (font
		(get (lambda (o)
			(tk o 'cget :font)))
		(set (lambda (o v)
			(tk o 'configure :font v))))
	       ;; foreground
	       (foreground::bstring
		(get (lambda (o)
			(tk o 'cget :foreground)))
		(set (lambda (o v)
			(tk o 'configure :foreground v))))
	       ;; resolution
	       (resolution
		(get (lambda (o)
			(tk o 'cget :resolution)))
		(set (lambda (o v)
			(tk o 'configure :resolution v))))
	       ;; from
	       (from
		(get (lambda (o)
			(tk o 'cget :from)))
		(set (lambda (o v)
			(tk o 'configure :from v))))

	       ;; length
	       (length::int
		(get (lambda (o)
			(tk o 'cget :length)))
		(set (lambda (o v)
			(tk o 'configure :length v))))
	       ;; orientation
	       (orientation
		(get (lambda (o)
			(tk o 'cget :orient)))
		(set (lambda (o v)
			(tk o 'configure :orient v))))
	       ;; show-value
	       (show-value::bool
		(get (lambda (o)
			(tk o 'cget :showvalue)))
		(set (lambda (o v)
			(tk o 'configure :showvalue v))))
	       ;; slider-length
	       (slider-length::int
		(get (lambda (o)
			(tk o 'cget :sliderlength)))
		(set (lambda (o v)
			(tk o 'configure :sliderlength v))))
	       ;; state
	       (state
		(get (lambda (o)
			(tk o 'cget :state)))
		(set (lambda (o v)
			(tk o 'configure :state v))))
	       ;; text
	       (text::bstring
		(get (lambda (o)
			(tk o 'cget :label)))
		(set (lambda (o v)
			(tk o 'configure :label v))))
	       ;; to
	       (to
		(get (lambda (o)
			(tk o 'cget :to)))
		(set (lambda (o v)
			(tk o 'configure :to v))))
	       ;; trough-color
	       (trough-color
		(get (lambda (o)
			(tk o 'cget :troughcolor)))
		(set (lambda (o v)
			(tk o 'configure :troughcolor v))))
	       ;; variable
	       (variable
		(get (lambda (o)
			(tk o 'cget :variable)))
		(set (lambda (o v)
			(tk o 'configure :variable v))))
	       ;; width
	       (width::int
		(get (lambda (o)
			(tk o 'cget :width)))
		(set (lambda (o v)
			(tk o 'configure :width v))))
	       ;; value
	       (value
		(get (lambda (o)
			(tk o 'get)))
		(set (lambda (o v)
			(tk o 'set v)))))

	    scale-init))

;*---------------------------------------------------------------------*/
;*    scale widget ...                                                 */
;*---------------------------------------------------------------------*/
(define scale-init
   (let ((scale (make-widget-constructor "scale")))
      (lambda (o)
	 (simple-widget-init o scale))))

;*---------------------------------------------------------------------*/
;*    Global parameters                                                */
;*---------------------------------------------------------------------*/
(define dragging   #f)
(define init-value #f)
(define delta-x	   0)
(define delta-y	   0)
(define active-bg  "")

;*---------------------------------------------------------------------*/
;*    The code below creates the default class bindings                */
;*    for entries.                                                     */
;*---------------------------------------------------------------------*/
;; Standard Motif bindings:
(add-binding! "Scale" "<Enter>" (WW x y)
  (when *tk-strict-Motif* 
     (set! active-bg (tk-get WW :activebackground))
     (tk-set! WW :activebackground (tk-get WW :background)))
  (scale-activate WW x y))

(add-binding! "Scale" "<Motion>" (WW x y)
  (scale-activate WW x y))

(add-binding! "Scale" "<Leave>" (WW)
  (if *tk-strict-Motif*
      (tk-set! WW :activebackground active-bg))
  (if (equal? (tk-get WW :state) "active")
      (tk-set! WW :state "normal")))

(add-binding! "Scale" "<1>" (WW x y)
  (scale-button-down WW x y))

(add-binding! "Scale" "<B1-Motion>" (WW x y)
  (scale-drag WW x y))

(add-binding! "Scale" "<B1-Leave>" () "")

(add-binding! "Scale" "<B1-Enter>" () "")

(add-binding! "Scale" "<ButtonRelease-1>" (WW x y)
  (cancel-repeat)
  (scale-end-drag WW)
  (scale-activate WW x y))

(add-binding! "Scale" "<2>" (WW x y)
  (scale-button-2-down WW x y))

(add-binding! "Scale" "<B2-Motion>" (WW x y)
  (scale-drag WW x y))

(add-binding! "Scale" "<B2-Leave>" () "")

(add-binding! "Scale" "<B2-Enter>" () "")

(add-binding! "Scale" "<ButtonRelease-2>" (WW x y)
  (cancel-repeat)
  (scale-end-drag WW)
  (scale-activate WW x y))

(add-binding! "Scale" "<Control-1>" (WW x y)
  (scale-control-press WW x y))

(add-binding! "Scale" "<Up>" (WW)
  (scale-increment WW 'up 'little 'no-repeat))

(add-binding! "Scale" "<Down>" (WW)
  (scale-increment WW 'down 'little 'no-repeat))

(add-binding! "Scale" "<Left>" (WW)
  (scale-increment WW 'up 'little 'no-repeat))

(add-binding! "Scale" "<Right>" (WW)
  (scale-increment WW 'down 'little 'no-repeat))

(add-binding! "Scale" "<Control-Up>" (WW)
  (scale-increment WW 'up 'big 'no-repeat))

(add-binding! "Scale" "<Control-Down>" (WW)
  (scale-increment WW 'down 'big 'no-repeat))

(add-binding! "Scale" "<Control-Left>" (WW)
  (scale-increment WW 'up 'big 'no-repeat))

(add-binding! "Scale" "<Control-Right>" (WW)
  (scale-increment WW 'down 'big 'no-repeat))

(add-binding! "Scale" "<Home>" (WW)
  (WW 'set (tk-get WW :from)))

(add-binding! "Scale" "<End>" (WW)
  (WW 'set (tk-get WW :to)))

;*---------------------------------------------------------------------*/
;*    scale-activate ...                                               */
;*    -------------------------------------------------------------    */
;*    This procedure is invoked to check a given x-y position in the   */
;*    scale and activate the slider if the x-y position falls within   */
;*    the slider.                                                      */
;*    w    -	The scale widget.                                      */
;*    x, y -	Mouse coordinates.                                     */
;*---------------------------------------------------------------------*/
(define (scale-activate w x y)
   (unless (equal? (tk-get w :state) "disabled")
	   (tk-set! w :state (if (equal? (w 'identify x y) "slider")
				 "active" "normal"))))

;*---------------------------------------------------------------------*/
;*    scale-button-down ...                                            */
;*    -------------------------------------------------------------    */
;*    This procedure is invoked when a button is pressed in a scale.   */
;*    it takes different actions depending on where the button was     */
;*    pressed.                                                         */
;*    w    -  The scale widget.                                        */
;*    x, y -	Mouse coordinates of button press.                     */
;*---------------------------------------------------------------------*/
(define (scale-button-down w x y)
   (let ((el (w 'identify x y)))
      (set! dragging #f)
      (cond
	 ((string=? el "trough1")
	  (scale-increment w 'up   'little 'initial))
	 ((string=? el "trough2")
	  (scale-Increment w 'down 'little 'initial))
	 ((string=? el "slider")
	  (set! dragging #t)
	  (set! init-value (w 'get))
	  (let ((coords (w 'coords)))
	     (set! delta-x (- x (car  coords)))
	     (set! delta-y (- y (cadr coords))))))))
;* 	     (w 'configure :sliderrelief "sunken")                */

;*---------------------------------------------------------------------*/
;*    scale-drag ...                                                   */
;*    -------------------------------------------------------------    */
;*    This procedure is called when the mouse is dragged with          */
;*    mouse button 1 down.  If the drag started inside the slider      */
;*    (i.e. the scale is active) then the scale's value is adjusted    */
;*    to reflect the mouse's position.                                 */
;*    w    - The scale widget.                                         */
;*    x, y - Mouse coordinates.                                        */
;*---------------------------------------------------------------------*/
(define (scale-drag w x y)
   (when dragging
	 (w 'set (w 'get (- x delta-x) (- y delta-y)))))

;*---------------------------------------------------------------------*/
;*    scale-end-drag ...                                               */
;*    -------------------------------------------------------------    */
;*    This procedure is called to end an interactive drag of the       */
;*    slider. It just marks the drag as over.                          */
;*---------------------------------------------------------------------*/
(define (scale-end-drag w)
   (set! dragging #f)
   (w 'configure :sliderrelief "raised"))

;*---------------------------------------------------------------------*/
;*    scale-increment ...                                              */
;*---------------------------------------------------------------------*/
;*    This procedure is invoked to increment the value of a scale and  */
;*    to set up auto-repeating of the action if that is desired.  The  */
;*    way the value is incremented depends on the "dir" and "big"      */
;*    arguments.                                                       */
;*    w  -	The scale widget.                                      */
;*    dir -	"up" means move value towards -from, "down" means      */
;*    	move towards -to.                                              */
;*    size -	Size of increments: "big" or "little".                 */
;*    repeat -Whether and how to auto-repeat the action:  "no-repeat"  */
;*    	means don't auto-repeat, "initial" means this is the           */
;*    	first action in an auto-repeat sequence, and "again"           */
;*    	means this is the second repetition or later.                  */
;*---------------------------------------------------------------------*/
(define (scale-increment w dir size repeat)
   (when (winfo 'exists w)
	 (let ((inc  0)
	       (from (tk-get w :from))
	       (to   (tk-get w :to)))
	    
	    (if (eqv? size 'big)
		(begin
		   (set! inc (tk-get w :bigincrement))
		   (if (= inc 0)
		       (set! inc (abs (/ (- to from) 10.0))))
		   (set! inc (max (tk-get w :resolution) inc)))
		(set! inc (tk-get w :resolution)))
	    
	    (if (or (and (> from to) (eqv? dir 'down))
		    (and (<= from to) (eqv? dir 'up)))
		(set! inc (- inc)))
	    
	    (w 'set (+ (w 'get) inc))
	    
	    (case repeat
	       ((again)
		(set! *after-id* 
		      (after (tk-get w :repeatinterval)
			     (lambda () 
				(scale-increment w dir size 'again)))))
	       ((initial)
		(let ((delay (tk-get w :repeatdelay)))
		   (if (> delay 0)
		       (set! *after-id* 
			     (after delay
				    (lambda () 
				       (scale-increment w dir 
							size 'again)))))))))))

;*---------------------------------------------------------------------*/
;*    scale-control-press ...                                          */
;*    -------------------------------------------------------------    */
;*    This procedure handles button presses that are made with the     */
;*    Control key down. Depending on the mouse position, it adjusts    */
;*    the scale value to one end of the range or the other.            */
;*    Arguments:                                                       */
;*    w    - The scale widget.                                         */
;*    x, y - Mouse coordinates where the button was pressed.           */
;*---------------------------------------------------------------------*/
(define (scale-control-press w x y)
  (let ((el (w 'identify x y)))
    (cond
      ((string=? el "trough1")  (w 'set (tk-get w :from)))
      ((string=? el "trough2")  (w 'set (tk-get w :to))))))

;*---------------------------------------------------------------------*/
;*    scale-button-2-down ...                                          */
;*    -------------------------------------------------------------    */
;*    This procedure is invoked when button 2 is pressed over a scale. */
;*    It sets the value to correspond to the mouse position and starts */
;*    a slider drag.                                                   */
;*    Arguments:                                                       */
;*    w    - The scrollbar widget.                                     */
;*    x, y - Mouse coordinates within the widget.                      */
;*---------------------------------------------------------------------*/
(define (scale-button-2-down w x y)
   (unless (equal? (tk-get w :state) "disabled")
	   (tk-set! w :state "active")
	   (w 'set (w 'get x y))
	   (set! dragging #t)
	   (set! init-value (w 'get))
	   (set! delta-x 0)
	   (set! delta-y 0)))


   
