;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bglk/runtime/Lwidget/scrollbar.scm          */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Tue Oct 20 17:39:15 1998                          */
;*    Last change :  Mon Apr 26 15:58:28 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The scrollbar widget                                             */
;*    -------------------------------------------------------------    */
;*    Source documentation:                                            */
;*       @path ../../manual/scrollbar.texi@                            */
;*       @node Scrollbar@                                              */
;*    Examples:                                                        */
;*       @path ../../demos/listbox/listbox.scm@                        */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_scrollbar

   (include "Llib/biglook.sch")

   (option  (loadq "Llib/biglook.init"))
   
   (import  (__biglook_glue "Llib/scmglue.scm")
	    (__biglook_misc "Llib/misc.scm")
	    (__biglook_tk-object "Llib/tk-object.scm"))

   (export  (class scrollbar::simple-widget
	       (scrollbar-init)
	       ;; active-background
	       (active-background::bstring
		(get (lambda (o)
			(tk o 'cget :activeback)))
		(set (lambda (o v)
			(tk o 'configure :activeback v))))	       
	       ;; active-relief
	       (active-relief
		(get (lambda (o)
			(tk o 'cget :activerelief)))
		(set (lambda (o v)
			(tk o 'configure :activerelief v))))	       
	       ;; command
	       (command
		(get (lambda (o)
			(tk o 'cget :command)))
		(set (lambda (o v)
			(if (and (procedure? v) (correct-arity? v 1))
			    (tk o 'configure :command v)
			    (error "scrollbar-command-set!"
				   "Illegal procedure"
				   v)))))
	       ;; orientation
	       (orientation
		(get (lambda (o)
			(tk o 'cget :orient)))
		(set (lambda (o v)
			(tk o 'configure :orient v))))	       
	       ;; trough-color
	       (trough-color
		(get (lambda (o)
			(tk o 'cget :troughcolor)))
		(set (lambda (o v)
			(tk o 'configure :troughcolor v))))	       
	       ;; width
	       (width::int
		(get (lambda (o)
			(tk o 'cget :width)))
		(set (lambda (o v)
			(tk o 'configure :width v)))
		(default 12))
	       ;; value
	       (value
		(get (lambda (o)
			(tk o 'get)))
		(set (lambda (o v)
			(apply tk o 'set v)))))

	    scrollbar-init

	    (h-scroll-side      ::scrollbar)
	    (h-scroll-side-set! ::scrollbar ::obj)
	    (v-scroll-side      ::scrollbar)
	    (v-scroll-side-set! ::scrollbar ::obj)))

;*---------------------------------------------------------------------*/
;*    scrollbar widget ...                                             */
;*---------------------------------------------------------------------*/
(define scrollbar-init
   (let ((scrollbar (make-widget-constructor "scrollbar")))
      (lambda (o)
	 (simple-widget-init o scrollbar))))


;*---------------------------------------------------------------------*/
;*    Global parameters                                                */
;*---------------------------------------------------------------------*/
(define init-pos    "")
(define init-values '())
(define active-bg   "")
(define press-x     0)
(define press-y     0)
(define relief      "sunken")

;*---------------------------------------------------------------------*/
;*    The code below creates the default class bindings                */
;*    for entries.                                                     */
;*---------------------------------------------------------------------*/
;; Standard Motif bindings:
(add-binding! "Scrollbar" "<Enter>" (WW x y)
	      (when *tk-strict-motif* 
		    (set! active-bg (tk-get WW :activebackground))
		    (tk-set! WW :activebackground (tk-get WW :background)))
	      (WW 'activate (WW 'identify x y)))

(add-binding! "Scrollbar" "<Motion>" (WW x y)
	      (WW 'activate (WW 'identify x y)))

(add-binding! "Scrollbar" "<Leave>" (WW)
	      (if *tk-strict-motif*
		  (tk-set! WW :activebackground active-bg))
	      (WW 'activate ""))

(add-binding! "Scrollbar" "<1>" (WW x y)
	      (scroll-button-down WW x y))

(add-binding! "Scrollbar" "<B1-Motion>" (WW x y)
	      (scroll-drag WW x y))

(add-binding! "Scrollbar" "<B1-B2-Motion>" (WW x y)
	      (scroll-drag WW x y))

(add-binding! "Scrollbar" "<ButtonRelease-1>" (WW x y)
	      (scroll-button-up WW x y))

(add-binding! "Scrollbar" "<B1-Leave>" ()
	      ;; Prevents <Leave> binding from being invoked.
	      'nop)

(add-binding! "Scrollbar" "<B1-Enter>" ()
	      ;; Prevents <Enter> binding from being invoked.
	      'nop)

(add-binding! "Scrollbar" "<2>" (WW x y)
	      (scroll-button-2-down WW x y))

(add-binding! "Scrollbar" "<B1-2>" ()
	      ; Do nothing, since button 1 is already down.
	      'nop)

(add-binding! "Scrollbar" "<B2-1>" (WW x y)
	      ; Do nothing, since button 2 is already down.
	      'nop)

(add-binding! "Scrollbar" "<B2-Motion>" (WW x y)
	      (scroll-drag WW x y))

(add-binding! "Scrollbar" "<ButtonRelease-2>" (WW x y)
	      (scroll-button-up WW x y))

(add-binding! "Scrollbar" "<B1-ButtonRelease-2>" ()
	      ;Do nothing:  B1 release will handle it.
	      'nop)

(add-binding! "Scrollbar" "<B2-ButtonRelease-1>" ()
	      ;Do nothing:  B1 release will handle it.
	      'nop)

(add-binding! "Scrollbar" "<B2-Leave>" ()
	      ;; Prevents <Leave> binding from being invoked.
	      'nop)

(add-binding! "Scrollbar" "<B2-Enter>" ()
	      ;; Prevents <Enter> binding from being invoked.
	      'nop)

(add-binding! "Scrollbar" "<Control-1>" (WW x y)
	      (scroll-top-bottom WW x y))

(add-binding! "Scrollbar" "<Control-2>" (WW x y)
	      (scroll-top-bottom WW x y))

(add-binding! "Scrollbar" "<Up>" (WW)
	      (scroll-by-units WW 'v -1))
(add-binding! "Scrollbar" "<Down>" (WW)
	      (scroll-by-units WW 'v +1))
(add-binding! "Scrollbar" "<Control-Up>" (WW)
	      (scroll-by-pages WW 'v -1))
(add-binding! "Scrollbar" "<Control-Down>" (WW)
	      (scroll-by-pages WW 'v +1))
(add-binding! "Scrollbar" "<Left>" (WW)
	      (scroll-by-units WW 'h -1))
(add-binding! "Scrollbar" "<Right>" (WW)
	      (scroll-by-units WW 'h +1))
(add-binding! "Scrollbar" "<Control-Left>" (WW)
	      (scroll-by-pages WW 'h -1))
(add-binding! "Scrollbar" "<Control-Right>" (WW)
	      (scroll-by-pages WW 'hd +1))
(add-binding! "Scrollbar" "<Prior>" (WW)
	      (scroll-by-pages WW 'hv -1))
(add-binding! "Scrollbar" "<Next>" (WW)
	      (scroll-by-pages WW 'hv +1))

(add-binding! "Scrollbar" "<Home>" (WW)
	      (scroll-to-pos WW 0))
(add-binding! "Scrollbar" "<End>" (WW)
	      (scroll-to-pos WW 1))


;; scroll-button-down --
;; This procedure is invoked when a button is pressed in a scrollbar.
;; It changes the way the scrollbar is displayed and takes actions
;; depending on where the mouse is.
;;
;; w -		The scrollbar widget.
;; x, y -	Mouse coordinates.

(define (scroll-button-down w x y)
   (let ((element (w 'identify x y)))
      (set! relief (tk-get w :activerelief))
;*       (tk-set! w :activerelief "sunken")                            */
      (if (equal? element "slider")
	  (scroll-start-drag w x y)
	  (scroll-select w element "initial"))))

;; scroll-button-up --
;; This procedure is invoked when a button is released in a scrollbar.
;; It cancels scans and auto-repeats that were in progress, and restores
;; the way the active element is displayed.
;;
;; w -		The scrollbar widget.
;; x, y -	Mouse coordinates.

(define (scroll-button-up w x y)
  (cancel-repeat)
  (tk-set! w :activerelief relief)
  (scroll-end-drag w x y)
  (w 'activate (w 'identify x y)))


;; scroll-select --
;; This procedure is invoked when a button is pressed over the scrollbar.
;; It invokes one of several scrolling actions depending on where in
;; the scrollbar the button was pressed.
;;
;; w -		The scrollbar widget.
;; element -	The element of the scrollbar that was selected, such
;;		as "arrow1" or "trough2".  Shouldn't be "slider".
;; repeat -	WWhether and how to auto-repeat the action:  "noRepeat"
;;		means don't auto-repeat, "initial" means this is the
;;		first action in an auto-repeat sequence, and "again"
;;		means this is the second repetition or later.

(define (scroll-select w element repeat)
   (when (winfo 'exists w)
	 (let ((k (lambda ()
		     (cond
			((string=? repeat "again") 
			 (set! *after-id*
			       (after (tk-get w :repeatinterval)
				      (lambda ()
					 (scroll-select w 
							element 
							"again")))))
			((string=? repeat "initial")
			 (let ((delay (tk-get w :repeatdelay)))
			    (if (> delay 0)
				(set! *after-id* 
				      (after delay
					     (lambda ()
						(scroll-select w 
							       element 
							       "again")))))))))))
	    (cond
	       ((equal? element "arrow1")
		(scroll-by-units w 'hv -1) (k))
	       ((equal? element "trough1")
		(scroll-by-pages w 'hv -1) (k))
	       ((equal? element "trough2")
		(scroll-by-pages w 'hv +1) (k))
	       ((equal? element "arrow2")
		(scroll-by-units w 'hv +1) (k))))))


;; scroll-start-drag --
;; This procedure is called to initiate a drag of the slider.  It just
;; remembers the starting position of the mouse and slider.
;;
;; w -		The scrollbar widget.
;; x, y -	The mouse position at the start of the drag operation.

(define (scroll-start-drag w x y)
   (unless (equal? (tk-get w :command) "")
	   (set! press-x x)
	   (set! press-y y)
	   (set! init-values (w 'get))
	   (let ((iv0 (car init-values)))
	      (if (= (length init-values) 2)
		  (set! init-pos iv0)
		  (if (= iv0 0)
		      (set! init-pos 0.0)
		      (set! init-pos (/ (caddr init-values) 
					(car init-values))))))))

;; scroll-drag --
;; This procedure is called for each mouse motion even when the slider
;; is being dragged.  It notifies the associated widget if we're not
;; jump scrolling, and it just updates the scrollbar if we are jump
;; scrolling.
;;
;; w -		The scrollbar widget.
;; x, y -	The current mouse position.

(define (scroll-drag w x y)
   (unless (equal? init-pos "")
	   (let ((delta (w 'delta (- x press-x) (- y press-y))))
	      (if (tk-get w :jump)
		  (if (equal? (length init-values) 2)
		      (w 'set (+ (car  init-values) delta)
		         (+ (cadr init-values) delta))
		      (let ((delta (floor (* delta (car init-values)))))
			 (w 'set (car  init-values)
			    (cadr init-values)
			    (+ (caddr init-values)  delta)
			    (+ (cadddr init-values) delta))))
		  (scroll-to-pos w (+ init-pos delta))))))

;; scroll-end-drag --
;; This procedure is called to end an interactive drag of the slider.
;; It scrolls the window if we're in jump mode, otherwise it does nothing.
;;
;; w -		The scrollbar widget.
;; x, y -	The mouse position at the end of the drag operation.

(define  (scroll-end-drag w x y)
   (unless (equal? init-pos "")
	   (if (tk-get w :jump)
	       (let ((delta (w 'delta (- x press-x) (- y press-y))))
		  (scroll-to-pos w (+ init-pos delta))))
	   (set! init-pos "")))


;; scroll-by-units --
;; This procedure tells the scrollbar's associated widget to scroll up
;; or down by a given number of units.  It notifies the associated widget
;; in different ways for old and new command syntaxes.
;;
;; w -		The scrollbar widget.
;; orient -	Which kinds of scrollbars this applies to:  "h" for
;;		horizontal, "v" for vertical, "hv" for both.
;; amount -	How many units to scroll:  typically 1 or -1.

(define (scroll-by-units w orient amount)
   (let ((cmd     (tk-get w :command))
	 (worient (tk-get w :orient)))
      (unless (equal? cmd "")
	      (when (or (eq? orient 'hv) 
			(and (eq? orient 'h) (string=? worient "horizontal"))
			(and (eq? orient 'v) (string=? worient "vertical")))
		    (let ((info (w 'get)))
		       (if (= (length info) 2)
			   (cmd 'scroll amount 'units)
			   (cmd (+ (caddr info) amount))))))))

;; scroll-by-pages --
;; This procedure tells the scrollbar's associated widget to scroll up
;; or down by a given number of screenfuls.  It notifies the associated
;; widget in different ways for old and new command syntaxes.
;;
;; Arguments:
;; w -		The scrollbar widget.
;; orient -	Which kinds of scrollbars this applies to:  "h" for
;;		horizontal, "v" for vertical, "hv" for both.
;; amount -	How many screens to scroll:  typically 1 or -1.

(define (scroll-by-pages w orient amount)
  (let ((cmd     (tk-get w :command))
	(worient (tk-get w :orient)))
    (unless (equal? cmd "")
       (when (or (eq? orient 'hv) 
		 (and (eq? orient 'h) (string=? worient "horizontal"))
		 (and (eq? orient 'v) (string=? worient "vertical")))
	   (let ((info (w 'get)))
	     (if (= (length info) 2)
		 (cmd 'scroll amount 'pages)
		 (cmd (+ (caddr info) (* (cadr info) amount) -1))))))))

;; scroll-ToPos --
;; This procedure tells the scrollbar's associated widget to scroll to
;; a particular location, given by a fraction between 0 and 1.  It notifies
;; the associated widget in different ways for old and new command syntaxes.
;;
;; Arguments:
;; w -		The scrollbar widget.
;; pos -		A fraction between 0 and 1 indicating a desired position
;;		in the document.

(define (scroll-to-pos w pos)
  (let ((cmd (tk-get w :command)))
    (unless (equal? cmd "")
	(let ((info (w 'get)))
	  (if (= (length info) 2)
	      (cmd 'moveto pos)
	      (cmd (floor (* (car info) pos))))))))

;; scroll-top-bottom
;; Scroll to the top or bottom of the document, depending on the mouse
;; position.
;;
;; w -		The scrollbar widget.
;; x, y -	Mouse coordinates within the widget.

(define (scroll-top-bottom w x y)
  (let ((element (w 'identify x y)))
    (cond
       ((member element '("arrow1" "trough1")) (scroll-to-pos w 0))
       ((member element '("arrow2" "trough2")) (scroll-to-pos w 1)))

    ;; Set relief, since it's needed by scroll-button-up.
    (set! relief (tk-get w :activerelief))))


;; scroll-button-2-down
;; This procedure is invoked when button 2 is pressed over a scrollbar.
;; If the button is over the trough or slider, it sets the scrollbar to
;; the mouse position and starts a slider drag.  Otherwise it just
;; behaves the same as button 1.
;;
;; Arguments:
;; w -		The scrollbar widget.
;; x, y -	Mouse coordinates within the widget.

(define (scroll-button-2-down w x y)
   (let ((element (w 'identify x y)))
      (if (or (equal? element "arrow1") (equal? element "arrow2"))
	  (scroll-button-down w x y)
	  (begin
	     (scroll-to-pos w (w 'fraction x y))
	     (set! relief (tk-get w :activerelief))
	     
	     ; Need the "update idletasks" below so that the widget calls us
	     ; back to reset the actual scrollbar position before we start the
	     ; slider drag.
	     (update 'idletasks)
;* 	     (tk-set! w :activerelief "sunken")                        */
	     (w 'activate 'slider)
	     (scroll-start-drag w x y)))))

;*---------------------------------------------------------------------*/
;*    get-keyword ...                                                  */
;*---------------------------------------------------------------------*/
(define (get-keyword kwd lst)
   (cadr (member kwd lst)))

;*---------------------------------------------------------------------*/
;*    h-scroll-side ...                                                */
;*---------------------------------------------------------------------*/
(define (h-scroll-side hs::scrollbar)
   (let ((info (grid 'info hs)))
      (if (null? info)
	  #f
	  (case (get-keyword :row info)
	     ((0) "top")
	     ((2) "bottom")))))

;*---------------------------------------------------------------------*/
;*    h-scroll-side-set! ...                                           */
;*---------------------------------------------------------------------*/
(define (h-scroll-side-set! hs::scrollbar v)
   (cond 
      ((equal? v "bottom")  (grid hs :row 2 :column 1))
      ((equal? v "top")     (grid hs :row 0 :column 1))
      ((not v)	            (grid 'remove hs))))

;*---------------------------------------------------------------------*/
;*    v-scroll-side ...                                                */
;*---------------------------------------------------------------------*/
(define (v-scroll-side vs::scrollbar)
   (let ((info (grid 'info vs)))
      (if (null? info)
	  #f
	  (case (get-keyword :column info)
	     ((0) "left")
	     ((2) "right")))))

;*---------------------------------------------------------------------*/
;*    v-scroll-side-set! ...                                           */
;*---------------------------------------------------------------------*/
(define (v-scroll-side-set! vs::scrollbar v)
   (cond
      ((equal? v "left")  (grid vs :row 1 :column 0))
      ((equal? v "right") (grid vs :row 1 :column 2))
      ((not v)	          (grid 'remove vs))))

