;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    .../serrano/prgm/project/bglk/runtime/Lwidget/scrollbox.scm      */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Mon Dec 21 12:18:25 1998                          */
;*    Last change :  Fri May  7 16:12:19 1999 ()                       */
;*    -------------------------------------------------------------    */
;*    Scrollbox                                                        */
;*    -------------------------------------------------------------    */
;*    Source documentation:                                            */
;*       @path ../../manual/scrollbox.texi@                            */
;*       @node Scrollbox@                                              */
;*    Example:                                                         */
;*       @path ../../demos/scrollbox/scrollbox.scm@                    */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_scrollbox
    
   (include "Llib/biglook.sch")
   
   (option  (loadq "Llib/biglook.init"))
   
   (import  (__biglook_glue      "Llib/scmglue.scm")
	    (__biglook_misc      "Llib/misc.scm")
	    (__biglook_key       "Llib/key.scm")
	    (__biglook_tk-object "Llib/tk-object.scm")
	    (__biglook_frame     "Lwidget/frame.scm")
	    (__biglook_listbox   "Lwidget/listbox.scm")
	    (__biglook_scrollbar "Lwidget/scrollbar.scm"))
   
   (export  (class scrollbox::listbox
	       ;; initialization
	       (scrollbox-init)
	       ;; frame
	       (frame (default #unspecified))
	       ;; listbox
	       (listbox (default #unspecified))
	       ;; h-scrollbar
	       (h-scrollbar (default #unspecified))
	       ;; v-scrollbar
	       (v-scrollbar (default #unspecified))
	       ;; h-scroll-side
	       (h-scroll-side
		(get (lambda (o)
			(with-access::scrollbox o (h-scrollbar)
			   (h-scroll-side h-scrollbar))))
		(set (lambda (o v)
			(with-access::scrollbox o (h-scrollbar)
			   (h-scroll-side-set! h-scrollbar v)))))
	       ;; v-scroll-side
	       (v-scroll-side
		(get (lambda (o)
			(with-access::scrollbox o (v-scrollbar)
			   (v-scroll-side v-scrollbar))))
		(set (lambda (o v)
			(with-access::scrollbox o (v-scrollbar)
			   (v-scroll-side-set! v-scrollbar v)))))
	       ;; background
	       (background
		(get (lambda (o)
			(with-access::scrollbox o (frame)
			   (frame-background frame))))
		(set (lambda (o v)
			(with-access::scrollbox o (frame listbox h-scrollbar v-scrollbar)
			   (frame-background-set! frame v)
			   (listbox-background-set! listbox v)
			   (scrollbar-background-set! h-scrollbar v)
			   (scrollbar-background-set! v-scrollbar v)))))
	       ;; trough-color
	       (trough-color
		(get (lambda (o)
			(with-access::scrollbox o (h-scrollbar v-scrollbar)
			   (tk h-scrollbar 'cget :troughcolor))))
		(set (lambda (o v)
			(with-access::scrollbox o (h-scrollbar v-scrollbar)
			   (tk h-scrollbar 'conf :troughcolor v)
			   (tk v-scrollbar 'conf :troughcolor v)))))
	       ;; scrollbar-width
	       (scrollbar-width
		(get (lambda (o)
			(with-access::scrollbox o (h-scrollbar v-scrollbar)
			   (tk h-scrollbar 'cget :width))))
		(set (lambda (o v)
			(with-access::scrollbox o (h-scrollbar v-scrollbar)
			   (tk h-scrollbar 'conf :width v)
			   (tk v-scrollbar 'conf :width v)))))
	       ;; border-width
	       (border-width
		(get (lambda (o)
			(with-access::scrollbox o (frame)
			   (frame-border-width frame))))
		(set (lambda (o v)
			(with-access::scrollbox o (frame)
			   (frame-border-width-set! frame v)))))
	       (relief
		(get (lambda (o)
			(with-access::scrollbox o (frame)
			   (frame-relief frame))))
		(set (lambda (o v)
			(with-access::scrollbox o (frame)
			   (frame-relief-set! frame v))))))
	    
	    (scrollbox-init ::scrollbox)))

;*---------------------------------------------------------------------*/
;*    scrollbox-init ...                                               */
;*---------------------------------------------------------------------*/
(define (scrollbox-init widget::scrollbox)
   (with-access::scrollbox widget (id eid parent frame listbox
					h-scrollbar v-scrollbar)
      (set! frame (if (eq? parent #unspecified)
		      (instantiate::frame)
		      (instantiate::frame (parent parent))))
      (let ((hs (instantiate::scrollbar
		   (parent frame)
		   (orientation "horizontal")))
	    (vs (instantiate::scrollbar
		   (parent frame)
		   (orientation "vertical")))
	    (l  (instantiate::listbox
		   (parent frame))))
	 
	 ;; Set internal true slots
	 (set! id (listbox-id l))
	 (set! eid (frame-id frame))
	 (set! h-scrollbar hs)
	 (set! v-scrollbar vs)
	 (set! listbox l)

	 ;; tk <-> biglook connection
	 (tk-command-object-set! eid widget)
	 
	 ;; Place internal widgets
	 (grid hs :row 0 :column 1 :sticky "we")
	 (grid 'remove hs)
	 (grid l  :row 1 :column 1 :sticky "news")
	 (grid vs :row 1 :column 2 :sticky "ns")
	 (grid 'rowconf    frame 1 :weight 1)
	 (grid 'columnconf frame 1 :weight 1)
	   
	 ;; Attach command to scrollbar and listbox
	 (tk l 'configure :xscroll
	     (lambda l (with-access::scrollbar hs (value) (set! value l))))
	 (tk l 'configure :yscroll
	     (lambda l (with-access::scrollbar vs (value) (set! value l))))
	 
	 (scrollbar-command-set! hs (lambda args (apply tk l 'xview args)))
	 (scrollbar-command-set! vs (lambda args (apply tk l 'yview args))))))

;*---------------------------------------------------------------------*/
;*    Example                                                          */
;*---------------------------------------------------------------------*/
;* (define l1 (instantiate::scrollbox                                  */
;* 	      (h-scroll-side "bottom")                                 */
;* 	      (relief "ridge")                                         */
;* 	      (border-width 2)                                         */
;* 	      (value '(1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17))))   */
;* (define l2 (instantiate::scrollbox                                  */
;* 	      (v-scroll-side #f)                                       */
;* 	      (h-scroll-side "top")                                    */
;* 	      (relief "ridge")                                         */
;* 	      (border-width 2)                                         */
;* 	      (value '("A long long phrase..."))))                     */
;* (pack l1 l2 :side 'left :expand #t :fill 'both)                     */
