;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    .../prgm/project/bglk/runtime/Lwidget/scrollcanvas.scm           */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Mon Dec 21 12:18:25 1998                          */
;*    Last change :  Fri May  7 16:12:30 1999 ()                       */
;*    -------------------------------------------------------------    */
;*    Scrollcanvas                                                     */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_scrollcanvas
   
   (include "Llib/biglook.sch")
   
   (option  (loadq "Llib/biglook.init"))
   
   (import  (__biglook_glue      "Llib/scmglue.scm")
	    (__biglook_misc      "Llib/misc.scm")
	    (__biglook_key  "Llib/key.scm")
	    (__biglook_tk-object "Llib/tk-object.scm")
	    (__biglook_frame     "Lwidget/frame.scm")
	    (__biglook_scrollbar "Lwidget/scrollbar.scm")
	    (__biglook_canvas    "Lwidget/canvas.scm")
	    (__biglook_canvitem  "Lwidget/canvitem.scm"))
   
   (export  (class scrollcanvas::canvas
	       (scrollcanvas-init)
	       ;; class
	       (class::bstring read-only (default "Scrollcanvas"))
	       ;; frame
	       (frame (default #unspecified))
	       ;; canvas
	       (canvas (default #unspecified))
	       ;; h-scrollbar
	       (h-scrollbar (default #unspecified))
	       ;; v-scrollbar
	       (v-scrollbar (default #unspecified))
	       ;; h-scroll-side
	       (h-scroll-side
		(get (lambda (o)
			(with-access::scrollcanvas o (h-scrollbar)
			   (h-scroll-side h-scrollbar))))
		(set (lambda (o v)
			(with-access::scrollcanvas o (h-scrollbar)
			   (h-scroll-side-set! h-scrollbar v)))))
	       ;; v-scroll-side
	       (v-scroll-side
		(get (lambda (o)
			(with-access::scrollcanvas o (v-scrollbar)
			   (v-scroll-side v-scrollbar))))
		(set (lambda (o v)
			(with-access::scrollcanvas o (v-scrollbar)
			   (v-scroll-side-set! v-scrollbar v)))))
	       ;; background
	       (background
		(get (lambda (o)
			(with-access::scrollcanvas o (frame)
			   (frame-background frame))))
		(set (lambda (o v)
			(with-access::scrollcanvas o (frame h-scrollbar v-scrollbar canvas)
			   (frame-background-set! frame v)
			   (canvas-background-set! canvas v)
			   (scrollbar-background-set! h-scrollbar v)
			   (scrollbar-background-set! v-scrollbar v)))))
	       ;; border-width
	       (border-width
		(get (lambda (o)
			(with-access::scrollcanvas o (frame)
			   (frame-border-width frame))))
		(set (lambda (o v)
			(with-access::scrollcanvas o (frame)
			   (frame-border-width-set! frame v)))))
	       ;; relief
	       (relief
		(get (lambda (o)
			(with-access::scrollcanvas o (frame)
			   (frame-relief frame))))
		(set (lambda (o v)
			(with-access::scrollcanvas o (frame)
			   (frame-relief-set! frame v))))))
	    
	    (scrollcanvas-init ::scrollcanvas)))

;*---------------------------------------------------------------------*/
;*    scrollcanvas-init ...                                            */
;*---------------------------------------------------------------------*/
(define (scrollcanvas-init widget::scrollcanvas)
   (with-access::scrollcanvas widget (id eid parent frame canvas
					h-scrollbar v-scrollbar)
      (set! frame (if (eq? parent #unspecified)
		      (instantiate::frame)
		      (instantiate::frame (parent parent))))
      (let ((hs (instantiate::scrollbar
		   (parent frame)
		   (orientation "horizontal")))
	    (vs (instantiate::scrollbar
		   (parent frame)
		   (orientation "vertical")))
	    (c  (instantiate::canvas
		   (parent frame))))
	   
	 ;; Set internal true slots
	 (set! id (canvas-id c))
	 (set! eid (frame-id frame))
	 (set! h-scrollbar hs)
	 (set! v-scrollbar vs)
	 (set! canvas c)

	 ;; tk <-> biglook connection
	 (tk-command-object-set! eid widget)
	 
	 ;; Place internal widgets
	 (grid hs :row 0 :column 1 :sticky "we")
	 (grid c  :row 1 :column 1 :sticky "news")
	 (grid vs :row 1 :column 2 :sticky "ns")
	 (grid 'rowconf    frame 1 :weight 1)
	 (grid 'columnconf frame 1 :weight 1)
	 
	 ;; Attach command to scrollbar and text
	 (with-access::canvas c (x-scroll-command y-scroll-command)
	    (set! x-scroll-command (lambda l
				      (with-access::scrollbar hs (value)
					 (set! value l))))
	    (set! y-scroll-command (lambda l
				      (with-access::scrollbar vs (value)
					 (set! value l)))))
	 
	 (scrollbar-command-set! hs (lambda args (apply tk c 'xview args)))
	 (scrollbar-command-set! vs (lambda args (apply tk c 'yview args))))))

;*---------------------------------------------------------------------*/
;*    Example                                                          */
;*---------------------------------------------------------------------*/
;* (define c (instantiate::scrollcanvas                                */
;* 	     (scroll-region '(-1000 -1000 1000 1000))))                */
;* (define r (instantiate::oval                                        */
;* 	     (tk-options '(0 0 300 300))                               */
;* 	     (fill "blue")                                             */
;* 	     (parent c)))                                              */
;* (pack c :expand #t :fill 'both)                                     */
