;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bglk/runtime/Lwidget/scrollframe.scm        */
;*    -------------------------------------------------------------    */
;*    Author      :  Erick Gallesio                                    */
;*    Creation    :  Wed Apr 28 18:50:58  1999                         */
;*    Last change :  Wed May 26 13:37:45 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    Scrollbox                                                        */
;*    -------------------------------------------------------------    */
;*    Source documentation:                                            */
;*       @path ../../manual/scrollframe.texi@                          */
;*       @node Scrollframe@                                            */
;*    Example:                                                         */
;*       @path ../../demos/scrollframe/scrollframe.scm@                */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_scrollframe
   
   (include "Llib/biglook.sch")
   
   (option  (loadq "Llib/biglook.init"))
   
   (import  (__biglook_glue      "Llib/scmglue.scm")
	    (__biglook_misc      "Llib/misc.scm")
	    (__biglook_key       "Llib/key.scm")
	    (__biglook_tk-object "Llib/tk-object.scm")
	    (__biglook_frame     "Lwidget/frame.scm")
	    (__biglook_canvas    "Lwidget/canvas.scm")
	    (__biglook_scrollbar "Lwidget/scrollbar.scm"))
   
   (export  (class scrollframe::frame
	       ;; initialization
	       (scrollframe-init)
	       ;; frame
	       (frame (default #unspecified))
	       ;; canvas
	       (canvas (default #unspecified))
	       ;; scrollbars
	       (h-scrollbar (default #unspecified))
	       (v-scrollbar (default #unspecified))
	       ;; internal frame
	       (iframe (default #unspecified))
	       ;; h-scroll-side
	       (h-scroll-side
		(get (lambda (o)
			(with-access::scrollframe o (h-scrollbar)
			   (h-scroll-side h-scrollbar))))
		(set (lambda (o v)
			(with-access::scrollframe o (h-scrollbar)
			   (h-scroll-side-set! h-scrollbar v)))))
	       ;; v-scroll-side
	       (v-scroll-side
		(get (lambda (o)
			(with-access::scrollframe o (v-scrollbar)
			   (v-scroll-side v-scrollbar))))
		(set (lambda (o v)
			(with-access::scrollframe o (v-scrollbar)
			   (v-scroll-side-set! v-scrollbar v)))))
	       ;; background
	       (background
		(get (lambda (o)
			(with-access::scrollframe o (frame)
			   (frame-background frame))))
		(set (lambda (o v)
			(with-access::scrollframe o (frame iframe h-scrollbar v-scrollbar canvas)
			   (frame-background-set! frame v)
			   (frame-background-set! iframe v)
			   (canvas-background-set! canvas v)
			   (scrollbar-background-set! h-scrollbar v)
			   (scrollbar-background-set! v-scrollbar v)))))
	       ;; trough-color
	       (trough-color
		(get (lambda (o)
			(with-access::scrollframe o (h-scrollbar v-scrollbar)
			   (tk h-scrollbar 'cget :troughcolor))))
		(set (lambda (o v)
			(with-access::scrollframe o (h-scrollbar v-scrollbar)
			   (tk h-scrollbar 'conf :troughcolor v)
			   (tk v-scrollbar 'conf :troughcolor v)))))
	       ;; scrollbar-width
	       (scrollbar-width
		(get (lambda (o)
			(with-access::scrollframe o (h-scrollbar v-scrollbar)
			   (tk h-scrollbar 'cget :width))))
		(set (lambda (o v)
			(with-access::scrollframe o (h-scrollbar v-scrollbar)
			   (tk h-scrollbar 'conf :width v)
			   (tk v-scrollbar 'conf :width v)))))
	       ;; border-width
	       (border-width
		(get (lambda (o)
			(with-access::scrollframe o (frame)
			   (frame-border-width frame))))
		(set (lambda (o v)
			(with-access::scrollframe o (frame)
			   (frame-border-width-set! frame v)))))
	       ;; relief
	       (relief
		(get (lambda (o)
			(with-access::scrollframe o (frame)
			   (frame-relief frame))))
		(set (lambda (o v)
			(with-access::scrollframe o (frame)
			   (frame-relief-set! frame v)))))
	       ;; height
	       (height
		(get (lambda (o)
			(with-access::scrollframe o (canvas)
			   (canvas-height canvas))))
		(set (lambda (o v)
			(with-access::scrollframe o (intf canvas)
			   (canvas-height-set! canvas v)))))
	       ;; width
	       (width
		(get (lambda (o)
			(with-access::scrollframe o (canvas)
			   (canvas-width canvas))))
		(set (lambda (o v)
			(with-access::scrollframe o (intf canvas)
			   (canvas-width-set! canvas v))))))
	    
	    (scrollframe-init ::scrollframe)))

;*---------------------------------------------------------------------*/
;*    scrollframe-init ...                                             */
;*---------------------------------------------------------------------*/
(define (scrollframe-init widget::scrollframe)
   (with-access::scrollframe widget (id eid parent frame iframe
					h-scrollbar v-scrollbar canvas)
      (set! frame (if (eq? parent #unspecified)
		      (instantiate::frame)
		      (instantiate::frame (parent parent))))
      (let ((hs   (instantiate::scrollbar
		       (parent frame)
		       (orientation "horizontal")))
	    (vs   (instantiate::scrollbar
		       (parent frame)
		       (orientation "vertical")))
	    (c    (instantiate::canvas
		       (parent frame)))
	    (intf (instantiate::frame
		       (parent frame))))
	 
	 ;; Set internal true slots
	 (set! id (frame-id intf))
	 (set! eid (frame-id frame))
	 (set! h-scrollbar hs)
	 (set! v-scrollbar vs)
	 (set! canvas c)
	 (set! iframe intf)

	 ;; tk <-> biglook connection
	 (tk-command-object-set! eid widget)
	 
	 ;; Place internal widgets
	 (grid hs :row 0 :column 1 :sticky "we")
	 (grid 'remove hs)
	 (grid intf  :row 1 :column 1 :sticky "news")
	 (grid vs :row 1 :column 2 :sticky "ns")
	 (grid c :row 1 :column 1 :sticky "nswe")
	 (grid 'rowconf    frame 1 :weight 1)
	 (grid 'columnconf frame 1 :weight 1)
	   
	 ;; Attach command to scrollbar and internal frame
	 (tk c 'configure :xscrollcommand
	     (lambda l (with-access::scrollbar hs (value) (set! value l))))
	 (tk c 'configure :yscrollcommand
	     (lambda l (with-access::scrollbar vs (value) (set! value l))))
	 
	 (scrollbar-command-set! hs (lambda args (apply tk c 'xview args)))
	 (scrollbar-command-set! vs (lambda args (apply tk c 'yview args)))

	 ;; Add a binding to frame such as adding widget to it will reconfigure 
	 ;; the canvas (and hence the scrollbar)
	 (tk c 'create 'window 0 0 :anchor 'nw :window intf :tag "window")
	 (canvas-scroll-region-set! c (list 0 0 3000 3000))
	 (add-binding! intf "<Configure>" ()
	    (canvas-scroll-region-set! c (list 0 0 
					       (winfo 'width intf) 
					       (winfo 'height intf))))
	 (add-binding! c "<Configure>" ()
	    (tk c 'itemconf "window" :width (winfo 'width c))))))
