;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    .../serrano/prgm/project/bglk/runtime/Lwidget/scrolltext.scm     */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Mon Dec 21 12:18:25 1998                          */
;*    Last change :  Fri May  7 16:18:19 1999 ()                       */
;*    -------------------------------------------------------------    */
;*    Scrolltext                                                       */
;*    -------------------------------------------------------------    */
;*    Source documentation:                                            */
;*       @path ../../manual/scrolltext.texi@                           */
;*       @node Scrolltext@                                             */
;*    Example:                                                         */
;*       @path ../../demos/scrolltext/scrolltext.scm@                  */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_scrolltext
   
   (include "Llib/biglook.sch")
   
   (option  (loadq "Llib/biglook.init"))
   
   (import  (__biglook_glue      "Llib/scmglue.scm")
	    (__biglook_misc      "Llib/misc.scm")
	    (__biglook_key       "Llib/key.scm")
	    (__biglook_tk-object "Llib/tk-object.scm")
	    (__biglook_frame     "Lwidget/frame.scm")
	    (__biglook_text      "Lwidget/text.scm")
	    (__biglook_scrollbar "Lwidget/scrollbar.scm"))
   
   (export  (class scrolltext::text
	       ;; initialization
	       (scrolltext-init)
	       ;; frame
	       (frame (default #unspecified))
	       ;; text
	       (text (default #unspecified))
	       ;; h-scrollbar
	       (h-scrollbar (default #unspecified))
	       ;; v-scrollbar
	       (v-scrollbar (default #unspecified))
	       ;; h-scroll-side
	       (h-scroll-side
		(get (lambda (o)
			(with-access::scrolltext o (h-scrollbar)
			   (h-scroll-side h-scrollbar))))
		(set (lambda (o v)
			(with-access::scrolltext o (h-scrollbar)
			   (h-scroll-side-set! h-scrollbar v)))))
	       ;; v-scroll-side
	       (v-scroll-side
		(get (lambda (o)
			(with-access::scrolltext o (v-scrollbar)
			   (v-scroll-side v-scrollbar))))
		(set (lambda (o v)
			(with-access::scrolltext o (v-scrollbar)
			   (v-scroll-side-set! v-scrollbar v)))))
	       ;; trough-color
	       (trough-color
		(get (lambda (o)
			(with-access::scrolltext o (h-scrollbar v-scrollbar)
			   (tk h-scrollbar 'cget :troughcolor))))
		(set (lambda (o v)
			(with-access::scrolltext o (h-scrollbar v-scrollbar)
			   (tk h-scrollbar 'conf :troughcolor v)
			   (tk v-scrollbar 'conf :troughcolor v)))))
	       ;; scrollbar-width
	       (scrollbar-width
		(get (lambda (o)
			(with-access::scrolltext o (h-scrollbar v-scrollbar)
			   (tk h-scrollbar 'cget :width))))
		(set (lambda (o v)
			(with-access::scrolltext o (h-scrollbar v-scrollbar)
			   (tk h-scrollbar 'conf :width v)
			   (tk v-scrollbar 'conf :width v)))))
	       ;; background
	       (background
		(get (lambda (o)
			(with-access::scrolltext o (frame)
			   (frame-background frame))))
		(set (lambda (o v)
			(with-access::scrolltext o (frame h-scrollbar v-scrollbar)
			   (frame-background-set! frame v)
			   (scrollbar-background-set! h-scrollbar v)
			   (scrollbar-background-set! v-scrollbar v)))))
	       ;; border-width
	       (border-width
		(get (lambda (o)
			(with-access::scrolltext o (frame)
			   (frame-border-width frame))))
		(set (lambda (o v)
			(with-access::scrolltext o (frame)
			   (frame-border-width-set! frame v)))))
	       (relief
		(get (lambda (o)
			(with-access::scrolltext o (frame)
			   (frame-relief frame))))
		(set (lambda (o v)
			(with-access::scrolltext o (frame)
			   (frame-relief-set! frame v))))))
	    
	    (scrolltext-init ::scrolltext)))

;*---------------------------------------------------------------------*/
;*    scrolltext-init ...                                              */
;*---------------------------------------------------------------------*/
(define (scrolltext-init widget::scrolltext)
   (with-access::scrolltext widget (id eid parent frame text
					h-scrollbar v-scrollbar)
      (set! frame (if (eq? parent #unspecified)
		      (instantiate::frame)
		      (instantiate::frame (parent parent))))
      (let ((hs (instantiate::scrollbar
		   (parent frame)
		   (orientation "horizontal")))
	    (vs (instantiate::scrollbar
		   (parent frame)
		   (orientation "vertical")))
	    (t  (instantiate::text
		   (parent frame))))
	   
	 ;; Set internal true slots
	 (set! id (text-id t))
	 (set! eid (frame-id frame))
	 (set! h-scrollbar hs)
	 (set! v-scrollbar vs)
	 (set! text t)

	 ;; tk <-> biglook connection
	 (tk-command-object-set! eid widget)
	 
	 ;; Place internal widgets
	 (grid hs :row 0 :column 1 :sticky "we")
	 (grid 'remove hs)
	 (grid t  :row 1 :column 1 :sticky "news")
	 (grid vs :row 1 :column 2 :sticky "ns")
	 (grid 'rowconf    frame 1 :weight 1)
	 (grid 'columnconf frame 1 :weight 1)
	 
	 ;; Attach command to scrollbar and text
	 (tk t 'configure :xscroll (lambda l
				      (with-access::scrollbar hs (value)
					 (set! value l))))
	 (tk t 'configure :yscroll (lambda l
				      (with-access::scrollbar vs (value)
					 (set! value l))))
	 
	 (scrollbar-command-set! hs (lambda args (apply tk t 'xview args)))
	 (scrollbar-command-set! vs (lambda args (apply tk t 'yview args))))))

;*---------------------------------------------------------------------*/
;*    Example                                                          */
;*---------------------------------------------------------------------*/
;* (define x (instantiate::scrolltext                                  */
;*    	     (width 30)                                                */
;*    	     (height 5)                                                */
;*    	     (wrap 'none)))                                            */
;* (pack x :expand #t :fill 'both)                                     */
;* (after 3000                                                         */
;*        (lambda () (with-access::scrolltext x (h-scroll-side)        */
;*    		     (set! h-scroll-side "top"))))                     */
