;*---------------------------------------------------------------------*/
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bglk/runtime/Lwidget/toolbar.scm            */
;*    -------------------------------------------------------------    */
;*    Author      :  Erick Gallesio                                    */
;*    Creation    :  Thu May 13 10:41:55 1999                          */
;*    Last change :  Sun Jun  6 14:20:01 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    Toolbar                                                          */
;*    -------------------------------------------------------------    */
;*    Source documentation:                                            */
;*       @path ../../manual/toolbar.texi@                              */
;*       @node Toolbar@                                                */
;*    Example:                                                         */
;*       @path ../../demos/toolbar/toolbar.scm@                        */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __biglook_toolbar
   
   (include "Llib/biglook.sch")
   
   (option  (loadq "Llib/biglook.init"))
   
   (import  (__biglook_glue        "Llib/scmglue.scm")
	    (__biglook_misc        "Llib/misc.scm")
	    (__biglook_tk-object   "Llib/tk-object.scm")
	    (__biglook_palette     "Llib/palette.scm")
	    (__biglook_entry       "Lwidget/entry.scm")
	    (__biglook_frame       "Lwidget/frame.scm")
	    (__biglook_menu        "Lwidget/menu.scm")
	    (__biglook_image       "Llib/image.scm")
	    (__biglook_button      "Lwidget/button.scm"))
   
   (export  (class toolbar::frame
	       ;; toolbar constructor
	       (toolbar-init)
	       
	       ;; internals
	       (%handle 	(default #f))
	       (%float  	(default #f))
	       (%clone		(default #f))
	       (%components     (default '()))
	       
	       ;; associated widgets
	       (handle::bool    (default #t) read-only)
	       (orientation     (default "horizontal"))
	       (release-command (default #f)))
	    
	    (toolbar-init                              ::toolbar)
	    
	    (fill-toolbar!                             ::toolbar l)
	    (toolbar-components-background             ::toolbar)
	    (toolbar-components-background-set!        ::toolbar v)
	    (toolbar-components-foreground             ::toolbar)
	    (toolbar-components-foreground-set!        ::toolbar v)
	    (toolbar-components-font                   ::toolbar)
	    (toolbar-components-font-set!              ::toolbar v)
	    (toolbar-components-active-foreground      ::toolbar)
	    (toolbar-components-active-foreground-set! ::toolbar v)
	    (toolbar-components-active-background      ::toolbar)
	    (toolbar-components-active-background-set! ::toolbar v)))

;*---------------------------------------------------------------------*/
;*    toolbar-init ...                                                 */
;*---------------------------------------------------------------------*/
(define (toolbar-init widget::toolbar)
   (with-access::toolbar widget (id eid parent %handle background)
      (let ((frame (if (eq? parent #unspecified)
		       (instantiate::frame
			  (class "ToolBar"))
		       (instantiate::frame
			  (parent parent)
			  (class "ToolBar")))))
	 (set! id (frame-id frame))
	 (set! eid id)
	 
	 ;; Create the handle to detach the tool bar
	 (set! %handle (instantiate::label 
		          (relief 	"raised")
			  (border-width 1)
			  (parent	widget)))
	 
	 (add-binding! %handle "<ButtonPress-1>" ()
		       (clone-toolbar widget)))))


;*---------------------------------------------------------------------*/
;*    Toolbar components constructors                                  */
;*---------------------------------------------------------------------*/
(define (add-button tb::toolbar img balloon action)
  (let* ((bg (toolbar-background tb))
	 (b  (instantiate::button 
	          (parent              tb)
		  (image               (if (image? img)
					   img
					   (instantiate::image (file img))))
		  (relief              "raised")
		  (command             action)
		  (highlight-thickness 0)
		  (background          bg)
		  (active-background   (make-lighter-color bg)))))
     (if (string? balloon)
	 (button-balloon-set! b balloon))
    ;; Change button bindings to add a relief when entering it
    (bindtags b (cons "ToolBar" (bindtags b)))
    (with-access::toolbar tb (%components)
	(set! %components (append %components 
				  (list (cons b (list img balloon action))))))
    b))

;*---------------------------------------------------------------------*/
;*    add-menubutton                                                   */
;*---------------------------------------------------------------------*/
(define (add-menubutton tb::toolbar title)
   (let* ((bg (toolbar-background tb))
	  (mb (instantiate::menu-button 
		 (parent tb)
		 (text title)
		 (background bg)
		 (active-background (make-lighter-color bg)))))
      (with-access::toolbar tb (%components)
	 (set! %components (append %components
				   (list (cons mb title)))))
      mb))

;*---------------------------------------------------------------------*/
;*    add-space                                                        */
;*---------------------------------------------------------------------*/
(define (add-space tb::toolbar space)
   (let ((f (instantiate::frame
	       (parent tb)
	       (background (frame-background tb)))))
      (with-access::toolbar tb (%components)
	 (set! %components (append %components (list (cons f space)))))
      f))

;*---------------------------------------------------------------------*/
;*    map-toolbar ...                                                  */
;*    -------------------------------------------------------------    */
;*    This procedure is in charge of packing the components of the     */
;*    toolbar. The arrangement depends of the orientation of the       */
;*    toolbar.                                                         */
;*---------------------------------------------------------------------*/
(define (map-toolbar tb::toolbar orientation handle?)
   (with-access::toolbar tb (%handle %components orientation background)
      (if handle? (label-background-set! %handle background))
      (let ((vertical? (equal? orientation "vertical")))
	 (if vertical?
	     ;; Arrangement is vertical
	     (begin
		(if handle?
		    (begin
		       (label-image-set! %handle
					 (instantiate::image
					    (file "hborder.xpm")))
		       (pack %handle :side 'top :fill 'x)))
		(for-each (lambda (x)
			     (let ((w    (car x))
				   (size (cdr x)))
				(if (and (integer? size) (< size 0))
				    (begin
				       (pack w :side 'top :fill 'both
					     :expand #t)
				       (if (eq? (object-class w) frame)
					   (frame-height-set! w (- size))))
				    (begin
				       (pack w :side 'top :fill 'x)
				       (if (eq? (object-class w) frame)
					   (frame-height-set! w (cdr x)))))))
			  %components))
	     ;; Arrangement is horizontal
	     (begin
		(if handle?
		    (begin
		       (label-image-set! %handle
					 (instantiate::image
					    (file "border.xpm")))
		       (pack %handle :side 'left :fill 'y)))
		(for-each (lambda (x)
			     (let ((w    (car x))
				   (size (cdr x)))
				(if (and (integer? size) (< size 0))
				    (begin
				       (pack w :side 'left :fill 'both
					     :expand #t)
				       (if (eq? (object-class w) frame)
					   (frame-width-set! w (- size))))
				    (begin
				       (pack w :side 'left :fill 'y)
				       (if (eq? (object-class w) frame)
					   (frame-width-set! w size))))))
			  %components))))))

;*---------------------------------------------------------------------*/
;*    clone-toolbar ...                                                */
;*---------------------------------------------------------------------*/
(define (clone-toolbar tb::toolbar)
   (warning "clone-toolbar" "not implemented yet" #unspecified))

;*---------------------------------------------------------------------*/
;*    fill-toolbar! ...                                                */
;*    -------------------------------------------------------------    */
;*    A simpler way to fill toolbars: In fact this is intended to be   */
;*    the only entry point for end-users.                              */
;*---------------------------------------------------------------------*/
(define (fill-toolbar! tb::toolbar l)
   ;; Predicates for testing the type of the items of a description
   (define (image? l) (and (list? l) (>= (length l) 3) (procedure? (caddr l))))
   (define space? integer?)
   (define menu?  list?)
   (define handle? #t)
   
   ;; make-submenu
   (define (make-submenu tb mb items)
      (let* ((bg (toolbar-background tb))
	     (m  (instantiate::menu
		    (parent mb)
		    (tear-off #f)
		    (background bg)
		    (active-background bg))))
	 (for-each (lambda (item)
		      (cond
			 ;; Tear-off
			 ((equal? (car item) "---")
			  (menu-tear-off-set! m #t))
			 ;; Separator
			 ((equal? (car item) "")
			  (menu-add! m 'separator))
			 ;; Normal Menu
			 ((and (= (length item) 2) (procedure? (cadr item)))
			  (menu-add! m
				     'command :label (car item)
				     :command (cadr item)))
			 ;; Normal Menu with acceleration
			 ((and (= (length item) 3)
			       (procedure? (cadr item))
			       (string? (caddr item)))
			  (menu-add! m
				     'command :label (car item)
				     :command (cadr item)
				     :accelerator (caddr item)))
			 ;; Cascade menu
			 ((and (>= (length item) 2) (list? (cadr item)))
			  (menu-add! m
				     'cascade :label (car item)
				     :menu (make-submenu tb m (cdr item))))
			 (else (apply menu-add! m item))))
		   items)
	 m))
   
   ;; make-item
   (define (make-item tb item)
      (cond 
	 ((image?  item) (apply add-button tb item))
	 ((space? item)  (add-space tb item))
	 ((menu? l)      (let ((mb (add-menubutton tb (car item))))
			    (make-submenu tb mb (cdr item))))
	 (else	         (error "fill-toobar!"
				"bad item specification"
				item))))

   (let ((handle? (or (null? l) (and (pair? l) (car l))))
	 (l       (cond
		     ((and (pair? l) (not (car l)))
		      (cdr l))
		     (else
		      l))))
      ;; Create the items
      (for-each (lambda (x) (make-item tb x)) l)       
      ;; map them and
      (map-toolbar tb (toolbar-orientation tb) handle?))
   ;; return the new toolbar
   tb)

;*---------------------------------------------------------------------*/
;*    toolbar-components-background ...                                */
;*---------------------------------------------------------------------*/
(define (toolbar-components-background tl::toolbar)
   (with-access::toolbar tl (%components)
      (let loop ((cs %components))
	 (if (null? cs)
	     #f
	     (match-case (car cs)
		(((and ?c (? button?)) . ?-) (button-background c))
		(((and ?f (? frame?)) . ?-) (frame-background f))
		(((and ?c (? menu-button?)) . ?-) (menu-button-background c))
		(else (loop (cdr cs))))))))
		
;*---------------------------------------------------------------------*/
;*    toolbar-components-background-set! ...                           */
;*---------------------------------------------------------------------*/
(define (toolbar-components-background-set! tl::toolbar v)
   (with-access::toolbar tl (%components)
      (let loop ((cs %components))
	 (if (null? cs)
	     #f
	     (begin
		(match-case (car cs)
		   (((and ?c (? button?)) . ?-)
		    (button-background-set! c v))
		   (((and ?f (? frame?)) . ?-)
		    (frame-background-set! f v))
		   (((and ?c (? menu-button?)) . ?-)
		    (menu-button-background-set! c v)))
		(loop (cdr cs)))))))
		
;*---------------------------------------------------------------------*/
;*    toolbar-components-foreground ...                                */
;*---------------------------------------------------------------------*/
(define (toolbar-components-foreground tl::toolbar)
   (with-access::toolbar tl (%components)
      (let loop ((cs %components))
	 (if (null? cs)
	     #f
	     (match-case (car cs)
		(((and ?c (? button?)) . ?-) (button-foreground c))
		(((and ?c (? menu-button?)) . ?-) (menu-button-foreground c))
		(else (loop (cdr cs))))))))
		
;*---------------------------------------------------------------------*/
;*    toolbar-components-foreground-set! ...                           */
;*---------------------------------------------------------------------*/
(define (toolbar-components-foreground-set! tl::toolbar v)
   (with-access::toolbar tl (%components)
      (let loop ((cs %components))
	 (if (null? cs)
	     #f
	     (begin
		(match-case (car cs)
		   (((and ?c (? button?)) . ?-)
		    (button-foreground-set! c v))
		   (((and ?c (? menu-button?)) . ?-)
		    (menu-button-foreground-set! c v)))
		(loop (cdr cs)))))))
		
;*---------------------------------------------------------------------*/
;*    toolbar-components-active-foreground ...                         */
;*---------------------------------------------------------------------*/
(define (toolbar-components-active-foreground tl::toolbar)
   (with-access::toolbar tl (%components)
      (let loop ((cs %components))
	 (if (null? cs)
	     #f
	     (match-case (car cs)
		(((and ?c (? button?)) . ?-)
		 (button-active-foreground c))
		(((and ?c (? menu-button?)) . ?-)
		 (menu-button-active-foreground c))
		(else
		 (loop (cdr cs))))))))
		
;*---------------------------------------------------------------------*/
;*    toolbar-components-active-foreground-set! ...                    */
;*---------------------------------------------------------------------*/
(define (toolbar-components-active-foreground-set! tl::toolbar v)
   (with-access::toolbar tl (%components)
      (let loop ((cs %components))
	 (if (null? cs)
	     #f
	     (begin
		(match-case (car cs)
		   (((and ?c (? button?)) . ?-)
		    (button-foreground-set! c v))
		   (((and ?c (? menu-button?)) . ?-)
		    (menu-button-foreground-set! c v)))
		(loop (cdr cs)))))))
		
;*---------------------------------------------------------------------*/
;*    toolbar-components-active-background ...                         */
;*---------------------------------------------------------------------*/
(define (toolbar-components-active-background tl::toolbar)
   (with-access::toolbar tl (%components)
      (let loop ((cs %components))
	 (if (null? cs)
	     #f
	     (match-case (car cs)
		(((and ?c (? button?)) . ?-)
		 (button-active-background c))
		(((and ?c (? menu-button?)) . ?-)
		 (menu-button-active-background c))
		(else
		 (loop (cdr cs))))))))
		
;*---------------------------------------------------------------------*/
;*    toolbar-components-active-background-set! ...                    */
;*---------------------------------------------------------------------*/
(define (toolbar-components-active-background-set! tl::toolbar v)
   (with-access::toolbar tl (%components)
      (let loop ((cs %components))
	 (if (null? cs)
	     #f
	     (begin
		(begin
		   (match-case (car cs)
		      (((and ?c (? button?)) . ?-)
		       (button-active-background-set! c v))
		      (((and ?c (? menu-button?)) . ?-)
		       (menu-button-active-background-set! c v)))
		   (loop (cdr cs))))))))

;*---------------------------------------------------------------------*/
;*    toolbar-components-font ...                                      */
;*---------------------------------------------------------------------*/
(define (toolbar-components-font tl::toolbar)
   (with-access::toolbar tl (%components)
      (let loop ((cs %components))
	 (if (null? cs)
	     #f
	     (match-case (car cs)
		(((and ?c (? button?)) . ?-)
		 (button-font c))
		(((and ?c (? menu-button?)) . ?-)
		 (menu-button-font c))
		(else
		 (loop (cdr cs))))))))
		
;*---------------------------------------------------------------------*/
;*    toolbar-components-font-set! ...                                 */
;*---------------------------------------------------------------------*/
(define (toolbar-components-font-set! tl::toolbar v)
   (with-access::toolbar tl (%components)
      (let loop ((cs %components))
	 (if (null? cs)
	     #f
	     (begin
		(match-case (car cs)
		   (((and ?c (? button?)) . ?-)
		    (button-font-set! c v))
		   (((and ?c (? menu-button?)) . ?-)
		    (menu-button-font-set! c v)))
		(loop (cdr cs)))))))

;*---------------------------------------------------------------------*/
;*    Default bindings                                                 */
;*---------------------------------------------------------------------*/
;* (add-binding! "ToolBar" "<Leave>" (WW) (WW 'configure :relief "raised")) */
;* (add-binding! "ToolBar" "<Enter>" (WW) (WW 'configure :relief "flat")) */

